'From Squeak3.4 of 1 March 2003 [latest update: #5170] on 8 November 2003 at 1:54:15 pm'!
"Change Set:		JonMajorShrink
Date:			October 8, 2002, November 2, 2003
Author:			Jon Hylands, Boris Gaertner

An update to majorShrink, for build 3.2-4956.

This change set, on a virgin 5170 image, will produce a ~1.5 MB MVC-only image.
(development tools, sources and class comments are kept. This is a requirement for
a school image.)

Make sure you back up your image before running this. After filing this change set in, enter an MVC project, and evaluate:

	Smalltalk preMajorShrink.
     Smalltalk majorShrink.

     Smalltalk obsoleteClasses size.

     Smalltalk condenseSources.

If you open the Transcript beforehand, you can get an idea of what is going on...
"!


!Object methodsFor: 'copying' stamp: 'BG 10/30/2003 18:57'!
veryDeepCopy
	"Do a complete tree copy using a dictionary.  An object in the tree twice is only copied once.  All references to the object in the copy of the tree will point to the new copy."

	| copier new refs newDep newModel |
	copier _ DeepCopier new initialize: self initialDeepCopierSize.
	new _ self veryDeepCopyWith: copier.
	
	copier references associationsDo: [:assoc | 
		assoc value veryDeepFixupWith: copier].
	"Fix dependents"
	refs _ copier references.
	DependentsFields associationsDo: [:pair |
		pair value do: [:dep | 
			(newDep _ refs at: dep ifAbsent: [nil]) ifNotNil: [
				newModel _ refs at: pair key ifAbsent: [pair key].
				newModel addDependent: newDep]]].
	^ new! !

!Object methodsFor: 'copying' stamp: 'BG 10/30/2003 18:57'!
veryDeepCopyWithSiblingOf: oldObject
	"Do a complete tree copy using a dictionary.  Substitute a clone of oldObject (a Player) for it.  Normally, a Player or non systemDefined object would have a new class.  We do not want one this time.  An object in the tree twice is only copied once.  All references to the object in the copy of the tree will point to the new copy."

	| copier new refs newDep newModel newObject |
	copier _ DeepCopier new initialize: self initialDeepCopierSize.
	newObject _ oldObject clone.		"Don't redefine clone for Players"
	copier references at: oldObject put: newObject.
	new _ self veryDeepCopyWith: copier.
	newObject veryDeepPvtSibling: copier.
	
	copier references associationsDo: [:assoc | 
		assoc value veryDeepFixupWith: copier].
	"Fix dependents"
	refs _ copier references.
	DependentsFields associationsDo: [:pair |
		pair value do: [:dep | 
			(newDep _ refs at: dep ifAbsent: [nil]) ifNotNil: [
				newModel _ refs at: pair key ifAbsent: [pair key].
				newModel addDependent: newDep]]].
	^ new! !

!Object methodsFor: 'macpal' stamp: 'BG 10/28/2003 21:01'!
currentVocabulary
	"Answer the currently-prevailing default vocabulary."

	^ Smalltalk isMorphic
		ifFalse:
			[Vocabulary fullVocabulary]! !

!Object methodsFor: 'macpal' stamp: 'BG 10/29/2003 08:43'!
currentWorld
	"Answer a morphic world that is the current UI focus.
		If in an embedded world, it's that world.
		If in a morphic project, it's that project's world.  
		If in an mvc project, it is the topmost morphic-mvc-window's worldMorph. 
		If in an mvc project that has no morphic-mvc-windows, then it's just some existing worldmorph instance.
		If in an mvc project in a Squeak that has NO WorldMorph instances, one is created.

	This method will never return nil, it will always return its best effort at returning a relevant world morph, but if need be -- if there are no worlds anywhere, it will create a new one."

	| aView aSubview |
	ActiveWorld ifNotNil:[^ActiveWorld].
	World ifNotNil:[^World].
	aView _ ScheduledControllers controllerSatisfying:
		[:ctrl | (aSubview _ ctrl view firstSubView) notNil and:
			[aSubview model isMorph and: [aSubview model isWorldMorph]]].
	^aView
		ifNotNil:
			[aSubview model]
		ifNil:
			[].! !

!Object methodsFor: 'private' stamp: 'BG 10/28/2003 21:01'!
primitiveError: aString 
	"This method is called when the error handling results in a recursion in 
	calling on error: or halt or halt:."

	| context |
	Sensor eventQueue: nil. "Or else we won't get keyboard and possibly run out of memory"
	(String
		streamContents: 
			[:s |
			s nextPutAll: '***System error handling failed***'.
			s cr; nextPutAll: aString.
			context _ thisContext sender sender.
			20 timesRepeat: [context == nil ifFalse: [s cr; print: (context _ context sender)]].
			s cr; nextPutAll: '-------------------------------'.
			s cr; nextPutAll: 'Type CR to enter an emergency evaluator.'.
			s cr; nextPutAll: 'Type any other character to restart.'])
		displayAt: 0 @ 0.
	[Sensor keyboardPressed] whileFalse.
	Sensor keyboard = Character cr ifTrue: [Transcripter emergencyEvaluator].
	Smalltalk isMorphic
		ifFalse: [ScheduledControllers searchForActiveController]! !


!ChangeSet methodsFor: 'accessing' stamp: 'BG 10/28/2003 20:37'!
editPostscript
	"edit the receiver's postscript, in a separate window.  "

	| deps found |
	self assurePostscriptExists.
	deps _ postscript dependents select:
		[:m | m isKindOf: StandardSystemView].
	deps size > 0 ifTrue:
		[Smalltalk isMorphic
			ifTrue:
				[found _ nil.
				found ifNotNil:  [^ found activate]]
			ifFalse:
				[found _ deps detect: [:obj | (obj isKindOf: StandardSystemView) and: [ScheduledControllers scheduledControllers includes: obj controller]] ifNone: [nil].
				found ifNotNil: [^ ScheduledControllers activateController: found controller]].
.
		self inform:
'Caution -- there', (deps size isOrAreStringWith: 'other window'), '
already open on this postscript elsewhere'].

	postscript openLabel: 'Postscript for ChangeSet named ', name! !


!ChangeSetCategory methodsFor: 'services' stamp: 'BG 10/28/2003 20:38'!
fillAggregateChangeSet
	"Create a change-set named Aggregate and pour into it all the changes in all the change-sets of the currently-selected category"

	| aggChangeSet |
	aggChangeSet _  ChangeSorter assuredChangeSetNamed: #Aggregate.
	aggChangeSet clear.
	aggChangeSet setPreambleToSay: '"Change Set:		Aggregate
Created at ', Time now printString, ' on ', Date today printString, ' by combining all the changes in all the change sets in the category ', categoryName printString, '"'.

	(self elementsInOrder copyWithout: aggChangeSet) do:
		[:aChangeSet  | aggChangeSet assimilateAllChangesFoundIn: aChangeSet].
! !


!ClassDescription methodsFor: 'initialize-release' stamp: 'BG 10/29/2003 09:09'!
updateInstancesFrom: oldClass
	"Recreate any existing instances of the argument, oldClass, as instances of 
	the receiver, which is a newly changed class. Permute variables as 
	necessary. Return the array of old instances (none of which should be
	pointed to legally by anyone but the array)."
	"ar 7/15/1999: The updating below is possibly dangerous. If there are any
	contexts having an old instance as receiver it might crash the system if
	the new receiver in which the context is executed has a different layout.
	See bottom below for a simple example:"
	| oldInstances |
	oldInstances _ oldClass allInstances asArray.
	oldInstances _ self updateInstances: oldInstances from: oldClass isMeta: self isMeta.
	"Now fix up instances in segments that are out on the disk."
	" ImageSegment allSubInstancesDo: [:seg |
		seg segUpdateInstancesOf: oldClass toBe: self isMeta: self isMeta]. "
	^oldInstances

"	| crashingBlock class |
	class _ Object subclass: #CrashTestDummy
		instanceVariableNames: 'instVar'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Crash-Test'.
	class compile:'instVar: value instVar _ value'.
	class compile:'crashingBlock ^[instVar]'.
	crashingBlock _ (class new) instVar: 42; crashingBlock.
	Object subclass: #CrashTestDummy
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Crash-Test'.
	crashingBlock.
	crashingBlock value.
	"
! !

!ClassDescription methodsFor: 'organization' stamp: 'BG 10/31/2003 19:11'!
compactOrganization
	"Remove the organization of this class by message categories.
	This is typically done to save space in small systems.  Classes and methods
	created or filed in subsequently will, nonetheless, be organized"

	organization updateFor: self.
	self isMeta ifFalse: [self class compactOrganization]! !


!AutoStart class methodsFor: 'class initialization' stamp: 'BG 10/30/2003 20:55'!
startUp
	| startupParameters launchers |
	startupParameters _ AbstractLauncher extractParameters.
	(startupParameters includesKey: 'apiSupported' asUppercase )
		ifTrue: [HTTPClient browserSupportsAPI: ((startupParameters at: 'apiSupported' asUppercase) asUppercase = 'TRUE')].
	self checkForUpdates
		ifTrue: [^self].
	
	launchers _ self installedLaunchers collect: [:launcher |
		launcher new].
	launchers do: [:launcher |
		launcher parameters: startupParameters].
	launchers do: [:launcher |
		Smalltalk at: #WorldState ifPresent: [ :ws | ws addDeferredUIMessage: [launcher startUp]]]! !


!ClassOrganizer methodsFor: 'updating' stamp: 'BG 10/31/2003 20:44'!
updateFor: aClass

  | stream selector changed |
 changed := false.
 stream := ReadStream on: elementArray copy.
 [stream atEnd]
   whileFalse:
     [selector := stream next.
      (aClass methodDictionary includesKey: selector)
         ifFalse: [self removeElement: selector.
                   changed := true].
     ].
  changed ifTrue: [self removeEmptyCategories.]! !


!ComplexProgressIndicator methodsFor: 'as yet unclassified' stamp: 'BG 10/28/2003 20:52'!
withProgressDo: aBlock

	| |

	Smalltalk isMorphic ifFalse: [^aBlock value].
! !


!CompositionScanner methodsFor: 'stop conditions' stamp: 'BG 10/29/2003 10:20'!
setFont
	super setFont.
	stopConditions == DefaultStopConditions 
		ifTrue:[stopConditions _ stopConditions copy].
	stopConditions at: Space asciiValue + 1 put: #space.
	wantsColumnBreaks == true ifTrue: [
		stopConditions at: "TextComposer characterForColumnBreak asciiValue" 1 + 1 put: #columnBreak.
	].
! !


!DataStream class methodsFor: 'as yet unclassified' stamp: 'BG 10/30/2003 19:00'!
initialize
	"TypeMap maps Smalltalk classes to type ID numbers which identify the data stream primitive formats.  nextPut: writes these IDs to the data stream.  NOTE: Changing these type ID numbers will invalidate all extant data stream files.  Adding new ones is OK.  
	Classes named here have special formats in the file.  If such a class has a subclass, it will use type 9 and write correctly.  It will just be slow.  (Later write the class name in the special format, then subclasses can use the type also.)
	 See nextPut:, next, typeIDFor:, & ReferenceStream>>isAReferenceType:"
	"DataStream initialize"

	| refTypes t |
	refTypes _ OrderedCollection new.
	t _ TypeMap _ Dictionary new: 80. "sparse for fast hashing"

	t at: UndefinedObject put: 1.   refTypes add: 0.
	t at: True put: 2.   refTypes add: 0.
	t at: False put: 3.   refTypes add: 0.
	t at: SmallInteger put: 4.	 refTypes add: 0.
	t at: String put: 5.   refTypes add: 1.
	t at: Symbol put: 6.   refTypes add: 1.
	t at: ByteArray put: 7.   refTypes add: 1.
	t at: Array put: 8.   refTypes add: 1.
	"(type ID 9 is for arbitrary instances of any class, cf. typeIDFor:)"
		refTypes add: 1.
	"(type ID 10 is for references, cf. ReferenceStream>>tryToPutReference:)"
		refTypes add: 0.
	t at: Bitmap put: 11.   refTypes add: 1.
	t at: Metaclass put: 12.   refTypes add: 0.
	"Type ID 13 is used for HyperSqueak User classes that must be reconstructed."
		refTypes add: 1.
	t at: Float put: 14.  refTypes add: 1.
	t at: Rectangle put: 15.  refTypes add: 1.	"Allow compact Rects."
	"type ID 16 is an instance with short header.  See beginInstance:size:"
		refTypes add: 1.
	t at: String put: 17.   refTypes add: 1.	"new String format, 1 or 4 bytes of length"
	t at: WordArray put: 18.  refTypes add: 1.	"bitmap-like"
	t at: WordArrayForSegment put: 19.  refTypes add: 1.		"bitmap-like"
"	t at: SoundBuffer put: 20.  refTypes add: 1. "	"And all other word arrays, both 
		16-bit and 32-bit.  See methods in ArrayedCollection.  Overridden in SoundBuffer."
	t at: CompiledMethod put: 21.  refTypes add: 1.	"special creation method"
	"t at:  put: 22.  refTypes add: 0."
	ReferenceStream refTypes: refTypes.		"save it"

	"For all classes that are like WordArrays, store them the way ColorArray is stored.  As bits, and able to change endianness."
	Smalltalk do: [:cls |
		cls isInMemory ifTrue: [
			cls isBehavior ifTrue: [
				cls isPointers not & cls isVariable & cls isWords ifTrue: [
					(t includesKey: cls) ifFalse: [t at: cls put: 20]]]]].! !


!DigitalSignatureAlgorithm class methodsFor: 'public' stamp: 'BG 10/30/2003 20:34'!
generateKeySet
	"Generate and answer a key set for code signing. The result is a pair (<private key><public key>). Each key is an array of four large integers. The signer must be sure to record this keys set and must keep the private key secret to prevent someone from forging their signature."
	"Note: Key generation can take some time. Open a transcript so you can see what's happening and take a coffee break!!"
	"Note: Unguessable random numbers are needed for key generation. The user will be prompted to type a really long random string (two or three lines) to initialize the random number generator before generating a key set. A different random string should be typed for every session; it is not a password and we wish to produce different random number streams."
	"DigitalSignatureAlgorithm generateKeySet"

	| dsa |
	dsa _ DigitalSignatureAlgorithm new.
	"(self confirm: 'Shall I seed the random generator from the current sound input?')"
        false
		ifTrue: ["dsa initRandom: (dsa randomBitsFromSoundInput: 512)"]
		ifFalse: [dsa initRandomFromUser].
	^ dsa generateKeySet
! !


!DiskProxy methodsFor: 'exceptions' stamp: 'BG 10/30/2003 20:37'!
enter: returningFlag revert: revertFlag saveForRevert: saveForRevert
	"Look for our project on the server, then try to enter it!!  DiskProxy is acting as a stub for the real thing.  Called from a ProjectViewMorph in the current project.  If have url, use it.  Else look in current Project's server and folder."

	constructorSelector == #namedUrl: ifTrue: ["Project namedUrl: xxx"
		^ ((Smalltalk at: globalObjectName) perform: #fromUrl:
					withArguments: constructorArgs) ].
	constructorSelector == #named: ifTrue: [].	"name"
! !


!DisplayText methodsFor: 'private' stamp: 'BG 10/28/2003 20:54'!
composeForm
	
	Smalltalk isMorphic
		ifTrue:
			[]
		ifFalse: [form _ self asParagraph asForm]! !


!DocLibrary class methodsFor: 'as yet unclassified' stamp: 'BG 11/1/2003 15:32'!
resetServers

  DropBox := nil! !


!EventSensor methodsFor: 'initialize' stamp: 'BG 10/28/2003 20:54'!
startUp
	"Run the I/O process"
	self shutDown.
	self initialize.
	self primSetInputSemaphore: (Smalltalk registerExternalObject: inputSemaphore).
	inputProcess _ [self ioProcess] forkAt: Processor lowIOPriority.
	super startUp.! !


!ExternalDropHandler class methodsFor: 'private' stamp: 'BG 10/29/2003 09:34'!
defaultImageHandler
	| image |
	^ExternalDropHandler
		type: 'image/'
		extension: nil
		action: [:stream :pasteUp :event |
			stream binary.
			image _ Form fromBinaryStream: ((RWBinaryOrTextStream with: stream contents) reset).] fixTemps! !


!FileStream class methodsFor: 'browser requests' stamp: 'BG 10/30/2003 21:42'!
httpPostDocument: url args: argsDict
	| argString |
	argString _ argsDict
		ifNotNil: ["argString _ HTTPSocket argString: argsDict"]
		ifNil: [''].
	^self post: argString url: url , argString ifError: [self halt]! !


!Flaps class methodsFor: 'predefined flaps' stamp: 'BG 10/31/2003 13:48'!
dropStandardFlaps
	"Initialize the standard default out-of-box set of global flaps. This method creates them and places them in my class variable #SharedFlapTabs, but does not itself get them displayed."

	SharedFlapTabs _ nil.! !


!Form methodsFor: 'displaying' stamp: 'BG 10/28/2003 20:58'!
setAsBackground
	"Set this form as a background image."

	Smalltalk isMorphic 
		
		ifFalse:
			[ScheduledControllers screenController model form: self.
			Display restoreAfter: []]! !


!DisplayScreen methodsFor: 'other' stamp: 'BG 10/28/2003 20:53'!
restore
	Smalltalk isMorphic
		ifTrue: []
		ifFalse: [ScheduledControllers unCacheWindows; restore].! !

!DisplayScreen methodsFor: 'other' stamp: 'BG 10/28/2003 20:53'!
restoreAfter: aBlock
	"Evaluate the block, wait for a mouse click, and then restore the screen."

	aBlock value.
	Sensor waitButton.
	Smalltalk isMorphic
		ifTrue: []
		ifFalse: [(ScheduledControllers restore; activeController) view emphasize]! !

!DisplayScreen methodsFor: 'blitter defaults' stamp: 'BG 10/29/2003 08:33'!
defaultCanvasClass
	"Return the WarpBlt version to use when I am active"
	^BitBlt! !


!Form class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:58'!
fromFileNamed: fileName
	"Read a Form or ColorForm from the given file."

	| file form |
	file _ (FileStream readOnlyFileNamed: fileName) binary.
	form _ self fromBinaryStream: file.
	
	file close.
	^ form
! !

!Form class methodsFor: 'fileIn/Out' stamp: 'BG 10/28/2003 20:58'!
openImageInWindow: fullName
	"Handle five file formats: GIF, JPG, PNG, Form stoteOn: (run coded), and BMP.
	Fail if file format is not recognized."

	| image myStream |

	myStream _ (FileStream readOnlyFileNamed: fullName) binary.
	image _ self fromBinaryStream: myStream.
	myStream close.

	
	Smalltalk isMorphic
		ifFalse: [FormView open: image named: fullName]! !


!DisplayScreen class methodsFor: 'display box access' stamp: 'BG 10/28/2003 20:54'!
checkForNewScreenSize
	"Check whether the screen size has changed and if so take appropriate actions"

	Display extent = DisplayScreen actualScreenSize ifTrue: [^ self].
	DisplayScreen startUp.
	Smalltalk isMorphic
		ifTrue:
			[]
		ifFalse:
			[ScheduledControllers restore; searchForActiveController]! !


!HTTPClient class methodsFor: 'post/get' stamp: 'BG 10/30/2003 21:35'!
httpGetDocument: url
	| stream content | 
	^self shouldUsePluginAPI
		ifTrue: [
			stream _ FileStream requestURLStream: url ifError: [self error: 'Error in get from ' , url printString].
			stream ifNil: [^''].
			stream position: 0.
			content _ stream upToEnd.
			stream close.
			"MIMEDocument content: content"]
		ifFalse: ["HTTPSocket httpGetDocument: url"]! !

!HTTPClient class methodsFor: 'post/get' stamp: 'BG 10/30/2003 21:36'!
httpPostDocument: url target: target args: argsDict
	| argString stream content |
	^self shouldUsePluginAPI
		ifTrue: [
			argString _ argsDict
				ifNotNil: ["argString _ HTTPSocket argString: argsDict"]
				ifNil: [''].
			stream _ FileStream post: argString , ' ' target: target url: url , argString ifError: [self error: 'Error in post to ' , url printString].
			stream position: 0.
			content _ stream upToEnd.
			stream close.
			"MIMEDocument content: content"]
		ifFalse: ["HTTPSocket httpPostDocument: url  args: argsDict"]! !


!ImageSegment methodsFor: 'testing' stamp: 'BG 10/30/2003 18:58'!
findRogueRootsAllMorphs: rootArray
	"This is a tool to track down unwanted pointers into the segment.  If we don't deal with these pointers, the segment turns out much smaller than it should.  These pointers keep a subtree of objects out of the segment.
1) assemble all objects should be in seg:  morph tree, presenter, scripts, player classes, metaclasses.  Put in a Set.
2) Remove the roots from this list.  Ask for senders of each.  Of the senders, forget the ones that are in the segment already.  Keep others.  The list is now all the 'incorrect' pointers into the segment."

| inSeg testRoots pointIn wld xRoots |
Smalltalk garbageCollect.
inSeg _ IdentitySet new: 200.
arrayOfRoots _ rootArray.
(testRoots _ self rootsIncludingPlayers) ifNil: [testRoots _ rootArray].
testRoots do: [:obj |
	(obj isKindOf: Project) ifTrue: [inSeg add: obj.
			wld _ obj world.
			inSeg add: wld presenter].
	(obj isKindOf: Presenter) ifTrue: [inSeg add: obj].
	].
xRoots _ wld ifNil: [testRoots] ifNotNil: [testRoots, (Array with: wld)].
xRoots do: [:obj |
	"root is a project"
	obj isMorph ifTrue: [
		obj allMorphs do: [:mm | inSeg add: mm.
			mm player ifNotNil: [inSeg add: mm player]].
		obj isWorldMorph ifTrue: [inSeg add: obj presenter]]].
inSeg do: [:obj |
	].
testRoots do: [:each | inSeg remove: each ifAbsent: []].
	"want them to be pointed at from outside"
pointIn _ IdentitySet new: 400.
inSeg do: [:ob |
	pointIn addAll: (Smalltalk pointersTo: ob except: inSeg)].
testRoots do: [:each | pointIn remove: each ifAbsent: []].
pointIn remove: inSeg array ifAbsent: [].
pointIn remove: pointIn array ifAbsent: [].
inSeg do: [:obj |
	(obj isKindOf: Morph) ifTrue: [
		pointIn remove: (obj "submorphs" instVarAt: 3) ifAbsent: [].
		"associations in extension"
		pointIn remove: obj extension ifAbsent: [].
		obj extension ifNotNil: [obj extension otherProperties ifNotNil: [
			obj extension otherProperties associationsDo: [:ass | 
				pointIn remove: ass ifAbsent: []]]
			"*** and extension actorState"
			"*** and ActorState instantiatedUserScriptsDictionary ScriptInstantiations"]].
	(obj isKindOf: Player) ifTrue: [obj class scripts values do: [:us | 
			pointIn remove: us ifAbsent: []]]].
"*** presenter playerlist"
self halt: 'Examine local variables pointIn and inSeg'.
^ pointIn! !

!ImageSegment methodsFor: 'fileIn/Out' stamp: 'BG 10/30/2003 20:02'!
comeFullyUpOnReload: smartRefStream
	"fix up the objects in the segment that changed size.  An object in the segment is the wrong size for the modern version of the class.  Construct a fake class that is the old size.  Replace the modern class with the old one in outPointers.  Load the segment.  Traverse the instances, making new instances by copying fields, and running conversion messages.  Keep the new instances.  Bulk forward become the old to the new.  Let go of the fake objects and classes.
	After the install (below), arrayOfRoots is filled in.  Globalize new classes.  Caller may want to do some special install on certain objects in arrayOfRoots. 
	May want to write the segment out to disk in its new form."

	| mapFakeClassesToReal ccFixups receiverClasses rootsToUnhiberhate myProject |

	self flag: #bobconv.	

	RecentlyRenamedClasses _ nil.		"in case old data hanging around"
	mapFakeClassesToReal _ smartRefStream reshapedClassesIn: outPointers.
		"Dictionary of just the ones that change shape.  Substitute them in outPointers."
	ccFixups _ self remapCompactClasses: mapFakeClassesToReal 
				refStrm: smartRefStream.
	ccFixups ifFalse: [^ self error: 'A class in the file is not compatible'].
	endMarker _ segment nextObject. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
	arrayOfRoots _ self loadSegmentFrom: segment outPointers: outPointers.
		"Can't use install.  Not ready for rehashSets"
	mapFakeClassesToReal isEmpty ifFalse: [
		self reshapeClasses: mapFakeClassesToReal refStream: smartRefStream
	].
	receiverClasses _ self restoreEndianness.		"rehash sets"
	smartRefStream checkFatalReshape: receiverClasses.

	"Classes in this segment."
	arrayOfRoots do: [:importedObject | 
		importedObject class class == Metaclass ifTrue: [self declare: importedObject]].
	arrayOfRoots do: [:importedObject | 
		(importedObject isKindOf: CompiledMethod) ifTrue: [
			importedObject sourcePointer > 0 ifTrue: [importedObject zapSourcePointer]].
		(importedObject isKindOf: Project) ifTrue: [
			myProject _ importedObject.
			importedObject ensureChangeSetNameUnique.
			Project addingProject: importedObject.
			importedObject restoreReferences.
			self dependentsRestore: importedObject.]].
	
	rootsToUnhiberhate _ arrayOfRoots select: [:importedObject | 
		importedObject respondsTo: #unhibernate	"ScriptEditors and ViewerFlapTabs"
	].
	myProject ifNotNil: [
		myProject world setProperty: #thingsToUnhibernate toValue: rootsToUnhiberhate
	].

	mapFakeClassesToReal isEmpty ifFalse: [
		mapFakeClassesToReal keys do: [:aFake | 
			aFake indexIfCompact > 0 ifTrue: [aFake becomeUncompact].
			aFake removeFromSystemUnlogged].
		SystemOrganization removeEmptyCategories].
	"^ self"! !

!ImageSegment methodsFor: 'fileIn/Out' stamp: 'BG 10/30/2003 19:18'!
prepareToBeSaved
	"Prepare objects in outPointers to be written on the disk.  They must be able to match up with existing objects in their new system.  outPointers is already a copy.
	Classes are already converted to a DiskProxy.  
	Associations in outPointers:
1) in Smalltalk.
2) in a classPool.
3) in a shared pool.
4) A pool dict pointed at directly"

| left pool myClasses outIndexes key |
myClasses _ Set new.
arrayOfRoots do: [:aRoot | aRoot class class == Metaclass ifTrue: [myClasses add: aRoot]].
outIndexes _ IdentityDictionary new.
outPointers withIndexDo: [:anOut :ind | 
	anOut isVariableBinding ifTrue: [
		(myClasses includes: anOut value)
			ifFalse: [outIndexes at: anOut put: ind]
			ifTrue: [(Smalltalk associationAt: anOut key ifAbsent: [3]) == anOut 
				ifTrue: [outPointers at: ind put: 
					(DiskProxy global: #Smalltalk selector: #associationDeclareAt: 
						args: (Array with: anOut key))]
				ifFalse: [outIndexes at: anOut put: ind]
				]].
	(anOut isKindOf: Dictionary) ifTrue: ["Pools pointed at directly"
		(key _ Smalltalk keyAtIdentityValue: anOut ifAbsent: [nil]) ifNotNil: [
			outPointers at: ind put: 
				(DiskProxy global: key selector: #yourself args: #())]].

	].
left _ outIndexes keys asSet.
left size > 0 ifTrue: ["Globals"
	(left copy) do: [:assoc |	"stay stable while delete items"
		(Smalltalk associationAt: assoc key ifAbsent: [3]) == assoc ifTrue: [
			outPointers at: (outIndexes at: assoc) put: 
				(DiskProxy global: #Smalltalk selector: #associationAt: 
					args: (Array with: assoc key)).
			left remove: assoc]]].
left size > 0 ifTrue: ["Class variables"
	Smalltalk allClassesDo: [:cls | cls classPool size > 0 ifTrue: [
		(left copy) do: [:assoc |	"stay stable while delete items"
			(cls classPool associationAt: assoc key ifAbsent: [3]) == assoc ifTrue: [
				outPointers at: (outIndexes at: assoc) put: 
					(DiskProxy new global: cls name
						preSelector: #classPool
						selector: #associationAt: 
						args: (Array with: assoc key)).
				left remove: assoc]]]]].
left size > 0 ifTrue: ["Pool variables"
	Smalltalk associationsDo: [:poolAssoc |
		poolAssoc value class == Dictionary ifTrue: ["a pool"
			pool _ poolAssoc value.
			(left copy) do: [:assoc |	"stay stable while delete items"
				(pool associationAt: assoc key ifAbsent: [3]) == assoc ifTrue: [
					outPointers at: (outIndexes at: assoc) put: 
						(DiskProxy global: poolAssoc key selector: #associationAt: 
							args: (Array with: assoc key)).
					left remove: assoc]]]]].
left size > 0 ifTrue: [
	"If points to class in arrayOfRoots, must deal with it separately"
	"OK to have obsolete associations that just get moved to the new system"
	self inform: 'extra associations'.
	left inspect].
! !


!MethodFinder class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:52'!
methodFor: dataAndAnswers
	"Return a Squeak expression that computes these answers.  (This method is called by the comment in the bottom pane of a MethodFinder.  Do not delete this method.)"

	| resultOC resultString |
	resultOC _ (self new) load: dataAndAnswers; findMessage.
	resultString _ String streamContents: [:strm |
		resultOC do: [:exp | strm nextPut: $(; nextPutAll: exp; nextPut: $); space]].
	
	^ resultString! !


!Model methodsFor: 'dependents' stamp: 'BG 10/29/2003 08:37'!
containingWindow
	"Answer the window that holds the receiver.  The dependents technique is odious and may not be airtight, if multiple windows have the same model."

	^ self dependents detect:
		[:d | (d isKindOf: StandardSystemView) and: [d model == self]] ifNone: [nil]! !

!Model methodsFor: 'dependents' stamp: 'BG 10/28/2003 21:00'!
topView
	"Find the first top view on me. Is there any danger of their being two with the same model?  Any danger from ungarbage collected old views?  Ask if schedulled?"

	dependents ifNil: [^ nil].
	
	dependents do: [:v | v superView ifNil: [v model == self ifTrue: [^ v]]].
	^ nil
! !

!Model methodsFor: 'keyboard' stamp: 'BG 10/29/2003 08:54'!
arrowKey: aChar from: view
	"Process the up and down arrows in a list pane.  Note that the listView tells us what index variable, how to get the list, and how to move the index.  Derived from a Martin Pammer submission, 02/98"

     | keyEvent oldSelection nextSelection max min howMany |

	(#(1 4 11 12 30 31) includes: (keyEvent _ aChar asciiValue)) ifFalse:
		[
			^ self].

     oldSelection := view getCurrentSelectionIndex.
     nextSelection := oldSelection.
     max := view maximumSelection.
     min := view minimumSelection.
     howMany := view numSelectionsInView.	"get this exactly??"

     keyEvent == 31 ifTrue:
		["down-arrow; move down one, wrapping to top if needed"
		nextSelection := oldSelection + 1.
		nextSelection > max ifTrue: [nextSelection _ 1]].

     keyEvent == 30 ifTrue:
		["up arrow; move up one, wrapping to bottom if needed"
		nextSelection := oldSelection - 1.
		nextSelection < 1 ifTrue: [nextSelection _ max]].

     keyEvent == 1  ifTrue: [nextSelection := 1].  "home"
     keyEvent == 4  ifTrue: [nextSelection := max].   "end"
     keyEvent == 11 ifTrue: [nextSelection := min max: (oldSelection - howMany)].  "page up"
     keyEvent == 12  ifTrue: [nextSelection := (oldSelection + howMany) min: max].  "page down"
     nextSelection = oldSelection  ifFalse:
		[self okToChange
			ifTrue:
				[view changeModelSelection: nextSelection.
				"view controller moveMarker"]]
			! !


!CPUWatcher methodsFor: 'porcine capture' stamp: 'BG 10/29/2003 01:06'!
openWindowForSuspendedProcess: aProcess

	Smalltalk isMorphic
		ifTrue: [ ]
		ifFalse: [ [ self openMVCWindowForSuspendedProcess: aProcess ] forkAt: Processor userSchedulingPriority ]
! !


!DualChangeSorter methodsFor: 'initialization' stamp: 'BG 10/28/2003 20:54'!
open
	| topView |
	
	leftCngSorter _ ChangeSorter new myChangeSet: Smalltalk changes.
	leftCngSorter parent: self.
	rightCngSorter _ ChangeSorter new myChangeSet: 
			ChangeSorter secondaryChangeSet.
	rightCngSorter parent: self.

	topView _ (StandardSystemView new) model: self; borderWidth: 1.
	topView label: leftCngSorter label.
	topView minimumSize: 300 @ 200.
	leftCngSorter openView: topView offsetBy: 0@0.
	rightCngSorter openView: topView offsetBy: 360@0.
	topView controller open.
! !


!ParagraphEditor methodsFor: 'menu messages' stamp: 'BG 11/1/2003 14:27'!
saveContentsInFile
	"Save the receiver's contents string to a file, prompting the user for a file-name.  Suggest a reasonable file-name."

	| fileName stringToSave parentWindow labelToUse suggestedName lastIndex |
	stringToSave _ paragraph text string.
	stringToSave size == 0 ifTrue: [^ self inform: 'nothing to save.'].
	parentWindow _ self model dependents
						detect: [:dep | dep isKindOf: StandardSystemView]
						ifNone: [nil].
	labelToUse _ parentWindow
		ifNil: 		['Untitled']
		ifNotNil: 	[parentWindow label].
	suggestedName _ nil.
	#(('Decompressed contents of: '		'.gz')) do:  "can add more here..."
		[:leaderTrailer |
			(labelToUse beginsWith: leaderTrailer first) ifTrue:
				[suggestedName _ labelToUse copyFrom: leaderTrailer first size + 1 to: labelToUse size.
				(labelToUse endsWith: leaderTrailer last)
					ifTrue:
						[suggestedName _ suggestedName copyFrom: 1 to: suggestedName size - leaderTrailer last size]
					ifFalse:
						[lastIndex _ suggestedName lastIndexOf: $. ifAbsent: [0].
						(lastIndex = 0 or: [lastIndex = 1]) ifFalse:
							[suggestedName _ suggestedName copyFrom: 1 to: lastIndex - 1]]]].

	suggestedName ifNil:
		[suggestedName _ labelToUse, '.text'].
			
	fileName _ FillInTheBlank request: 'File name?'
			initialAnswer: suggestedName.
	fileName isEmptyOrNil ifFalse:
		[(FileStream newFileNamed: fileName) nextPutAll: stringToSave; close]! !

!ParagraphEditor methodsFor: 'editing keys' stamp: 'BG 10/29/2003 08:00'!
changeEmphasis: characterStream 
	"Change the emphasis of the current selection or prepare to accept characters with the change in emphasis. Emphasis change amounts to a font change.  Keeps typeahead."
	| keyCode attribute oldAttributes index thisSel colors extras indexOfOldAttributes |		 "control 0..9 -> 0..9"
	keyCode _ ('0123456789-=' indexOf: sensor keyboard ifAbsent: [1]) - 1.

	"grab the old set of attributes"
	indexOfOldAttributes _
		startBlock stringIndex < stopBlock stringIndex
			ifTrue: [ startBlock stringIndex ]
			ifFalse: [
				"if selection is empty, look on character to the left; this is consistent with setEmphasisHere"
				(startBlock stringIndex - 1) max: 1 ].
	oldAttributes _ paragraph text attributesAt: indexOfOldAttributes forStyle: paragraph textStyle.

	thisSel _ self selection.

	"Decipher keyCodes for Command 0-9..."
	(keyCode between: 1 and: 5) ifTrue:
		[attribute _ TextFontChange fontNumber: keyCode].
	keyCode = 6 ifTrue:
		[colors _ #(black magenta red yellow green blue cyan white).
		extras _ false "not a system window"
				ifTrue: [#()]
				ifFalse: [#('Link to comment of class' 'Link to definition of class' 
						'Link to hierarchy of class' 'Link to method')].
		index _ (PopUpMenu labelArray: colors , #('choose color...' 'Do it' 'Print it'), 
			extras, #('be a web URL link' 
			'Edit hidden info' 'Copy hidden info')
							lines: (Array with: colors size +1)) startUp.
		index = 0 ifTrue: [^ true].
		index <= colors size
		ifTrue:
			[attribute _ TextColor color: (Color perform: (colors at: index))]
		ifFalse:
			[index _ index - colors size - 1.	"Re-number!!!!!!"
			index = 0 ifTrue: [attribute _ self chooseColor].
			index = 1 ifTrue: [attribute _ TextDoIt new.
				thisSel _ attribute analyze: self selection asString].
			index = 2 ifTrue: [attribute _ TextPrintIt new.
				thisSel _ attribute analyze: self selection asString].
			(extras size = 0) & (index > 2) ifTrue: [index _ index + 5].	"skip those"
			index = 3 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Comment'].
			index = 4 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Definition'].
			index = 5 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString with: 'Hierarchy'].
			index = 6 ifTrue: [attribute _ TextLink new. 
				thisSel _ attribute analyze: self selection asString].
		"	index = 7 ifTrue: [attribute _ TextURL new. 
				thisSel _ attribute analyze: self selection asString]. "
			index = 8 ifTrue: ["Edit hidden info"
				thisSel _ self hiddenInfo.	"includes selection"
				attribute _ TextEmphasis normal].
			index = 9 ifTrue: ["Copy hidden info"
				self copyHiddenInfo.  ^ true].	"no other action"
		thisSel ifNil: [^ true]].	"Could not figure out what to link to"
		].
	(keyCode between: 7 and: 11) ifTrue:
		[sensor leftShiftDown
		ifTrue:
			[keyCode = 10 ifTrue: [attribute _ TextKern kern: -1].
			keyCode = 11 ifTrue: [attribute _ TextKern kern: 1]]
		ifFalse:
			[attribute _ TextEmphasis perform:
					(#(bold italic narrow underlined struckOut) at: keyCode - 6).
			oldAttributes do:
				[:att | (att dominates: attribute) ifTrue: [attribute turnOff]]]].
	(keyCode = 0) ifTrue:
		[attribute _ TextEmphasis normal].

	beginTypeInBlock ~~ nil
		ifTrue:  "only change emphasisHere while typing"
			[self insertTypeAhead: characterStream]
		ifFalse:
			[self replaceSelectionWith: (thisSel asText addAttribute: attribute)].

	emphasisHere _ 	Text addAttribute: attribute toArray: oldAttributes.
	^ true! !

!ParagraphEditor methodsFor: 'nonediting/nontyping keys' stamp: 'BG 10/28/2003 21:01'!
escapeToDesktop: characterStream 
	"Pop up a morph to field keyboard input in the context of the desktop"

	
	^ true! !


!ParagraphEditor class methodsFor: 'class initialization' stamp: 'BG 10/28/2003 17:46'!
initializeTextEditorMenus
	"Initialize the yellow button pop-up menu and corresponding messages."
	"ParagraphEditor initializeTextEditorMenus"

	TextEditorYellowButtonMenu _ SelectionMenu
		labels:
'find...(f)
find again (g)
set search string (h)
do again (j)
undo (z)
copy (c)
cut (x)
paste (v)
paste...
do it (d)
print it (p)
inspect it (i)
debug it
accept (s)
cancel (l)
show bytecodes
more...'
		lines: #(3 5 9 13 15 16)
		selections: #(find findAgain setSearchString again undo copySelection cut paste pasteRecent doIt printIt inspectIt debugIt accept cancel showBytecodes shiftedTextPaneMenuRequest).! !

!ParagraphEditor class methodsFor: 'class initialization' stamp: 'BG 11/1/2003 14:29'!
shiftedYellowButtonMenu
	"Answer the menu to be presented when the yellow button is pressed while the shift key is down"

	^ SelectionMenu fromArray: #(
		('set font... (k)'					offerFontMenu)
		('set style... (K)'					changeStyle)
		('set alignment...'				chooseAlignment)
		-
		('explain'						explain)
		('pretty print'					prettyPrint)
		('pretty print with color'		prettyPrintWithColor)
		('file it in (G)'					fileItIn)
		('recognizer (r)'					recognizeCharacters)
		('spawn (o)'						spawn)
"		-
		('definition of word'				wordDefinition)
		('verify spelling of word'		verifyWordSpelling)
		('spell check it'					spellCheckIt)	
		('translate it'					translateIt)
		('choose language'				languagePrefs) "
		-
		('browse it (b)'					browseIt)
		('senders of it (n)'				sendersOfIt)
		('implementors of it (m)'		implementorsOfIt)
		('references to it (N)'			referencesToIt)
		-
		('selectors containing it (W)'		methodNamesContainingIt)
		('method strings with it (E)'		methodStringsContainingit)
		('method source with it'			methodSourceContainingIt)
		('class names containing it'		classNamesContainingIt)
		('class comments with it'		classCommentsContainingIt)
		('change sets with it'			browseChangeSetsWithSelector)
		-
		('save contents to file...'			saveContentsInFile)
		-
		('special menu...'				presentSpecialMenu)
		('more...'						yellowButtonActivity))! !


!Parser methodsFor: 'public access' stamp: 'BG 10/29/2003 08:54'!
parse: sourceStream class: class noPattern: noPattern context: ctxt notifying: req ifFail: aBlock 
        "Answer a MethodNode for the argument, sourceStream, that is the root of 
        a parse tree. Parsing is done with respect to the argument, class, to find 
        instance, class, and pool variables; and with respect to the argument, 
        ctxt, to find temporary variables. Errors in parsing are reported to the 
        argument, req, if not nil; otherwise aBlock is evaluated. The argument 
        noPattern is a Boolean that is true if the the sourceStream does not 
        contain a method header (i.e., for DoIts)."

         | meth repeatNeeded myStream parser |
        (req notNil and: [RequestAlternateSyntaxSetting signal and: [(sourceStream isKindOf: FileStream) not]])
                ifTrue: [parser _ self as: DialectParser]
                ifFalse: [parser _ self].
        myStream _ sourceStream.
        [repeatNeeded _ false.
        parser init: myStream notifying: req failBlock: [^ aBlock value].
        doitFlag _ noPattern.
        failBlock_ aBlock.
        [meth _ parser method: noPattern context: ctxt
                                encoder: (Encoder new init: class context: ctxt notifying: parser)] 
                on: ParserRemovedUnusedTemps 
                do: 
                        [ :ex | repeatNeeded _ true.
                        myStream _ ReadStream on: requestor text string.
                        ex resume].
        repeatNeeded] whileTrue.
        encoder _ failBlock _ requestor _ parseNode _ nil. "break cycles & mitigate refct overflow"
        ^ meth! !


!PopUpMenu methodsFor: 'basic control sequence' stamp: 'BG 10/28/2003 21:02'!
startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean
	"Display the menu, with caption if supplied. Wait for the mouse button to go down, then track the selection as long as the button is pressed. When the button is released,
	Answer the index of the current selection, or zero if the mouse is not released over  any menu item. Location specifies the desired topLeft of the menu body rectangle. The final argument indicates whether the menu should seize the keyboard focus in order to allow the user to navigate it via the keyboard."

	| maxHeight |
	maxHeight _ Display height*3//4.
	self frameHeight > maxHeight ifTrue:
		[^ self
			startUpSegmented: maxHeight
			withCaption: captionOrNil
			at: location
			allowKeyboard: aBoolean].



	frame ifNil: [self computeForm].
	Cursor normal showWhile:
		[self
			displayAt: location
			withCaption: captionOrNil
			during: [self controlActivity]].
	^ selection! !


!Preference methodsFor: 'as yet unclassified' stamp: 'BG 10/31/2003 14:32'!
changeInformee

  ^changeInformee! !

!Preference methodsFor: 'as yet unclassified' stamp: 'BG 10/31/2003 14:47'!
deleteInformee

  changeInformee := nil! !


!Preferences class methodsFor: 'reacting to change' stamp: 'BG 10/28/2003 19:53'!
sharedFlapsSettingChanged
	"The current value of the showSharedFlaps flag has changed; now react"

	self showSharedFlaps  "viz. the new setting"
		ifFalse:		
			[]

		ifTrue:
			[Smalltalk isMorphic ifTrue:
				[self currentWorld addGlobalFlaps]]! !

!Preferences class methodsFor: 'preferences panel' stamp: 'BG 10/30/2003 21:17'!
openPreferencesInspector
	"Open a window on the current set of preferences choices, allowing the user to view and change their settings"
	
	self inspectPreferences! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/29/2003 08:39'!
chooseFontWithPrompt: aPrompt andSendTo: aReceiver withSelector: aSelector
	Smalltalk isMorphic
		ifFalse:
			[Utilities mvcPromptForFont: aPrompt andSendTo: aReceiver withSelector: aSelector]
		ifTrue:
			[]! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/30/2003 21:20'!
presentMvcFontConfigurationMenu
	| aMenu result |
	aMenu _ CustomMenu new.
	aMenu title: 'Standard System Fonts'.
	aMenu add: 'default text font...' action: #chooseSystemFont.
	aMenu add: 'list font...' action: #chooseListFont.
	aMenu add: 'menu font...' action: #chooseMenuFont.
	aMenu add: 'window-title font...' action: #chooseWindowTitleFont.
	"aMenu add: 'code font...' action: #chooseCodeFont."
	aMenu addLine.
	aMenu add: 'restore default font choices' action: #restoreDefaultFonts.

	(result _ aMenu startUp) ifNotNil:
		[self perform: result]
! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/30/2003 21:20'!
restoreDefaultFonts
	"Preferences restoreDefaultFonts"
	#(	(setButtonFontTo:		ComicBold		16)
		"(setCodeFontTo:			NewYork		12)"  "Later"
		(setListFontTo:			NewYork		12)
		(setMenuFontTo:			NewYork		12)
		(setWindowTitleFontTo:	NewYork		15)
		(setSystemFontTo:		NewYork		12)) do:
			[:triplet |
				self perform: triplet first with: (StrikeFont familyName: triplet second size: triplet third)].

	Smalltalk at: #BalloonMorph ifPresent:
		[:thatClass | thatClass setBalloonFontTo: (StrikeFont familyName: #ComicPlain size: 12)].

	"Note:  The standardCodeFont is not currently used -- the default font is instead; later hopefully we can split the code font out as  a separate choice, but only after we're able to have the protocols reorganized such that we can know whether it's code or not when we launch the text object.

	Note:  The standard button font is reset by this code but is not otherwise settable by a public UI (too many things can go afoul) "! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/28/2003 19:52'!
setCodeFontTo: aFont
	"Not currently sent, but once protocols are sorted out so that we can discriminate on whether a text object being launched is for code or not, might deserve to be reincorporated"

	Parameters at: #standardCodeFont put: aFont.
! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/28/2003 19:52'!
setListFontTo: aFont
	"Set the list font as indicated"

	Parameters at: #standardListFont put: aFont.
	ListParagraph initialize.
! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/28/2003 19:52'!
setSystemFontTo: aFont
	"Establish the default text font and style"

	| aStyle newDefaultStyle |
	aFont ifNil: [^ self].
	aStyle _ aFont textStyle ifNil: [^ self].
	newDefaultStyle _ aStyle copy.
	newDefaultStyle defaultFontIndex: (aStyle fontIndexOf: aFont).
	TextConstants at: #DefaultTextStyle put: newDefaultStyle.
! !

!Preferences class methodsFor: 'fonts' stamp: 'BG 10/28/2003 19:52'!
setWindowTitleFontTo: aFont
	"Set the window-title font to be as indicated"

	Parameters at: #windowTitleFont put: aFont.
	StandardSystemView setLabelStyle.! !

!Preferences class methodsFor: 'shrinking' stamp: 'BG 10/29/2003 18:50'!
removeFlaps
 "Preferences removeFlaps"

  DictionaryOfPreferences
     removeKey: #automaticFlapLayout ifAbsent: [];
     removeKey: #navigatorOnLeftEdge ifAbsent: [];
     removeKey: #classicNavigatorEnabled ifAbsent: []! !

!Preferences class methodsFor: 'as yet unclassified' stamp: 'BG 10/31/2003 14:46'!
removeObsoleteInstances
 "Preferences removeObsoleteInstances"
    | oc |
 oc := OrderedCollection new.

Preference allInstances
  collect: [:item | ( item changeInformee notNil and: [item changeInformee isObsolete])
         ifTrue: [item deleteInformee]
          ].! !


!ProcessBrowser methodsFor: 'stack list' stamp: 'BG 10/29/2003 10:17'!
messageTally
	| secString secs |
	secString _ FillInTheBlank request: 'Profile for how many seconds?' initialAnswer: '4'.
	secs _ secString asNumber asInteger.
	(secs isNil
			or: [secs isZero])
		ifTrue: [^ self].
	! !


!ProcessBrowser class methodsFor: 'instance creation' stamp: 'BG 10/29/2003 00:38'!
open
	"ProcessBrowser open"
	"Create and schedule a ProcessBrowser."
	Smalltalk garbageCollect.
	^ Smalltalk isMorphic
		ifTrue: [ ]
		ifFalse: [ self new openAsMVC ]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'BG 10/29/2003 08:16'!
nameAndRulesFor: aProcess 
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	(aProcess priority = Processor timingPriority
			and: [aProcess suspendedContext receiver == Delay])
		ifTrue: [^ {'the timer interrupt watcher'. false. false}].
	^ [ aProcess caseOf: {
		[] -> [{'no process'. false. false}].
		[Smalltalk lowSpaceWatcherProcess] -> [{'the low space watcher'. false. false}].
		[WeakArray runningFinalizationProcess] -> [{'the WeakArray finalization process'. false. false}].
		[Processor activeProcess] -> [{'the UI process'. false. true}].
		[Processor backgroundProcess] -> [{'the idle process'. false. false}].
		[Sensor inputProcess] -> [{'the I/O process'. false. false}].
		[Sensor interruptWatcherProcess] -> [{'the user interrupt watcher'. false. false}].
		[Project uiProcess] -> [{'the inactive Morphic UI process'. false. false}].
	"	[SoundPlayer playerProcess] -> [{'the Sound Player'. false. false}]. "
		[ScheduledControllers ifNotNil: [ ScheduledControllers activeControllerProcess ] ] -> [{'the inactive MVC controller process'. false. true}].
		[CPUWatcher currentWatcherProcess] -> [{'the CPUWatcher' . false . false}]
	}
		 otherwise: [{aProcess suspendedContext asString. true. true}] ]
			ifError: [ :err :rcvr | {aProcess suspendedContext asString. true. true} ]! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'BG 10/29/2003 01:07'!
tallyCPUUsageFor: seconds every: msec
	"Compute CPU usage using a msec millisecond sample for the given number of seconds,
	then dump the usage statistics on the Transcript. The UI is free to continue, meanwhile"
	"ProcessBrowser tallyCPUUsageFor: 10 every: 100"

	| promise |
	promise _ Processor tallyCPUUsageFor: seconds every: msec.

	[ | tally |
		tally _ promise value.
		Smalltalk isMorphic
			ifTrue: [ ]
			ifFalse: [ [ Transcript open ] forkAt: Processor userSchedulingPriority.
					[ (Delay forSeconds: 1) wait.
					self dumpTallyOnTranscript: tally ] forkAt: Processor userSchedulingPriority.]
	] fork.! !


!Project methodsFor: 'accessing' stamp: 'BG 10/29/2003 09:50'!
findProjectView: projectDescription
	| pName proj |
	"In this world, find the morph that holds onto the project described by projectDescription.  projectDescription can be a project, or the name of a project.  The project may be represented by a DiskProxy.  The holder morph may be at any depth in the world.
	Need to fix this if Projects have subclasses, or if a class other than ProjectViewMorph can officially hold onto a project.  (Buttons, links, etc)
	If parent is an MVC world, return the ProjectController."

	self flag: #bob.		"read the comment"


	pName _ (projectDescription isKindOf: String) 
		ifTrue: [projectDescription]
		ifFalse: [projectDescription name].
	self isMorphic 
		ifTrue: []
		ifFalse: [world scheduledControllers do: [:cont |
				(cont isKindOf: ProjectController) ifTrue: [
					((proj _ cont model) class == Project and: 
						[proj name = pName]) ifTrue: [^ cont view].

					"proj class == DiskProxy ifTrue: [ 
						dpName _ proj constructorArgs first.
						dpName _ (dpName findTokens: '/') last.
						dpName _ (Project parseProjectFileName: dpName unescapePercents) first.
						dpName = pName ifTrue: [^ cont view]]" ]]
			].
	^ nil! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/28/2003 21:03'!
assureNavigatorPresenceMatchesPreference
	"Make sure that the current project conforms to the presence/absence of the navigator"

	Smalltalk isMorphic ifFalse: [^ self].! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/29/2003 01:03'!
enter: returningFlag revert: revertFlag saveForRevert: saveForRevert
	"Install my ChangeSet, Transcript, and scheduled views as current globals. If returningFlag is true, we will return to the project from whence the current project was entered; don't change its previousProject link in this case.
	If saveForRevert is true, save the ImageSegment of the project being left.
	If revertFlag is true, make stubs for the world of the project being left.
	If revertWithoutAsking is true in the project being left, then always revert."

	| showZoom recorderOrNil old forceRevert response seg newProcess |

	
	self isCurrentProject ifTrue: [^ self].
	"Check the guards"
	guards ifNotNil:
		[guards _ guards reject: [:obj | obj isNil].
		guards do: [:obj | obj okayToEnterProject ifFalse: [^ self]]].
	CurrentProject world triggerEvent: #aboutToLeaveWorld.
	forceRevert _ false.
	CurrentProject rawParameters 
		ifNil: [revertFlag ifTrue: [^ self inform: 'nothing to revert to']]
		ifNotNil: [saveForRevert ifFalse: [
				forceRevert _ CurrentProject projectParameters 
								at: #revertWithoutAsking ifAbsent: [false]]].
	forceRevert not & revertFlag ifTrue: [
		response _ SelectionMenu confirm: 'Are you sure you want to destroy this Project\ and revert to an older version?\\(From the parent project, click on this project''s thumbnail.)' withCRs
			trueChoice: 'Revert to saved version' 
			falseChoice: 'Cancel'.
		response ifFalse: [^ self]].

	revertFlag | forceRevert 
		ifTrue: [seg _ CurrentProject projectParameters at: #revertToMe ifAbsent: [
					^ self inform: 'nothing to revert to']]
		ifFalse: [
			CurrentProject finalExitActions.
			CurrentProject makeThumbnail.
			returningFlag == #specialReturn
				ifTrue:
					[ProjectHistory forget: CurrentProject.		"this guy is irrelevant"
					Project forget: CurrentProject]
				ifFalse:
					[ProjectHistory remember: CurrentProject]].

	(revertFlag | saveForRevert | forceRevert) ifFalse:
		[(Preferences valueOfFlag: #projectsSentToDisk) ifTrue:
			[self storeToMakeRoom]].

	CurrentProject abortResourceLoading.
	Smalltalk isMorphic ifTrue: [CurrentProject world triggerClosingScripts].

	CurrentProject saveProjectPreferences.

	"Update the display depth and make a thumbnail of the current project"
	CurrentProject displayDepth: Display depth.
	old _ CurrentProject.		"for later"

	"Show the project transition.
	Note: The project zoom is run in the context of the old project,
		so that eventual errors can be handled accordingly"
	displayDepth == nil ifTrue: [displayDepth _ Display depth].
	self installNewDisplay: Display extent depth: displayDepth.
	(showZoom _ self showZoom) ifTrue: [
		self displayZoom: CurrentProject parent ~~ self].

	(world isMorph and: [world hasProperty: #letTheMusicPlay])
		ifTrue: [world removeProperty: #letTheMusicPlay]
		ifFalse: [Smalltalk at: #ScorePlayer ifPresentAndInMemory:
					[:playerClass | playerClass allSubInstancesDo:
						[:player | player pause]]].

	returningFlag == #specialReturn ifTrue: [
		old removeChangeSetIfPossible.	"keep this stuff from accumulating"
		nextProject _ nil
	] ifFalse: [
		returningFlag
			ifTrue: [nextProject _ CurrentProject]
			ifFalse: [previousProject _ CurrentProject].
	].

	CurrentProject saveState.
	CurrentProject isolationHead == self isolationHead ifFalse:
		[self invokeFrom: CurrentProject].
	CurrentProject _ self.
	self installProjectPreferences.
	Smalltalk newChanges: changeSet.
	TranscriptStream newTranscript: transcript.
	Sensor flushKeyboard.
	Smalltalk isMorphic ifTrue: [recorderOrNil _ World pauseEventRecorder].

	ProjectHistory remember: CurrentProject.

	world isMorph
		ifTrue:
			[World _ world.  "Signifies Morphic"
			world install.
			world transferRemoteServerFrom: old world.
			"(revertFlag | saveForRevert | forceRevert) ifFalse: [
				(Preferences valueOfFlag: #projectsSentToDisk) ifTrue: [
					self storeSomeSegment]]."
			recorderOrNil ifNotNil: [recorderOrNil resumeIn: world].
			world triggerOpeningScripts]
		ifFalse:
			[World _ nil.  "Signifies MVC"
			Smalltalk at: #ScheduledControllers put: world].

	saveForRevert ifTrue: [
		Smalltalk garbageCollect.	"let go of pointers"
		old storeSegment.
		"result _" old world isInMemory 
			ifTrue: ['Can''t seem to write the project.']
			ifFalse: [old projectParameters at: #revertToMe put: 
					old world xxxSegment clone].
				'Project written.'].
			"original is for coming back in and continuing."

	revertFlag | forceRevert ifTrue: [
		seg clone revert].	"non-cloned one is for reverting again later"
	self removeParameter: #exportState.

	"Complete the enter: by launching a new process"
	world isMorph ifTrue: [
	] ifFalse: [
		
		newProcess _ [	
			ScheduledControllers resetActiveController.	"in case of walkback in #restore"
			showZoom ifFalse: [ScheduledControllers restore].
			ScheduledControllers searchForActiveController
		] fixTemps newProcess priority: Processor userSchedulingPriority.
		newProcess resume.		"lose the current process and its referenced morphs"
		Processor terminateActive.
	]! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/30/2003 21:26'!
enterForEmergencyRecovery
	"This version of enter invokes an absolute minimum of mechanism.
	An unrecoverable error has been detected in an isolated project.
	It is assumed that the old changeSet has already been revoked.
	No new process gets spawned here.  This will happen in the debugger."

	self isCurrentProject ifTrue: [^ self].
	CurrentProject saveState.
	CurrentProject _ self.
	Display newDepthNoRestore: displayDepth.
	Smalltalk newChanges: changeSet.
	TranscriptStream newTranscript: transcript.


	world isMorph
		ifTrue:
			["Entering a Morphic project"
			World _ world.
			world install.
			world triggerOpeningScripts]
		ifFalse:
			["Entering an MVC project"
			World _ nil.
			Smalltalk at: #ScheduledControllers put: world.
			ScheduledControllers restore].
	UIProcess _ Processor activeProcess.
! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/29/2003 01:09'!
finalExitActions

	world isMorph ifTrue: [
	].
! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/28/2003 20:07'!
makeThumbnail
	"Make a thumbnail image of this project from the Display."

	world isMorph ifTrue: [world displayWorldSafely]. "clean pending damage"
	viewSize ifNil: [viewSize _ Display extent // 8].
	thumbnail _ Form extent: viewSize depth: Display depth.
	(WarpBlt current toForm: thumbnail)
			sourceForm: Display;
			cellSize: 2;  "installs a colormap"
			combinationRule: Form over;
			copyQuad: (Display boundingBox) innerCorners
			toRect: (0@0 extent: viewSize).

	^thumbnail
! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/28/2003 21:04'!
saveState
	"Save the current state in me prior to leaving this project"

	changeSet _ Smalltalk changes.
	thumbnail ifNotNil: [thumbnail hibernate].
	Smalltalk isMorphic
		
		ifFalse:
			[world _ ScheduledControllers.
			ScheduledControllers unCacheWindows].
	Sensor flushAllButDandDEvents. "Will be reinstalled by World>>install"
	transcript _ Transcript.
! !

!Project methodsFor: 'menu messages' stamp: 'BG 10/29/2003 09:53'!
viewLocFor: exitedProject
	"Look for a view of the exitedProject, and return its center"
	| ctlr |
	world isMorph
		ifTrue: []
		ifFalse: [ctlr _ world controllerWhoseModelSatisfies:
							[:p | p == exitedProject].
				ctlr ifNotNil: [^ ctlr view windowBox center]].

	^ Sensor cursorPoint.  "default result"

! !

!Project methodsFor: 'release' stamp: 'BG 10/29/2003 09:12'!
okToChange
	"Answer whether the window in which the project is housed can be dismissed -- which is destructive. We never clobber a project without confirmation"

	| ok |
	self subProjects size  >0 ifTrue:
		[self inform: 
'The project ', self name printString, '
contains sub-projects.  You must remove these
explicitly before removing their parent.'.
		^ false].
	ok _ world isMorph not and: [world scheduledControllers size <= 1].
	ok ifFalse: [self isMorphic ifTrue:
		[self parent == CurrentProject 
			ifFalse: [^ true]]].  "view from elsewhere.  just delete it."
	ok _ (self confirm:
'Really delete the project
', self name printString, '
and all its windows?').
		
	ok ifFalse: [^ false].



	self removeChangeSetIfPossible.
	"do this last since it will render project inaccessible to #allProjects and their ilk"
	ProjectHistory forget: self.
	Project deletingProject: self.
	^ true
! !

!Project methodsFor: 'release' stamp: 'BG 10/28/2003 21:04'!
subProjects
	"Answer a list of all the subprojects  of the receiver.  This is nastily idiosyncratic."

	^ self isMorphic
		
		ifFalse:
			[(world controllersSatisfying:
					[:m | m model isKindOf: Project])
				collect:
					[:c | c model]]
! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 01:08'!
armsLengthCommand: aCommand withDescription: aString
	| pvm |
	"Set things up so that this aCommand is sent to self as a message
after jumping to the parentProject.  For things that can't be executed
while in this project, such as saveAs, loadFromServer, storeOnServer.  See
ProjectViewMorph step."

	self isMorphic ifTrue: [
	] ifFalse: [
		parentProject ifNil: [^ self inform: 'The top project can''t do that'].
		pvm _ parentProject findProjectView: self.
		pvm armsLengthCommand: {self. aCommand}.
		self exit.
	].
! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 00:50'!
beep
	! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 08:18'!
currentVersionNumber

	version ifNil: [^0].
	version isInteger ifTrue:[^version].
	"version _ Base64MimeConverter decodeInteger: version unescapePercents."
	^version! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/30/2003 21:09'!
findAFolderToLoadProjectFrom

	
		^PluggableFileList getFolderDialog openLabel: 'Select a folder on a server:'
	! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 00:10'!
findAFolderToStoreProjectIn

	"Alan wants something prettier with a default"

	self couldOpenInMorphic ifTrue: [
	] ifFalse: [
		^PluggableFileList getFolderDialog openLabel: 'Select a folder on a server:'
	]! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 09:06'!
loadFromServer: newerAutomatically
	"If a newer version of me is on the server, load it."
	| pair resp server |
	self assureIntegerVersion.

	self isCurrentProject ifTrue: ["exit, then do the command"
		^ self armsLengthCommand: #loadFromServer withDescription: 'Loading'
	].
	server _ self tryToFindAServerWithMe ifNil: [^ nil].
	pair _ self class mostRecent: self name onServer: server.
	pair first ifNil: [^ self inform: 'can''t find file on server for ', self name].
	self currentVersionNumber > pair second ifTrue: [
		^ self inform: 'That server has an older version of the project.'].
	version = (Project parseProjectFileName: pair first) second ifTrue: [
		resp _ (PopUpMenu labels: 'Reload anyway\Cancel' withCRs) startUpWithCaption: 
					'The only changes are the ones you made here.'.
		resp ~= 1 ifTrue: [^ nil]
	] ifFalse: [
		newerAutomatically ifFalse: [
			resp _ (PopUpMenu labels: 'Load it\Cancel' withCRs) startUpWithCaption: 
						'A newer version exists on the server.'.
			resp ~= 1 ifTrue: [^ nil]
		].
	].

	"let's avoid renaming the loaded change set since it will be replacing ours"
	self projectParameters at: #loadingNewerVersion put: true.

	
			ProjectLoading
				installRemoteNamed: pair first
				from: server
				named: self name
				in: parentProject
! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 09:10'!
saveAs
	"Forget where stored before, and store.  Will ask user where."

	"self forgetExistingURL.
	self storeOnServer. "! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 10:08'!
serverList
	| servers server |
	"Take my list of server URLs and return a list of ServerDirectories to write on."

	urlList isEmptyOrNil ifTrue: [^ nil].
	servers _ OrderedCollection new.
	urlList do: [:url |
		"server _ ServerDirectory serverForURL: url.
		server ifNotNil: [servers add: server].
		server _ ServerDirectory serverForURL: url asUrl downloadUrl.
		server ifNotNil: [servers add: server] "].
	^servers isEmpty
		ifTrue: [nil]
		ifFalse: [servers]! !

!Project methodsFor: 'file in/out' stamp: 'BG 10/29/2003 09:16'!
storeSegment
	"Store my project out on the disk as an ImageSegment.  Keep the outPointers in memory.  Name it <project name>.seg.  *** Caller must be holding (Project alInstances) to keep subprojects from going out. ***"

(World == world) ifTrue: [^ false]. 
	"self inform: 'Can''t send the current world out'."
world isInMemory ifFalse: [^ false].  "already done"
world isMorph ifFalse: [
	self projectParameters at: #isMVC put: true.
	^ false].	"Only Morphic projects for now"
world ifNil: [^ false].  world presenter ifNil: [^ false].

! !

!Project methodsFor: 'flaps support' stamp: 'BG 10/29/2003 08:42'!
flapsSuppressed: aBoolean
	"Make the setting of the flag that governs whether global flaps are suppressed in the project be as indicated and add or remove the actual flaps"

	self projectPreferenceFlagDictionary at: #showSharedFlaps put: aBoolean not.
	self == Project current  "Typical case"
		ifTrue:
			[Preferences setPreference: #showSharedFlaps toValue: aBoolean not]
		ifFalse:   "Anomalous case where this project is not the current one."
			[aBoolean
				ifTrue:		
					[]

				ifFalse:
					[Smalltalk isMorphic  ifTrue:
						[]]].
	Project current assureNavigatorPresenceMatchesPreference! !

!Project methodsFor: 'displaying' stamp: 'BG 10/29/2003 08:31'!
imageFormOfSize: extentPoint depth: d
	| newDisplay |
	newDisplay _ DisplayScreen extent: extentPoint depth: d.
	Display replacedBy: newDisplay do:[
		world restore. "MVC"
	].
	^newDisplay! !

!Project methodsFor: 'isolation layers' stamp: 'BG 10/29/2003 09:27'!
beIsolated
	"Establish an isolation layer at this project.
	This requires clearing the current changeSet or installing a new one."

   self error: 'Not implemented'.
"	isolatedHead ifTrue: [^ self error: 'Already isolated'].
	self isCurrentProject ifFalse:
		[^ self inform: 'Must be in this project to isolate it'.].
	changeSet isEmpty ifFalse: [changeSet _ ChangeSorter newChangeSet].
	changeSet beIsolationSetFor: self.
	isolatedHead _ true.
	inForce _ true.
	environment _ Environment new setName: self name outerEnvt: Smalltalk. "

! !


!Project class methodsFor: 'instance creation' stamp: 'BG 10/29/2003 09:28'!
newMorphicOn: aPasteUpOrNil

"	| newProject |

	newProject _ self basicNew initMorphic.
	self addingProject: newProject.
	aPasteUpOrNil ifNotNil: [newProject installPasteUpAsWorld: aPasteUpOrNil].
	newProject createViewIfAppropriate.
	^newProject "
! !

!Project class methodsFor: 'utilities' stamp: 'BG 10/29/2003 09:38'!
addItem: item toMenu: menu selection: action project: aProject
	| aColor |
	aColor _ aProject isMorphic 
		ifTrue: [aProject world isInMemory 
			ifTrue: [Color red darker] 
			ifFalse: [Color brown]]
		ifFalse: [aProject world isInMemory 
			ifTrue: [Color veryVeryDarkGray]
			ifFalse: [Color blue]].
	menu add: item action: action! !

!Project class methodsFor: 'utilities' stamp: 'BG 10/29/2003 09:18'!
deletingProject: outgoingProject

	
	Project allProjects do: [:p | p deletingProject: outgoingProject].	"ones that are in"
	

	AllProjects _ self allProjects copyWithout: outgoingProject.! !

!Project class methodsFor: 'utilities' stamp: 'BG 10/28/2003 21:04'!
interruptName: labelString
	"Create a Notifier on the active scheduling process with the given label."
	
	Smalltalk isMorphic ifFalse:
		[^ ScheduledControllers interruptName: labelString].! !

!Project class methodsFor: 'squeaklet on server' stamp: 'BG 10/29/2003 08:21'!
parseProjectFileName: aString
	"It was formerly possible to have periods in projct names and this messed up some parsing methods. Try to handle that more gracefully and allow for a change in scheme at a later time.
	ar 10/11/2000: Switch to a different version encoding scheme. The new scheme is
		baseName.NNN.ext
	where NNN is at least three digits wide and encodes the version in a human readable form.
	Examples:
		Project parseProjectFileName: 'My Project.007.pr'.
		Project parseProjectFileName: 'My.First.Project.042.prj'.
		Project parseProjectFileName: 'My Project.123456.p r o j e c t'.
	The 'dot' is determined on FileDirectory>>dot to compensate for platforms wishing to use something different from a period. Also allows parsing the former encoding of file using Base64 encoded versions of the form
	Project parseProjectFileName: 'aa.bb.cc|AQ.ss'
	"
	| suffix baseName version versionAndSuffix index tokens |
	"answer an array with: 
		1 = basic project name
		2 = version string
		3 = suffix (pr)"

	"First check for the old style versions"
	index _ aString findLast:[:ch| ch = $|].
	index = 0 ifFalse:["Old style version"
		baseName _ aString copyFrom: 1 to: index-1.
		versionAndSuffix _ aString copyFrom: index+1 to: aString size.
		(versionAndSuffix occurrencesOf: $.) = 0 ifTrue: [^ #('no suffix')].
		version _ versionAndSuffix copyUpTo: $..
		suffix _ versionAndSuffix copyFrom: version size+1 to: versionAndSuffix size.
		"Decode Base64 encoded version"
		version isEmpty
			ifTrue:[version _ 0]
			ifFalse:[version _ 0 "Base64MimeConverter decodeInteger: version unescapePercents"].
		^{baseName. version. suffix}].
	"New style versions"
	tokens _ aString findTokens: FileDirectory dot.
	tokens size < 2 "Not even a single dot"
		ifTrue:[^{aString. 0. ''}].
	tokens size < 3 ifTrue:["Only one dot"
		self flag: #arNote. "We could allow project file names of the form 'project.001' (e.g., no project extension) or '.001.pr' (without a base name) but I don't think its a good idea."
		^{tokens first. 0. tokens last}].
	suffix _ tokens last.
	version _ tokens at: tokens size - 1.
	(version anySatisfy:[:ch| ch isDigit not]) ifTrue:[
		"Non-digit version??? I don't think so..."
		baseName _ aString copyFrom: 1 to: aString size - suffix size - 1.
		^{baseName. 0. suffix}].
	baseName _ aString copyFrom: 1 to: aString size - suffix size - version size - 2.
	version _ version asInteger.
	^{baseName. version. suffix}! !

!Project class methodsFor: 'squeaklet on server' stamp: 'BG 10/29/2003 10:01'!
serverFileFromURL: aURLString

	"|  dir | "
	self flag: #bob.		"need to include swikis in this - hacked for now"

"	(aURLString findString: '/SuperSwikiProj/') > 0 ifTrue: [
		dir _ SuperSwikiServer new fullPath: (aURLString copyUpToLast: $/).
		^dir readOnlyFileNamed: (aURLString findTokens: '/') last
	]. 
	^ServerFile new fullPath: aURLString "

  self halt.! !


!ProjectLoading class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:29'!
openName: aFileName stream: preStream fromDirectory: aDirectoryOrNil withProjectView: existingView
	"Reconstitute a Morph from the selected file, presumed to be represent a Morph saved via the SmartRefStream mechanism, and open it in an appropriate Morphic world."
 	| |
	(preStream isNil or: [preStream size = 0]) ifTrue: [
		ProgressNotification  signal: '9999 about to enter project'.		"the hard part is over"
		^self inform: 
'It looks like a problem occurred while
getting this project. It may be temporary,
so you may want to try again,'
	].! !

!ProjectLoading class methodsFor: 'as yet unclassified' stamp: 'BG 10/30/2003 15:12'!
thumbnailFromUrl: urlString

	"Load the project, and make a thumbnail to it in the current project.
ProjectLoading thumbnailFromUrl: 'http://www.squeak.org/Squeak2.0/2.7segments/SqueakEasy.extSeg'.
"

	Project canWeLoadAProjectNow ifFalse: [^ self].
! !


!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:53'!
assignCollapseFrameFor: aSSView 
	"Offer up a location along the left edge of the screen for a collapsed SSView. Make sure it doesn't overlap any other collapsed frames."

	| grid otherFrames topLeft viewBox collapsedFrame extent newFrame verticalBorderDistance top |
	grid _ 8.
	verticalBorderDistance _ 8.
	aSSView isMorph
		ifTrue: []
		ifFalse: [otherFrames _ ScheduledControllers scheduledWindowControllers
						collect: [:aController | aController view ~= aSSView ifTrue: [aController view collapsedFrame]]
						thenSelect: [:rect | rect notNil].
				viewBox _ Display boundingBox].
	collapsedFrame _ aSSView collapsedFrame.
	extent _ collapsedFrame notNil
				ifTrue: [collapsedFrame extent]
				ifFalse: [aSSView isMorph
					ifTrue: [aSSView getRawLabel width + aSSView labelWidgetAllowance @ (aSSView labelHeight + 2)]
					ifFalse: [(aSSView labelText extent x + 70) @ aSSView labelHeight
							min: aSSView labelDisplayBox extent]].
	collapsedFrame notNil
		ifTrue: [(otherFrames anySatisfy: [:f | collapsedFrame intersects: f])
				ifFalse: ["non overlapping"
					^ collapsedFrame]].
	top _ viewBox top + verticalBorderDistance.
	[topLeft _ viewBox left @ top.
	newFrame _ topLeft extent: extent.
	newFrame bottom <= (viewBox height - verticalBorderDistance)]
		whileTrue: 
			[(otherFrames anySatisfy: [:w | newFrame intersects: w])
				ifFalse: ["no overlap"
					^ newFrame].
			top _ top + grid].
	"If all else fails... (really to many wins here)"
	^ 0 @ 0 extent: extent! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:54'!
initialFrameFor: aView initialExtent: initialExtent world: aWorld
	"Find a plausible initial screen area for the supplied view, which should be a StandardSystemView, taking into account the 'reverseWindowStagger' Preference, the size needed, and other windows currently on the screen."

	| allOrigins screenRight screenBottom putativeOrigin putativeFrame allowedArea staggerOrigin otherFrames |

	Preferences reverseWindowStagger ifTrue:
		[^ self strictlyStaggeredInitialFrameFor: aView initialExtent: initialExtent world: aWorld].

	allowedArea _ self maximumUsableAreaInWorld: aWorld.
	screenRight _ allowedArea right.
	screenBottom _ allowedArea bottom.

	otherFrames _ Smalltalk isMorphic
		ifTrue: []
		ifFalse: [ScheduledControllers scheduledWindowControllers
				select: [:aController | aController view ~~ nil]
				thenCollect: [:aController | aController view isCollapsed
								ifTrue: [aController view expandedFrame]
								ifFalse: [aController view displayBox]]].

	allOrigins _ otherFrames collect: [:f | f origin].
	(self standardPositionsInWorld: aWorld) do:  "First see if one of the standard positions is free"
		[:aPosition | (allOrigins includes: aPosition)
			ifFalse:
				[^ (aPosition extent: initialExtent) squishedWithin: allowedArea]].

	staggerOrigin _ (self standardPositionsInWorld: aWorld) first.  "Fallback: try offsetting from top left"
	putativeOrigin _ staggerOrigin.

	[putativeOrigin _ putativeOrigin + StaggerOffset.
	putativeFrame _ putativeOrigin extent: initialExtent.
	(putativeFrame bottom < screenBottom) and:
					[putativeFrame right < screenRight]]
				whileTrue:
					[(allOrigins includes: putativeOrigin)
						ifFalse:
							[^ (putativeOrigin extent: initialExtent) squishedWithin: allowedArea]].
	^ (self scrollBarSetback @ self screenTopSetback extent: initialExtent) squishedWithin: allowedArea! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 08:24'!
reduceByFlaps: aScreenRect 
	"Return a rectangle that won't interfere with default shared flaps"

	^ aScreenRect insetBy: 18! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:54'!
strictlyStaggeredInitialFrameFor: aStandardSystemView initialExtent: initialExtent world: aWorld
	"This method implements a staggered window placement policy that I (di) like.
	Basically it provides for up to 4 windows, staggered from each of the 4 corners.
	The windows are staggered so that there will always be a corner visible."

	| allowedArea grid initialFrame otherFrames cornerSel corner delta putativeCorner free maxLevel |

	allowedArea _(self maximumUsableAreaInWorld: aWorld)
		insetBy: (self scrollBarSetback @ self screenTopSetback extent: 0@0).
	
	"Number to be staggered at each corner (less on small screens)"
	maxLevel _ allowedArea area > 300000 ifTrue: [3] ifFalse: [2].
	"Amount by which to stagger (less on small screens)"
	grid _ allowedArea area > 500000 ifTrue: [40] ifFalse: [20].
	initialFrame _ 0@0 extent: ((initialExtent
							"min: (allowedArea extent - (grid*(maxLevel+1*2) + (grid//2))))
							min: 600@400")).
	otherFrames _ Smalltalk isMorphic
		ifTrue: []
		ifFalse: [ScheduledControllers scheduledWindowControllers
				select: [:aController | aController view ~~ nil]
				thenCollect: [:aController | aController view isCollapsed
								ifTrue: [aController view expandedFrame]
								ifFalse: [aController view displayBox]]].
	0 to: maxLevel do:
		[:level | 
		1 to: 4 do:
			[:ci | cornerSel _ #(topLeft topRight bottomRight bottomLeft) at: ci.
			corner _ allowedArea perform: cornerSel.
			"The extra grid//2 in delta helps to keep title tabs distinct"
			delta _ (maxLevel-level*grid+(grid//2)) @ (level*grid).
			1 to: ci-1 do: [:i | delta _ delta rotateBy: #right centerAt: 0@0]. "slow way"
			putativeCorner _ corner + delta.
			free _ true.
			otherFrames do:
				[:w |
				free _ free & ((w perform: cornerSel) ~= putativeCorner)].
			free ifTrue:
				[^ (initialFrame align: (initialFrame perform: cornerSel)
								with: putativeCorner)
						 squishedWithin: allowedArea]]].
	"If all else fails..."
	^ (self scrollBarSetback @ self screenTopSetback extent: initialFrame extent)
		squishedWithin: allowedArea! !


!ResourceCollector methodsFor: 'resource writing' stamp: 'BG 10/30/2003 19:55'!
writeResourceForm: aForm locator: aLocator
	"Store the given form on a file. Return an array with the name and the size of the file"
	| fName fStream fullSize result writerClass |
	aLocator ifNotNil:[
		result _ self writeResourceForm: aForm fromLocator: aLocator.
		result ifNotNil:[^result]
		"else fall through"
	].
	fName _ localDirectory nextNameFor:'resource' extension:'form'.
	fStream _ localDirectory newFileNamed: fName.
	fStream binary.
	"aForm storeResourceOn: fStream."
	writerClass _ ((Smalltalk includesKey: #JPEGReadWriter2)
		and: [(Smalltalk at: #JPEGReadWriter2) new isPluginPresent])
		ifTrue: [(Smalltalk at: #JPEGReadWriter2)]
		ifFalse: ["GIFReadWriter"].
	writerClass putForm: aForm onStream: fStream.
	fStream open.
	fullSize _ fStream size.
	fStream close.

	"Compress contents here"
"		fStream position: 0.
		fStream compressFile.
		localDirectory deleteFileNamed: fName.
		localDirectory rename: fName, FileDirectory dot, 'gz' toBe: fName.
		fStream _ localDirectory readOnlyFileNamed: fName.
		fullSize _ fStream size.
		fStream close.
"
	^{fName. fullSize}! !


!RunArray class methodsFor: 'instance creation' stamp: 'BG 10/29/2003 18:59'!
scanFrom: strm
	"Read the style section of a fileOut or sources file.  nextChunk has already been done.  We need to return a RunArray of TextAttributes of various kinds.  These are written by the implementors of writeScanOn:"
	| rr vv aa this |
	(strm peekFor: $( ) ifFalse: [^ nil].
	rr _ OrderedCollection new.
	[strm skipSeparators.
	 strm peekFor: $)] whileFalse: 
		[rr add: (Number readFrom: strm)].
	vv _ OrderedCollection new.	"Value array"
	aa _ OrderedCollection new.	"Attributes list"
	[(this _ strm next) == nil] whileFalse: [
		this == $, ifTrue: [vv add: aa asArray.  aa _ OrderedCollection new].
		this == $a ifTrue: [aa add: 
			(TextAlignment new alignment: (Number readFrom: strm))].
		this == $f ifTrue: [aa add: 
			(TextFontChange new fontNumber: (Number readFrom: strm))].
		this == $F ifTrue: [aa add: (TextFontReference new toFont: 
			(StrikeFont familyName: (strm upTo: $#) size: (Number readFrom: strm)))].
		this == $b ifTrue: [aa add: (TextEmphasis bold)].
		this == $i ifTrue: [aa add: (TextEmphasis italic)].
		this == $u ifTrue: [aa add: (TextEmphasis underlined)].
		this == $= ifTrue: [aa add: (TextEmphasis struckOut)].
		this == $n ifTrue: [aa add: (TextEmphasis normal)].
		this == $- ifTrue: [aa add: (TextKern kern: -1)].
		this == $+ ifTrue: [aa add: (TextKern kern: 1)].
		this == $c ifTrue: [aa add: (TextColor scanFrom: strm)]. "color"
		this == $L ifTrue: [aa add: (TextLink scanFrom: strm)].	"L not look like 1"
		this == $R ifTrue: [aa add: (TextURL scanFrom: strm)].
				"R capitalized so it can follow a number"
	"	this == $q ifTrue: [aa add: (TextSqkPageLink scanFrom: strm)]. "
	"	this == $p ifTrue: [aa add: (TextSqkProjectLink scanFrom: strm)]. "
		this == $P ifTrue: [aa add: (TextPrintIt scanFrom: strm)].
		this == $d ifTrue: [aa add: (TextDoIt scanFrom: strm)].
		"space, cr do nothing"
		].
	aa size > 0 ifTrue: [vv add: aa asArray].
	^ self runs: rr asArray values: vv asArray
"
RunArray scanFrom: (ReadStream on: '(14 50 312)f1,f1b,f1LInteger +;i')
"! !


!ScreenController methodsFor: 'nested menus' stamp: 'BG 10/30/2003 21:19'!
appearanceMenu 
	"Answer the appearance menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#('system fonts...'
			'full screen on'
			'full screen off'
			'set display depth...'
			'set desktop color...' ) 

		lines: #(1 3)
		selections: #( configureFonts
fullScreenOn fullScreenOff setDisplayDepth setDesktopColor)
"
ScreenController new appearanceMenu startUp
"! !

!ScreenController methodsFor: 'nested menus' stamp: 'BG 10/30/2003 20:39'!
changesMenu
	"Answer a menu for changes-related items"
	^ SelectionMenu labelList:
		#(	'simple change sorter'
			'dual change sorter'

			'file out current change set'
			'create new change set...'
			'browse changed methods'
			'check change set for slips'

		
			'browse recent submissions'
			'recently logged changes...'
			'recent log file...'
			)
		lines: #(2 6 )
		selections: #(
openSimpleChangeSorter openChangeManager
fileOutChanges newChangeSet browseChangedMessages lookForSlips
browseRecentSubmissions browseRecentLog fileForRecentLog)
"
ScreenController new changesMenu startUp
"! !

!ScreenController methodsFor: 'nested menus' stamp: 'BG 11/2/2003 20:57'!
helpMenu 
	"Answer the help menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'about this system...'
			'preferences...'

			'command-key help'
			'font size summary'
			'useful expressions'
			'view graphical imports'),

			(Array with: (Preferences soundsEnabled
							ifFalse: ['turn sound on']
							ifTrue: ['turn sound off'])) ,

		#(	'definition for...'
			'set author initials...'
			'vm statistics'
			'space left')
		lines: #(2 4 8)
		selections: #( aboutThisSystem 
editPreferences  openCommandKeyHelp fontSizeSummary openStandardWorkspace viewImageImports
 soundOnOrOff  lookUpDefinition setAuthorInitials vmStatistics garbageCollect)
"
ScreenController new helpMenu startUp
"! !

!ScreenController methodsFor: 'nested menus' stamp: 'BG 10/28/2003 10:54'!
openMenu
	^ SelectionMenu labelList:
		#(	'browser'
			'package browser'
			'method finder'
			'workspace'
			'file list'
			'file...'
			'transcript'

			'simple change sorter'
			'dual change sorter'

			'mvc project'
			)
		lines: #(7 9 )
		selections: #(
openBrowser openPackageBrowser openSelectorBrowser openWorkspace openFileList openFile openTranscript 
openSimpleChangeSorter openChangeManager
openProject    )
"
ScreenController  new openMenu startUp
"! !

!ScreenController methodsFor: 'nested menus' stamp: 'BG 10/28/2003 10:38'!
projectScreenMenu
	"Answer the project screen menu."

	^ SelectionMenu labelList:
		#(	'previous project'
			'jump to project...'
			'restore display'

			'open...'
			'windows...'
			'changes...'
			'help...'
			'appearance...'
			'do...'

			'save'
			'save as...'
			'save and quit'
			'quit')
		lines: #(3 9)
		selections: #(
returnToPreviousProject jumpToProject restoreDisplay
presentOpenMenu presentWindowMenu presentChangesMenu presentHelpMenu presentAppearanceMenu commonRequests
snapshot saveAs snapshotAndQuit quit )
"
ScreenController new projectScreenMenu startUp
"! !

!ScreenController methodsFor: 'nested menus' stamp: 'BG 10/28/2003 10:57'!
windowMenu
	"Answer a menu for windows-related items.  "

	^ SelectionMenu labelList:
		#(	'find window...'
			'find changed browsers...'
			'find changed windows...'

			'collapse all windows'
			'expand all windows'
			'close unchanged windows' ) , 
			(Array
				with: self bitCachingString
				with: self staggerPolicyString)
		lines: #(3 6)
		selections: #(
findWindow chooseDirtyBrowser chooseDirtyWindow
collapseAll expandAll closeUnchangedWindows
fastWindows changeWindowPolicy)
"
ScreenController new windowMenu startUp
"! !


!SmartRefStream methodsFor: 'read write' stamp: 'BG 10/30/2003 21:41'!
restoreClassInstVars
	"Install the values of the class instance variables of UniClasses
(i.e. scripts slotInfo).  classInstVars is ((#Player25 scripts slotInfo)
...).  Thank you Mark Wai for the bug fix."

	| normal aName newName newCls trans rList start |

	self flag: #bobconv.	


	self moreObjects ifFalse: [^ self]. 	"are no UniClasses with class inst vars"
	classInstVars _ super next.	"Array of arrays"
	normal _ Object class instSize.	"might give trouble if Player class superclass changes size"
	(structures at: #Player ifAbsent: [#()]) = #(0 'dependents' 'costume') ifTrue:
		[trans _ 1].	"now (0 costume costumes).  Do the conversion of Player class
			inst vars in Update 509."
	classInstVars do: [:list |
		aName _ (list at: 1) asSymbol.
		rList _ list.
		newName _ renamed at: aName ifAbsent: [aName].
		newCls _ Smalltalk at: newName
				ifAbsent: [self error: 'UniClass definition missing'].
			"costumeDictionary's value"
		start _ list second = 'Update to read classPool' ifTrue: [4] ifFalse: [2].
		newCls class instSize = (normal + (rList size) - start + 1) ifFalse:
			[self error: 'UniClass superclass class has changed size'].
			"Need to install a conversion method mechanism"
		start = 4 ifTrue: [newCls instVarAt: normal - 1 "classPool" put: (list at: 3)].
		start to: rList size do: [:ii |
			newCls instVarAt: normal + ii - start + 1 put: (rList at: ii)]].
! !


!StandardSystemView methodsFor: 'framing' stamp: 'BG 10/29/2003 09:46'!
collapse
	"If the receiver is not already collapsed, change its view to be that of its 
	label only."

	self isCollapsed ifFalse:
			[model modelSleep.
			(subViews ~~ nil and: [subViews size = 1 ])
				ifTrue: [subViews first deEmphasizeView].
			expandedViewport _ self viewport.
			savedSubViews _ subViews.
			self resetSubViews.
			labelText isNil ifTrue: [self label: nil.  bitsValid _ false.].
			self window: (self inverseDisplayTransform:
					((self labelDisplayBox topLeft extent: (labelText extent x + 70) @ self labelHeight)
						 intersect: self labelDisplayBox))]! !


!String methodsFor: 'internet' stamp: 'BG 10/29/2003 08:20'!
replaceHtmlCharRefs

        | pos ampIndex scIndex special specialValue outString outPos newOutPos |

        outString _ String new: self size.
        outPos _ 0.

        pos _ 1.
        
        [ pos <= self size ] whileTrue: [ 
                "read up to the next ampersand"
                ampIndex _ self indexOf: $& startingAt: pos ifAbsent: [0].
                
                ampIndex = 0 ifTrue: [
                        pos = 1 ifTrue: [ ^self ] ifFalse: [ ampIndex _ self size+1 ] ].

                newOutPos _ outPos + ampIndex - pos.
                outString
                        replaceFrom: outPos + 1
                        to: newOutPos
                        with: self
                        startingAt: pos.
                outPos _ newOutPos.
                pos _ ampIndex.

                ampIndex <= self size ifTrue: [
                        "find the $;"
                        scIndex _ self indexOf: $; startingAt: ampIndex ifAbsent: [ self size + 1 ].

                        special _ self copyFrom: ampIndex+1 to: scIndex-1.       
                        specialValue _ "HtmlEntity valueOfHtmlEntity: special" 16r32. 

                        specialValue
                                ifNil: [
                                        "not a recognized entity.  wite it back"
								  scIndex > self size ifTrue: [ scIndex _ self size ].

                                        newOutPos _ outPos + scIndex - ampIndex + 1.
                                        outString
                                                replaceFrom: outPos+1
                                                to: newOutPos
                                                with: self
                                                startingAt: ampIndex.
                                        outPos _ newOutPos.]
                                ifNotNil: [
                                        outPos _ outPos + 1.
                                        outString at: outPos put: specialValue isoToSqueak.].
                        
                        pos _ scIndex + 1. ]. ].


        ^outString copyFrom: 1 to: outPos! !


!StringHolder methodsFor: 'initialize-release' stamp: 'BG 10/28/2003 20:43'!
openLabel: aString andTerminate: terminateBoolean
	"Create a standard system view of the model, me, a StringHolder and open it.; do not terminate the active process if in mvc"
	| topView codeView |

	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
	topView label: aString.
	topView minimumSize: 100 @ 50.

	codeView _ PluggableTextView on: self 
			text: #contents accept: #acceptContents:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	codeView window: (0@0 extent: 200@200).
	topView addSubView: codeView.
	"self contents size > 0 ifTrue: [
			codeView hasUnacceptedEdits: true].  Is it already saved or not??"
	terminateBoolean
		ifTrue:
			[topView controller open]
		ifFalse:
			[topView controller openNoTerminate]! !

!StringHolder methodsFor: 'message list menu' stamp: 'BG 10/29/2003 09:40'!
offerMenuFrom: menuRetriever shifted: aBoolean
	"Pop up, in morphic or mvc as the case may be, a menu whose target is the receiver and whose contents are provided by sending the menuRetriever to the receiver.  The menuRetriever takes two arguments: a menu, and a boolean representing the shift state."

	| aMenu |
	Smalltalk isMorphic
		ifTrue:
			[]
		ifFalse:
			[aMenu _ CustomMenu new.
			self perform: menuRetriever with: aMenu with: aBoolean.
			aMenu invokeOn: self]! !


!CodeHolder methodsFor: 'commands' stamp: 'BG 10/28/2003 20:51'!
copyUpOrCopyDown
	"Used to copy down code from a superclass to a subclass or vice-versa in one easy step, if you know what you're doing.  Prompt the user for which class to copy down or copy up to, then spawn a fresh browser for that class, with the existing code planted in it, and with the existing method category also established."

	| |
	Smalltalk isMorphic ifFalse: [^ self inform: 
'Sorry, for the moment you have to be in
Morphic to use this feature.'].! !

!CodeHolder methodsFor: 'commands' stamp: 'BG 10/28/2003 20:51'!
makeSampleInstance
	| aClass nonMetaClass anInstance |
	(aClass _ self selectedClassOrMetaClass) ifNil: [^ self].
	nonMetaClass _ aClass theNonMetaClass.
	anInstance _ self sampleInstanceOfSelectedClass.
	(anInstance isNil and: [nonMetaClass ~~ UndefinedObject]) ifTrue: 
		[^ self inform: 'Sorry, cannot make an instance of ', nonMetaClass name].

	(Smalltalk isMorphic and: [anInstance isMorph])
		ifTrue:
			[]
		ifFalse:
			[anInstance inspectWithLabel: 'An instance of ', nonMetaClass name]! !

!CodeHolder methodsFor: 'commands' stamp: 'BG 10/28/2003 20:52'!
spawnHierarchy
	"Create and schedule a new hierarchy browser on the currently selected class or meta."

	| newBrowser aSymbol aBehavior messageCatIndex selectedClassOrMetaClass |
	(selectedClassOrMetaClass _ self selectedClassOrMetaClass)
		ifNil: [^ self].
	newBrowser _ HierarchyBrowser new initHierarchyForClass: selectedClassOrMetaClass.
	((aSymbol _ self selectedMessageName) notNil and: [(MessageSet isPseudoSelector: aSymbol) not])
		ifTrue:
			[aBehavior _ selectedClassOrMetaClass.
			messageCatIndex _ aBehavior organization numberOfCategoryOfElement: aSymbol.
			newBrowser messageCategoryListIndex: messageCatIndex + 1.
			newBrowser messageListIndex:
				((aBehavior organization listAtCategoryNumber: messageCatIndex) indexOf: aSymbol)].
	Browser
		openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: newBrowser labelString.
	! !

!CodeHolder methodsFor: 'controls' stamp: 'BG 10/29/2003 09:43'!
buttonWithSelector: aSelector
	"If receiver has a control button with the given action selector answer it, else answer nil.  morphic only at this point"

	^nil! !

!CodeHolder methodsFor: 'diffs' stamp: 'BG 10/29/2003 07:56'!
restoreTextualCodingPane
	"If the receiver is showing tiles, restore the textual coding pane"

	contentsSymbol _ #source.! !

!CodeHolder methodsFor: 'misc' stamp: 'BG 11/1/2003 13:26'!
refreshAnnotation
	"If the receiver has an annotation pane that does not bear unaccepted edits, refresh it"

	(self dependents detect: [:m | (m inheritsFromAnyIn: #('PluggableTextView')) and: [m getTextSelector == #annotation]] ifNone: [nil]) ifNotNilDo:
		[:aPane | aPane hasUnacceptedEdits ifFalse:
			[aPane update: #annotation]]! !


!Browser methodsFor: 'accessing' stamp: 'BG 10/29/2003 09:01'!
couldBrowseAnyClass
	"Answer whether the receiver is equipped to browse any class. This is in support of the system-brower feature that allows the browser to be redirected at the selected class name.  This implementation is clearly ugly, but the feature it enables is handsome enough.  3/1/96 sw"

	self dependents
		detect: [:d |
			((d isKindOf: PluggableListView)) and: 
			[d getListSelector == #systemCategoryList]]
		ifNone: [^ false].
	^ true
! !

!Browser methodsFor: 'class functions' stamp: 'BG 11/1/2003 13:59'!
shiftedClassListMenu: aMenu
	"Set up the menu to apply to the receiver's class list when the shift key is down"

	^ aMenu addList: #(
			-
			('unsent methods'			browseUnusedMethods	'browse all methods defined by this class that have no senders')
			('unreferenced inst vars'	showUnreferencedInstVars	'show a list of all instance variables that are not referenced in methods')
			('unreferenced class vars'	showUnreferencedClassVars	'show a list of all class variables that are not referenced in methods')
			('subclass template'			makeNewSubclass		'put a template into the code pane for defining of a subclass of this class')
			-
			('sample instance'			makeSampleInstance		'give me a sample instance of this class, if possible')
			('inspect instances'			inspectInstances			'open an inspector on all the extant instances of this class')
			('inspect subinstances'		inspectSubInstances		'open an inspector on all the extant instances of this class and of all of its subclasses')
			-
			
			('add all meths to current chgs'		addAllMethodsToCurrentChangeSet
																'place all the methods defined by this class into the current change set')
			('create inst var accessors'	createInstVarAccessors	'compile instance-variable access methods for any instance variables that do not yet have them')
			-
			('more...'					offerUnshiftedClassListMenu	'return to the standard class-list menu'))! !

!Browser methodsFor: 'initialize-release' stamp: 'BG 10/30/2003 21:09'!
openEditString: aString
        "Create a pluggable version of all the views for a Browser, including views and controllers."
        | systemCategoryListView classListView 
        messageCategoryListView messageListView browserCodeView topView switchView underPane y optionalButtonsView annotationPane |

        topView _ StandardSystemView new model: self.
        topView borderWidth: 1. "label and minSize taken care of by caller"

        systemCategoryListView _ PluggableListView on: self
                list: #systemCategoryList
                selected: #systemCategoryListIndex
                changeSelected: #systemCategoryListIndex:
                menu: #systemCategoryMenu:
                keystroke: #systemCatListKey:from:.
        systemCategoryListView window: (0 @ 0 extent: 50 @ 70).
        topView addSubView: systemCategoryListView.

        classListView _ PluggableListView on: self
                list: #classList
                selected: #classListIndex
                changeSelected: #classListIndex:
                menu: #classListMenu:shifted:
                keystroke: #classListKey:from:.
        classListView window: (0 @ 0 extent: 50 @ 62).
        topView addSubView: classListView toRightOf: systemCategoryListView.

        switchView _ self buildInstanceClassSwitchView.
        switchView borderWidth: 1.
        topView addSubView: switchView below: classListView.

        messageCategoryListView _ PluggableListView on: self
                list: #messageCategoryList
                selected: #messageCategoryListIndex
                changeSelected: #messageCategoryListIndex:
                menu: #messageCategoryMenu:. 
        messageCategoryListView controller terminateDuringSelect: true.
        messageCategoryListView window: (0 @ 0 extent: 50 @ 70).
        topView addSubView: messageCategoryListView toRightOf: classListView.

        messageListView _ PluggableListView on: self
                list: #messageList
                selected: #messageListIndex
                changeSelected: #messageListIndex:
                menu: #messageListMenu:shifted:
                keystroke: #messageListKey:from:.
        messageListView window: (0 @ 0 extent: 50 @ 70).
        messageListView menuTitleSelector: #messageListSelectorTitle.
        topView addSubView: messageListView toRightOf: messageCategoryListView.

       self wantsAnnotationPane
                ifTrue:
                        [annotationPane _ PluggableTextView on: self
                                text: #annotation accept: nil
                                readSelection: nil menu: nil.
                        annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
                        topView addSubView: annotationPane below: systemCategoryListView.
                        underPane _ annotationPane.
                        y _ 110 - self optionalAnnotationHeight]
                ifFalse: [
                        underPane _ systemCategoryListView.
                        y _ 110].

        self wantsOptionalButtons ifTrue:
                [optionalButtonsView _ self buildOptionalButtonsView.
                optionalButtonsView borderWidth: 1.
                topView addSubView: optionalButtonsView below: underPane.
                underPane _ optionalButtonsView.
                y _ y - self optionalButtonHeight].

        browserCodeView _ PluggableTextView on: self 
                        text: #contents accept: #contents:notifying:
                        readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
        browserCodeView window: (0@0 extent: 200@y).
        topView addSubView: browserCodeView below: underPane.
        aString ifNotNil: [browserCodeView editString: aString.
                        browserCodeView hasUnacceptedEdits: true].
        topView setUpdatablePanesFrom: #(systemCategoryList classList messageCategoryList messageList).

        ^ topView! !

!Browser methodsFor: 'initialize-release' stamp: 'BG 10/30/2003 21:09'!
openMessageCatEditString: aString
        "Create a pluggable version of the views for a Browser that just shows one message category."
        | messageCategoryListView messageListView browserCodeView topView annotationPane underPane y optionalButtonsView |

        topView _ (StandardSystemView new) model: self.
        topView borderWidth: 1.
                "label and minSize taken care of by caller"

        messageCategoryListView _ PluggableListView on: self
                list: #messageCatListSingleton
                selected: #indexIsOne 
                changeSelected: #indexIsOne:
                menu: #messageCategoryMenu:.
        messageCategoryListView window: (0 @ 0 extent: 200 @ 12).
        topView addSubView: messageCategoryListView.

        messageListView _ PluggableListView on: self
                list: #messageList
                selected: #messageListIndex
                changeSelected: #messageListIndex:
                menu: #messageListMenu:shifted:
                keystroke: #messageListKey:from:.
        messageListView menuTitleSelector: #messageListSelectorTitle.
        messageListView window: (0 @ 0 extent: 200 @ 70).
        topView addSubView: messageListView below: messageCategoryListView.

        self wantsAnnotationPane
                ifTrue:
                        [annotationPane _ PluggableTextView on: self
                                text: #annotation accept: nil
                                readSelection: nil menu: nil.
                        annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
                        topView addSubView: annotationPane below: messageListView.
                        underPane _ annotationPane.
                        y _ (200 - 12 - 70) - self optionalAnnotationHeight]
                ifFalse:
                        [underPane _ messageListView.
                        y _ (200 - 12 - 70)].

        self wantsOptionalButtons ifTrue:
                [optionalButtonsView _ self buildOptionalButtonsView.
                optionalButtonsView borderWidth: 1.
                topView addSubView: optionalButtonsView below: underPane.
                underPane _ optionalButtonsView.
                y _ y - self optionalButtonHeight].

        browserCodeView _ PluggableTextView on: self 
                        text: #contents accept: #contents:notifying:
                        readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
        browserCodeView window: (0@0 extent: 200@y).
        topView addSubView: browserCodeView below: underPane.
        aString ifNotNil: [browserCodeView editString: aString.
                        browserCodeView hasUnacceptedEdits: true].
        topView setUpdatablePanesFrom: #(messageCatListSingleton messageList).
        ^ topView! !

!Browser methodsFor: 'initialize-release' stamp: 'BG 10/28/2003 20:45'!
openMessageEditString: aString
	"Create a pluggable version of the views for a Browser that just shows one message."
	| messageListView browserCodeView topView annotationPane underPane y |


	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	messageListView _ PluggableListView on: self
		list: #messageListSingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #messageListMenu:shifted:.
	messageListView window: (0 @ 0 extent: 200 @ 12).
	topView addSubView: messageListView.

	 self wantsAnnotationPane
		ifTrue:
			[annotationPane _ PluggableTextView on: self
				text: #annotation accept: nil
				readSelection: nil menu: nil.
			annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
			topView addSubView: annotationPane below: messageListView.
			underPane _ annotationPane.
			y _ (200 - 12) - self optionalAnnotationHeight]
		ifFalse:
			[underPane _ messageListView.
			y _ 200 - 12].

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@y).
	topView addSubView: browserCodeView below: underPane.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView! !

!Browser methodsFor: 'initialize-release' stamp: 'BG 10/28/2003 20:45'!
openOnClassWithEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	| classListView messageCategoryListView messageListView browserCodeView topView switchView annotationPane underPane y optionalButtonsView |


	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	classListView _ PluggableListView on: self
		list: #classListSingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #classListMenu:shifted:
		keystroke: #classListKey:from:.
	classListView window: (0 @ 0 extent: 100 @ 12).
	topView addSubView: classListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 100 @ 70).
	topView addSubView: messageCategoryListView below: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	messageListView window: (0 @ 0 extent: 100 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	switchView _ self buildInstanceClassSwitchView.
	switchView borderWidth: 1.
	switchView 
		window: switchView window 
		viewport: (classListView viewport topRight 
					corner: messageListView viewport topRight).
	topView addSubView: switchView toRightOf: classListView.

	 self wantsAnnotationPane
		ifTrue:
			[annotationPane _ PluggableTextView on: self
				text: #annotation accept: nil
				readSelection: nil menu: nil.
			annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
			topView addSubView: annotationPane below: messageCategoryListView.
			underPane _ annotationPane.
			y _ (200-12-70) - self optionalAnnotationHeight]
		ifFalse:
			[underPane _ messageCategoryListView.
			y _ (200-12-70)].

	self wantsOptionalButtons ifTrue:
		[optionalButtonsView _ self buildOptionalButtonsView.
		optionalButtonsView borderWidth: 1.
		topView addSubView: optionalButtonsView below: underPane.
		underPane _ optionalButtonsView.
		y _ y - self optionalButtonHeight].

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@y).
	topView addSubView: browserCodeView below: underPane.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].

	topView setUpdatablePanesFrom: #(messageCategoryList messageList).
	^ topView! !

!Browser methodsFor: 'initialize-release' stamp: 'BG 10/28/2003 20:45'!
openSystemCatEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."
	| systemCategoryListView classListView messageCategoryListView messageListView browserCodeView topView switchView y annotationPane underPane optionalButtonsView |


	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	systemCategoryListView _ PluggableListView on: self
		list: #systemCategorySingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #systemCatSingletonMenu:
		keystroke: #systemCatSingletonKey:from:.
	systemCategoryListView window: (0 @ 0 extent: 200 @ 12).
	topView addSubView: systemCategoryListView.

	classListView _ PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:shifted:
		keystroke: #classListKey:from:.
	classListView window: (0 @ 0 extent: 67 @ 62).
	topView addSubView: classListView below: systemCategoryListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView controller terminateDuringSelect: true.
	messageCategoryListView window: (0 @ 0 extent: 66 @ 70).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	switchView _ self buildInstanceClassSwitchView.
	switchView 
		window: switchView window 
		viewport: (classListView viewport bottomLeft 
					corner: messageCategoryListView viewport bottomLeft).
	switchView borderWidth: 1.
	topView addSubView: switchView below: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	messageListView window: (0 @ 0 extent: 67 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	 self wantsAnnotationPane
		ifTrue:
			[annotationPane _ PluggableTextView on: self
				text: #annotation accept: nil
				readSelection: nil menu: nil.
			annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
			topView addSubView: annotationPane below: switchView.
			y _ 110 - 12 - self optionalAnnotationHeight.
			underPane _ annotationPane]
		ifFalse:
			[y _ 110 - 12.
			underPane _ switchView].

	self wantsOptionalButtons ifTrue:
		[optionalButtonsView _ self buildOptionalButtonsView.
		optionalButtonsView borderWidth: 1.
		topView addSubView: optionalButtonsView below: underPane.
		underPane _ optionalButtonsView.
		y _ y - self optionalButtonHeight].

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@y).
	topView addSubView: browserCodeView below: underPane.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	topView setUpdatablePanesFrom: #(classList messageCategoryList messageList).
	^ topView! !

!Browser methodsFor: 'message functions' stamp: 'BG 11/1/2003 13:27'!
messageListMenu: aMenu shifted: shifted
	"Answer the message-list menu"

	shifted ifTrue: [^ self shiftedMessageListMenu: aMenu].
	aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('browse hierarchy (h)'					classHierarchy)
			('browse method (O)'					openSingleMessageBrowser)
			('browse protocol (p)'					browseFullProtocol)
			-
			('fileOut (o)'							fileOutMessage)
			('printOut'								printOutMessage)
			('copy selector (c)'						copySelector)
			-
			('senders of... (n)'						browseSendersOfMessages)
			('implementors of... (m)'					browseMessages)
			('inheritance (i)'						methodHierarchy)
			('versions (v)'							browseVersions)
			-
			('inst var refs...'						browseInstVarRefs)
			('inst var defs...'						browseInstVarDefs)
			('class var refs...'						browseClassVarRefs)
			('class variables'						browseClassVariables)
			('class refs (N)'							browseClassRefs)
			-
			('remove method (x)'					removeMessage)
			-
			('more...'								shiftedYellowButtonActivity)).
	^ aMenu! !

!Browser methodsFor: 'message functions' stamp: 'BG 11/1/2003 13:58'!
shiftedMessageListMenu: aMenu
	"Fill aMenu with the items appropriate when the shift key is held down"

	aMenu addList: #(
		('toggle diffing (D)'						toggleDiffing)
		('implementors of sent messages'			browseAllMessages)
		-
		('local senders of...'						browseLocalSendersOfMessages)
		('local implementors of...'				browseLocalImplementors)
		-
		('spawn sub-protocol'					spawnProtocol)
		('spawn full protocol'					spawnFullProtocol)
		-
		('sample instance'						makeSampleInstance)
		('inspect instances'						inspectInstances)
		('inspect subinstances'					inspectSubInstances)).

	self addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(
		-
		('change category...'					changeCategory)).

	self canShowMultipleMessageCategories ifTrue: [aMenu addList:
		 #(('show category (C)'						showHomeCategory))].
	aMenu addList: #(
		-
		('change sets with this method'			findMethodInChangeSets)
		('revert to previous version'				revertToPreviousVersion)
		('remove from current change set'		removeFromCurrentChanges)
		('revert & remove from changes'		revertAndForget)
		('add to current change set'				adoptMessageInCurrentChangeset)
		('copy up or copy down...'				copyUpOrCopyDown)
		-
		('more...' 								unshiftedYellowButtonActivity)).
	^ aMenu
! !


!ChangeList methodsFor: 'menu actions' stamp: 'BG 10/28/2003 20:46'!
changeListMenu: aMenu
	"Fill aMenu up so that it comprises the primary changelist-browser menu"

	aMenu addList: #(

	('fileIn selections'							fileInSelections						'import the selected items into the image')
	('fileOut selections...	'						fileOutSelections						'create a new file containing the selected items')
	-
	('compare to current'						compareToCurrentVersion			'open a separate window which shows the text differences between the on-file version and the in-image version.' )
	('toggle diffing (D)'							toggleDiffing						'start or stop showing diffs in the code pane.')
	-
	('select conflicts with any changeset'		selectAllConflicts					'select methods in the file which also occur in any change-set in the system')
	('select conflicts with current changeset'	selectConflicts						'select methods in the file which also occur in the current change-set')
	('select conflicts with...'						selectConflictsWith					'allows you to designate a file or change-set against which to check for code conflicts.')
	-
	('select unchanged methods'					selectUnchangedMethods				'select methods in the file whose in-image versions are the same as their in-file counterparts' )
	('select new methods'						selectNewMethods					'select methods in the file that do not current occur in the image')
	('select methods for this class'				selectMethodsForThisClass			'select all methods in the file that belong to the currently-selected class')

	-
	('select all (a)'								selectAll								'select all the items in the list')
	('deselect all'								deselectAll							'deselect all the items in the list')
	('invert selections'							invertSelections						'select every item that is not currently selected, and deselect every item that *is* currently selected')
	-
	('browse current versions of selections'		browseCurrentVersionsOfSelections	'open a message-list browser showing the current (in-image) counterparts of the selected methods')
	('destroy current methods of selections'		destroyCurrentCodeOfSelections		'remove (*destroy*) the in-image counterparts of all selected methods')
	-
	('remove doIts'								removeDoIts							'remove all items that are doIts rather than methods')
	('remove older versions'						removeOlderMethodVersions			'remove all but the most recent versions of methods in the list')
	('remove up-to-date versions'				removeExistingMethodVersions		'remove all items whose code is the same as the counterpart in-image code')
	('remove selected items'						removeSelections					'remove the selected items from the change-list')
	('remove unselected items'					removeNonSelections					'remove all the items not currently selected from the change-list')).

	^ aMenu

! !


!ChangeSorter methodsFor: 'creation' stamp: 'BG 10/28/2003 20:49'!
open
	"ChangeSorterPluggable new open"
	| topView |
	
	topView _ StandardSystemView new.
	topView model: self.
	myChangeSet ifNil: [self myChangeSet: Smalltalk changes]. 
	topView label: self labelString.
	topView borderWidth: 1; minimumSize: 360@360.
	self openView: topView offsetBy: 0@0.
	topView controller open.
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/29/2003 08:09'!
changeSetMenu: aMenu shifted: isShifted 
	"Set up aMenu to hold commands for the change-set-list pane.  This could be for a single or double changeSorter"

	isShifted ifTrue: [^ self shiftedChangeSetMenu: aMenu].
	Smalltalk isMorphic
		ifTrue:
			[]
		ifFalse:
			[aMenu title: 'Change Set:
' , myChangeSet name].

	aMenu add: 'make changes go to me (m)' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'new change set... (n)' action: #newSet.
	aMenu add: 'find...(f)' action: #findCngSet.
	aMenu add: 'show category... (s)' action:  #chooseChangeSetCategory.
	aMenu balloonTextForLastItem:
'Lets you choose which change sets should be listed in this change sorter'.
	aMenu add: 'select change set...' action: #chooseCngSet.
	aMenu addLine.
	aMenu add: 'rename change set (r)' action: #rename.
	aMenu add: 'file out (o)' action: #fileOut.
"	aMenu add: 'mail to list' action: #mailOut. "
	aMenu add: 'browse methods (b)' action: #browseChangeSet.
	aMenu add: 'browse change set (B)' action: #openChangeSetBrowser.
	aMenu addLine.
	parent
		ifNotNil: 
			[aMenu add: 'copy all to other side (c)' action: #copyAllToOther.
			aMenu add: 'submerge into other side' action: #submergeIntoOtherSide.
			aMenu add: 'subtract other side (-)' action: #subtractOtherSide.
			aMenu addLine].
	myChangeSet hasPreamble
		ifTrue: 
			[aMenu add: 'edit preamble (p)' action: #addPreamble.
			aMenu add: 'remove preamble' action: #removePreamble]
		ifFalse: [aMenu add: 'add preamble (p)' action: #addPreamble].
	myChangeSet hasPostscript
		ifTrue: 
			[aMenu add: 'edit postscript...' action: #editPostscript.
			aMenu add: 'remove postscript' action: #removePostscript]
		ifFalse: [aMenu add: 'add postscript...' action: #editPostscript].
	aMenu addLine.

	aMenu add: 'category functions...' action: #offerCategorySubmenu.
	aMenu balloonTextForLastItem:
'Various commands relating to change-set-categories'.
	aMenu addLine.


	aMenu add: 'destroy change set (x)' action: #remove.
	aMenu addLine.
	aMenu add: 'more...' action: #offerShiftedChangeSetMenu.
	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/28/2003 20:47'!
chooseChangeSetCategory
	"Present the user with a list of change-set-categories and let her choose one"

	|  cats aMenu result |
	self okToChange ifFalse: [^ self].
	

	cats _ ChangeSetCategories elementsInOrder.
	aMenu _ SelectionMenu
		labels: (cats collect: [:cat | cat categoryName])
		selections: cats.
	result _ aMenu startUp.
	result ifNotNil:
		[changeSetCategory _ result.
		self changed: #changeSetList.
		(self changeSetList includes: myChangeSet name) ifFalse:
			[self showChangeSet: (ChangeSorter changeSetNamed: self changeSetList first)].
		self changed: #relabel]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/29/2003 00:38'!
openChangeSetBrowser
	"Open a ChangeSet browser on the current change set"

	Smalltalk isMorphic
		ifFalse:
			[self browseChangeSet]  "msg-list browser only"
		ifTrue:
			[]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/28/2003 20:37'!
setRecentUpdatesMarker
	"Allow the user to change the recent-updates marker"

	| result |
	result _ FillInTheBlank request: 
('Enter the lowest change-set number
that you wish to consider "recent"?
(note: highest change-set number
in this image at this time is ', self class highestNumberedChangeSet asString, ')') initialAnswer: self class recentUpdateMarker asString.
	(result notNil and: [result startsWithDigit]) ifTrue:
		[self class recentUpdateMarker: result asInteger.]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/28/2003 20:48'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.
	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' action: #methodConflictsWithOtherSide.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'.

			aMenu add: 'conflicts with category opposite' action: #methodConflictsWithOppositeCategory.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in ANY change set in the category list on the opposite side of this change sorter, other of course than this change set itself.  (Caution -- this could be VERY slow)'].
	aMenu addLine.

	"CHECKS SECTION"
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.

	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'.

	aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
	aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Utilities authorInitials, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'go to change set''s project' action: #goToChangeSetsProject.
	aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'.

	aMenu add: 'promote to top of list' action: #promoteToTopChangeSet.
	aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.
	aMenu add: 'expunge uniclasses' action: #expungeUniclasses.
	aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'.

	aMenu add: 'uninstall this change set' action: #uninstallChangeSet.
	aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'.

	aMenu addLine.
	aMenu add: 'file into new...' action: #fileIntoNewChangeSet.
	aMenu balloonTextForLastItem: 
'Load a fileout from disk and place its changes into a new change set (seldom needed -- much better to do this from a file-list browser these days.)'.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'.

	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'BG 10/28/2003 20:51'!
showCategoriesOfChangeSet
	"Show a list of all the categories in which the selected change-set occurs at the moment.  Install the one the user chooses, if any."

	Smalltalk isMorphic
		ifFalse:
			[self inform:
'Only available in morphic, right now, sorry.
It would not take much to make this
also work in mvc, so if you are
inclined to do that, thanks in advance...']
		ifTrue:
			[]! !

!ChangeSorter methodsFor: 'class list' stamp: 'BG 10/28/2003 20:48'!
classListMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the class list"

	aMenu title: 'class list'.
	(parent notNil and: [shifted not])
		ifTrue: [aMenu addList: #( "These two only apply to dual change sorters"
			('copy class chgs to other side'			copyClassToOther)	
			('move class chgs to other side'			moveClassToOther))].

	aMenu addList: (shifted
		ifFalse: [#(
			-
			('delete class from change set (d)'		forgetClass)
			('remove class from system (x)'			removeClass)
			-
			('browse full (b)'						browseMethodFull)
			('browse hierarchy (h)'					spawnHierarchy)
			('browse protocol (p)'					browseFullProtocol)
			-
			('printOut'								printOutClass)
			('fileOut'								fileOutClass)
			-
			('inst var refs...'						browseInstVarRefs)
			('inst var defs...'						browseInstVarDefs)
			('class var refs...'						browseClassVarRefs)
			('class vars'								browseClassVariables)
			('class refs (N)'							browseClassRefs)
			-
			('more...'								offerShiftedClassListMenu))]

		ifTrue: [#(
			-
			('unsent methods'						browseUnusedMethods)
			('unreferenced inst vars'				showUnreferencedInstVars)
			('unreferenced class vars'				showUnreferencedClassVars)
			-
			('sample instance'						makeSampleInstance)
			('inspect instances'						inspectInstances)
			('inspect subinstances'					inspectSubInstances)
			-
			('more...'								offerUnshiftedClassListMenu ))]).
	^ aMenu! !

!ChangeSorter methodsFor: 'message list' stamp: 'BG 10/28/2003 20:48'!
messageMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	shifted ifTrue: [^ self shiftedMessageMenu: aMenu].

	aMenu title: 'message list'.
	
	parent ifNotNil:
		[aMenu addList: #(
			('copy method to other side'			copyMethodToOther)
			('move method to other side'			moveMethodToOther))].

	aMenu addList: #(
			('delete method from changeSet (d)'	forget)
			-
			('remove method from system (x)'	removeMessage)
				-
			('browse full (b)'					browseMethodFull)
			('browse hierarchy (h)'				spawnHierarchy)
			('browse method (O)'				openSingleMessageBrowser)
			('browse protocol (p)'				browseFullProtocol)
			-
			('fileOut'							fileOutMessage)
			('printOut'							printOutMessage)
			-
			('senders of... (n)'					browseSendersOfMessages)
			('implementors of... (m)'				browseMessages)
			('inheritance (i)'					methodHierarchy)
			('versions (v)'						browseVersions)
			-
			('more...'							shiftedYellowButtonActivity)).
	^ aMenu
! !

!ChangeSorter methodsFor: 'message list' stamp: 'BG 11/1/2003 13:24'!
shiftedMessageMenu: aMenu
	"Arm the menu so that it holds items appropriate to the message-list while the shift key is down.  Answer the menu."

	^ aMenu addList: #(
		-
		('toggle diffing (D)'					toggleDiffing)
		('implementors of sent messages'		browseAllMessages)
		('change category...'				changeCategory)
			-
		('sample instance'					makeSampleInstance)
		('inspect instances'					inspectInstances)
		('inspect subinstances'				inspectSubInstances)
		-
		('change sets with this method'		findMethodInChangeSets)
		('revert to previous version'			revertToPreviousVersion)
		('revert & remove from changes'	revertAndForget)
		-
		('more...'							unshiftedYellowButtonActivity))! !


!ChangeSetBrowser methodsFor: 'initialization' stamp: 'BG 10/28/2003 20:47'!
addModelItemsToWindowMenu: aMenu
	"Add model-related items to the given window menu"

	| oldTarget |
	oldTarget _ aMenu defaultTarget.
	aMenu defaultTarget: self.
	aMenu addLine.
	aMenu add: 'rename change set' action: #rename.
	aMenu add: 'make changes go to me' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'file out' action: #fileOut.
	aMenu add: 'browse methods' action: #browseChangeSet.
	aMenu addLine.
	myChangeSet hasPreamble
		ifTrue:
			[aMenu add: 'edit preamble' action: #addPreamble.
			aMenu add: 'remove preamble' action: #removePreamble]
		ifFalse:
			[aMenu add: 'add preamble' action: #addPreamble].

	myChangeSet hasPostscript
		ifTrue:
			[aMenu add: 'edit postscript...' action: #editPostscript.
			aMenu add: 'remove postscript' action: #removePostscript]
		ifFalse:
			[aMenu add: 'add postscript...' action: #editPostscript].
	aMenu addLine.
	
	aMenu add: 'destroy change set' action: #remove.
	aMenu addLine.
	
	aMenu addLine.
	aMenu add: 'more...' action: #offerShiftedChangeSetMenu.
	aMenu defaultTarget: oldTarget.

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'BG 10/28/2003 20:47'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"


	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.

	aMenu addLine.
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.


	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'.

		aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods any of whose previous authoring stamps do not start with "', Utilities authorInitials, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'go to change set''s project' action: #goToChangeSetsProject.
	aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.
	aMenu add: 'expunge uniclasses' action: #expungeUniclasses.
	aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'.

	aMenu add: 'uninstall this change set' action: #uninstallChangeSet.
	aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'.

	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !


!Debugger methodsFor: 'initialize' stamp: 'BG 10/28/2003 20:40'!
openFullNoSuspendLabel: aString
	"Create and schedule a full debugger with the given label. Do not terminate the current active process."

	| topView |

	
	topView _ self buildMVCDebuggerViewLabel: aString minSize: 300@200.
	topView controller openNoTerminate.
	^ topView
! !

!Debugger methodsFor: 'initialize' stamp: 'BG 10/28/2003 20:39'!
openNotifierContents: msgString label: label
	"Create and schedule a notifier view with the given label and message. A notifier view shows just the message or the first several lines of the stack, with a menu that allows the user to open a full debugger if so desired."
	"NOTE: When this method returns, a new process has been scheduled to run the windows, and thus this notifier, but the previous active porcess has not been suspended.  The sender will do this."
	| msg topView p |
	Sensor flushKeyboard.
	savedCursor _ Sensor currentCursor.
	Sensor currentCursor: Cursor normal.
	(label beginsWith: 'Space is low')
		ifTrue: [msg _ self lowSpaceChoices, (msgString ifNil: [''])]
		ifFalse: [msg _ msgString].
	isolationHead ifNotNil:
		["We have already revoked the isolation layer -- now jump to the parent project."
		msg _ self isolationRecoveryAdvice, msgString.
		failedProject _ Project current.
		isolationHead parent enterForEmergencyRecovery].



	Display fullScreen.
	topView _ self 
		buildMVCNotifierViewLabel: label 
		message: thisContext sender sender shortStack 
		minSize: 350@((14 * 5) + 16 + self optionalButtonHeight).
	ScheduledControllers activeController
		ifNil: [p _ Display boundingBox center]
		ifNotNil: [p _ ScheduledControllers activeController view displayBox center].
	topView controller openNoTerminateDisplayAt: (p max: (200@60)).
	^ topView! !

!Debugger methodsFor: 'notifier menu' stamp: 'BG 10/28/2003 20:40'!
debug
	"Open a full DebuggerView."
	| topView |
	topView _ self topView.
	topView model: nil.  "so close won't release me."
	

	topView controller controlTerminate.
	topView deEmphasize; erase.

	"a few hacks to get the scroll selection artifacts out when we got here by clicking in the list"
	topView subViewWantingControl ifNotNil: [
		topView subViewWantingControl controller controlTerminate
	].
	topView controller status: #closed.

	self openFullNoSuspendLabel: topView label.
	topView controller closeAndUnscheduleNoErase.
	Processor terminateActive.
! !

!Debugger methodsFor: 'context stack menu' stamp: 'BG 10/29/2003 09:57'!
mailOutBugReport
	"Compose a useful bug report showing the state of the process as well as vital image statistics as suggested by Chris Norton - 
'Squeak could pre-fill the bug form with lots of vital, but
oft-repeated, information like what is the image version, last update
number, VM version, platform, available RAM, author...'

and address it to the list with the appropriate subject prefix."

	| messageStrm |
	(Smalltalk includesKey: #Celeste)
		ifFalse: [^ self inform: 'no mail reader present'].

	Cursor write
		showWhile: 
			["Prepare the message"
			messageStrm _ WriteStream on: (String new: 1500).
			messageStrm nextPutAll: 'From: ';
			 nextPutAll: (Smalltalk at: #Celeste) userName;
			 cr;
			 nextPutAll: 'To: squeak-dev@lists.squeakfoundation.org';
			 cr;
			 nextPutAll: 'Subject: ';
			 nextPutAll: '[BUG]'; nextPutAll: self interruptedContext printString;
			 cr;cr;
			 nextPutAll: 'here insert explanation of what you were doing, suspect changes you''ve made and so forth.';cr;cr.
			self interruptedContext errorReportOn: messageStrm.

			(Smalltalk at: #CelesteComposition)
				openForCeleste: (Smalltalk at: #Celeste) current 
				initialText: messageStrm contents].
! !


!FileContentsBrowser methodsFor: 'infoView' stamp: 'BG 10/28/2003 20:55'!
updateInfoView

	Smalltalk isMorphic 
		ifFalse: [
			self infoString contents: self infoViewContents.
			self infoString changed].! !

!FileContentsBrowser methodsFor: 'creation' stamp: 'BG 10/28/2003 20:55'!
createViews
	"Create a pluggable version of all the views for a Browser, including views and controllers."

	| hasSingleFile width topView packageListView classListView switchView messageCategoryListView messageListView browserCodeView infoView |
	contentsSymbol _ self defaultDiffsSymbol.  "#showDiffs or #prettyDiffs"
	
	(hasSingleFile _ self packages size = 1)
		ifTrue: [width _ 150]
		ifFalse: [width _ 200].

	(topView _ StandardSystemView new) 
		model: self;
		borderWidth: 1.
		"label and minSize taken care of by caller"
	
	hasSingleFile 
		ifTrue: [
			self systemCategoryListIndex: 1.
			packageListView _ PluggableListView on: self
				list: #systemCategorySingleton
				selected: #indexIsOne 
				changeSelected: #indexIsOne:
				menu: #packageListMenu:
				keystroke: #packageListKey:from:.
			packageListView window: (0 @ 0 extent: width @ 12)]
		ifFalse: [
			packageListView _ PluggableListView on: self
				list: #systemCategoryList
				selected: #systemCategoryListIndex
				changeSelected: #systemCategoryListIndex:
				menu: #packageListMenu:
				keystroke: #packageListKey:from:.
			packageListView window: (0 @ 0 extent: 50 @ 70)].
	topView addSubView: packageListView.

	classListView _ PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:
		keystroke: #classListKey:from:.
	classListView window: (0 @ 0 extent: 50 @ 62).
	hasSingleFile 
		ifTrue: [topView addSubView: classListView below: packageListView]
		ifFalse: [topView addSubView: classListView toRightOf: packageListView].

	switchView _ self buildInstanceClassSwitchView.
	switchView borderWidth: 1.
	topView addSubView: switchView below: classListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:
		keystroke: #messageListKey:from:.
	messageListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: width@110).
	topView 
		addSubView: browserCodeView 
		below: (hasSingleFile 
			ifTrue: [switchView]
			ifFalse: [packageListView]).

	infoView _ StringHolderView new
		model: self infoString;
		window: (0@0 extent: width@12);
		borderWidth: 1.
	topView addSubView: infoView below: browserCodeView.

	^ topView
! !


!FileList methodsFor: 'initialization' stamp: 'BG 10/29/2003 10:06'!
modelWakeUp
	"User has entered or expanded the window -- reopen any remote connection."

"	(directory isKindOf: ServerDirectory) 
		ifTrue: [directory wakeUp] " "It would be good to implement a null method wakeUp on the root of directory"! !

!FileList methodsFor: 'initialization' stamp: 'BG 10/28/2003 20:45'!
updateButtonRow
	"Dynamically update the contents of the button row, if any."

! !

!FileList methodsFor: 'volume list and pattern' stamp: 'BG 10/29/2003 10:06'!
listForPattern: pat
	"Make the list be those file names which match the pattern."

	| sizePad newList |
	newList _ (self entriesMatching: pat) asSortedCollection: self sortBlock.
	sizePad _ (newList inject: 0 into: [:mx :entry | mx max: (entry at: 5)])
					asStringWithCommas size - 1.
	newList _ newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^ newList asArray ",
		(ServerDirectory serverNames collect: [:n | '^' , n , self folderString])"].
	^ newList asArray! !

!FileList methodsFor: 'file list' stamp: 'BG 10/29/2003 10:05'!
fileListIndex: anInteger
	"Select the file name having the given index, and display its contents."

	| item name |
	self okToChange ifFalse: [^ self].
	listIndex := anInteger.
	listIndex = 0 
		ifTrue: [fileName := nil]
		ifFalse:
			[item := self fileNameFromFormattedItem: (list at: anInteger).
			(item endsWith: self folderString)
				ifTrue:
					["remove [...] folder string and open the folder"
					name := item copyFrom: 1 to: item size - self folderString size.
					listIndex := 0.
					brevityState := #FileList.
					self addPath: name.
					name first = $^
						ifTrue: [self halt. "self directory: (ServerDirectory serverNamed: name allButFirst)"]
						ifFalse: [volListIndex = 1 ifTrue: [name _ name, directory slash].
							self directory: (directory directoryNamed: name)]]
				ifFalse: [fileName := item]].  "open the file selected"

	brevityState := #needToGetBrief.
	self changed: #fileListIndex.
	self changed: #contents.
	self updateButtonRow! !

!FileList methodsFor: 'obsolete methods' stamp: 'BG 10/28/2003 20:56'!
loadCRDictionary

	self error: 'should not be used keep for temporary documentation'.
	Smalltalk isMorphic ifFalse: 
		[self beep.
		^ self inform: 'Only available within morphic'].! !

!FileList methodsFor: 'obsolete methods' stamp: 'BG 10/28/2003 20:55'!
loadCRDisplayProperties

	self error: 'should not be used keep for temporary documentation'.
	Smalltalk isMorphic ifFalse: 
		[self beep.
		^ self inform: 'Only available within morphic'].! !

!FileList methodsFor: 'obsolete methods' stamp: 'BG 10/28/2003 20:56'!
openModelintoAlice
	"If a Wonderland exists, load this model into it as an actor.  If it doesn't, make one first"

	self error: 'should not be used keep for temporary documentation'.
	Smalltalk isMorphic ifFalse: [^self error: 'Only works in Morphic -  sorry!!'].

	! !

!FileList methodsFor: 'to be transformed in registration' stamp: 'BG 10/29/2003 10:07'!
removeServer

	| choice names |
	self flag: #ViolateNonReferenceToOtherClasses.
	names := #(). "ServerDirectory serverNames asSortedArray."
	choice := (SelectionMenu labelList: names selections: names) startUp.
	choice == nil ifTrue: [^ self].
	"ServerDirectory removeServerNamed: choice"! !

!FileList methodsFor: 'to be transformed in registration' stamp: 'BG 10/29/2003 10:04'!
volumeMenu: aMenu
	^ aMenu labels:
'recent...
remove server...
delete directory...'
		lines: # (1 2)
		selections: #(recentDirs removeServer deleteDirectory)
! !

!FileList methodsFor: 'file list menu' stamp: 'BG 11/2/2003 21:05'!
fullFileListMenu: aMenu shifted: aBoolean
	"Fill the menu with all possible items for the file list pane, regardless of selection."

	| services servicesPlus extraLines linePointer |
	aMenu title: 'all possible file operations'.
	servicesPlus := self servicesFromSelectorSpecs:  #(
		openImageInWindow: importImage: openAsBackground: 
		-
		fromFileName: openFromFile:  
		-
		openOn: fileIntoNewChangeSet: fileIn: browseChangesFile: browseRecentLogOnPath:
		-
		playMidiFile: openAsMovie: openAsFlash: openTTFFile:
		open3DSFile:  openTapeFromFile: openVRMLFile:
		-
		viewContents: saveContents: openOn:
		-
		
		removeLineFeeds: ).
	extraLines _ OrderedCollection new.
	linePointer _ 1.
	services _ OrderedCollection new.
	servicesPlus doWithIndex:
		[:svc :ind |
			svc == #-
				ifTrue:
					[extraLines add: linePointer - 1]
				ifFalse:
					[services add: svc.
					linePointer _ linePointer + 1]].
	aMenu 
		addServices: services 
		for: self fullName
		extraLines: extraLines! !

!FileList methodsFor: 'file list menu' stamp: 'BG 11/2/2003 21:04'!
myServicesForFile: fullName suffix: suffix

	^(FileStream isSourceFileSuffix: suffix)
		ifTrue: [ #() ]
		ifFalse: [ #() ]! !

!FileList methodsFor: 'private' stamp: 'BG 10/29/2003 10:07'!
readContentsBrief: brevityFlag
	"Read the contents of the receiver's selected file, unless it is too long, in which case show just the first 5000 characters. Don't create a file if it doesn't already exist."
	| f fileSize first5000 |

	brevityFlag ifTrue: [
		"(directory isKindOf: ServerDirectory) ifTrue: [^ self readServerBrief]"].
	f _ directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	(brevityFlag not or: [(fileSize _ f size) <= 100000]) ifTrue:
		[contents _ f contentsOfEntireFile.
		brevityState _ #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 _ f next: 5000.
	f close.
	contents _ 'File ''', fileName, ''' is ', fileSize printString, ' bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
', first5000 , '
------------------------------------------
... end of the first 5000 characters.'.
	brevityState _ #briefFile.   "don't change till actually read"
	^ contents.
! !


!FileList2 methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 09:35'!
openImageInWindow
	"Handle five file formats: GIF, JPG, PNG, Form stoteOn: (run coded), and BMP.
	Fail if file format is not recognized."

	| image myStream |

	myStream _ (directory readOnlyFileNamed: fileName) binary.
	image _ Form fromBinaryStream: myStream.
	myStream close.

	Smalltalk isMorphic
		ifTrue: []
		ifFalse: [FormView open: image named: fileName]! !


!Inspector methodsFor: 'menu commands' stamp: 'BG 10/29/2003 08:14'!
addCollectionItemsTo: aMenu
	"If the current selection is an appropriate collection, add items to aMenu that cater to that kind of selection"

	| sel |
	((((sel _ self selection) isMemberOf: Array) or: [sel isMemberOf: OrderedCollection]) and: 
		[sel size > 0]) ifTrue: [
			aMenu addList: #(
				('inspect element...'					inspectElement))].
! !

!Inspector methodsFor: 'menu commands' stamp: 'BG 10/28/2003 21:00'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.  If in mvc, an old-style protocol browser is opened instead."

	Smalltalk isMorphic ifFalse: [^ self spawnProtocol].! !

!Inspector methodsFor: 'menu commands' stamp: 'BG 10/28/2003 21:00'!
fieldListMenu: aMenu
	"Arm the supplied menu with items for the field-list of the receiver"



	aMenu addList: #(
		('inspect (i)'						inspectSelection)).

	self addCollectionItemsTo: aMenu.

	aMenu addList: #(
		-
		('method refs to this inst var'		referencesToSelection)
		('methods storing into this inst var'	defsOfSelection)
		('objects pointing to this value'		objectReferencesToSelection)
		('chase pointers'					chasePointers)
		-
		('browse full (b)'					browseMethodFull)
		('browse class'						browseClass)
		('browse hierarchy'					classHierarchy)
		('browse protocol (p)'				browseFullProtocol)
		-
		('inst var refs...'					browseInstVarRefs)
		('inst var defs...'					browseInstVarDefs)
		('class var refs...'					classVarRefs)
		('class variables'					browseClassVariables)
		('class refs (N)'						browseClassRefs)
		-
		('copy name (c)'					copyName)		
		('basic inspect'						inspectBasic)).


	^ aMenu


"			-
			('alias for this value'			aliasForValue)
			('watcher for this slot'			watcherForSlot)"

! !

!Inspector methodsFor: 'menu commands' stamp: 'BG 11/1/2003 22:25'!
inspectElement
	| sel selSize countString count |
	"Create and schedule an Inspector on an element of the receiver's model's currently selected collection."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	((sel _ self selection) isKindOf: SequenceableCollection) ifFalse:
		[
		^ sel inspect].
	(selSize _ sel size) == 1 ifTrue: [^ sel first inspect].
	selSize <= 15 ifTrue:
		[count _ (SelectionMenu selections: (1 to: selSize) asArray) startUpWithCaption: 'which element?'.
		count ifNil: [^ self] ifNotNil: [^ (sel at: count) inspect]].

	countString _ FillInTheBlank request: 'Which element? (1 - ', selSize printString, ')' initialAnswer: '1'.
	countString isEmptyOrNil ifTrue: [^ self].
	count _ Integer readFrom: (ReadStream on: countString).
	(count > 0 and: [count <= selSize])
		ifTrue:
			[(sel  at: count) inspect]
		ifFalse:
			[self beep]! !


!MessageSet methodsFor: 'message list' stamp: 'BG 11/1/2003 14:51'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built.  Add items specific to MessageSet"

	self growable ifTrue:
		[aMenu addList: #(
			-
			('remove from this browser'		removeMessageFromBrowser)
			)].
	aMenu add: 'sort by date' action: #sortByDate! !


!PackagePaneBrowser methodsFor: 'package list' stamp: 'BG 10/30/2003 21:09'!
openEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	"PackageBrowser openBrowser"

	| packageListView systemCategoryListView classListView messageCategoryListView
	  messageListView browserCodeView topView switchView annotationPane underPane y optionalButtonsView |

	topView := StandardSystemView new model: self.
	topView borderWidth: 1.  "label and minSize taken care of by caller"

	packageListView := PluggableListView on: self
		list: #packageList
		selected: #packageListIndex
		changeSelected: #packageListIndex:
		menu: #packageMenu:.
	packageListView window: (0 @ 0 extent: 20 @ 70).
	topView addSubView: packageListView.

	systemCategoryListView := PluggableListView on: self
		list: #systemCategoryList
		selected: #systemCategoryListIndex
		changeSelected: #systemCategoryListIndex:
		menu: #systemCategoryMenu:.
	systemCategoryListView window: (20 @ 0 extent: 30 @ 70).
	topView addSubView: systemCategoryListView.

	classListView := PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:shifted:.
	classListView window: (0 @ 0 extent: 50 @ 62).
	topView addSubView: classListView toRightOf: systemCategoryListView.

	switchView := self buildInstanceClassSwitchView.
	switchView borderWidth: 1.
	topView addSubView: switchView below: classListView.

	messageCategoryListView := PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	messageListView := PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	self wantsAnnotationPane
		ifTrue:
			[annotationPane _ PluggableTextView on: self
				text: #annotation accept: nil
				readSelection: nil menu: nil.
			annotationPane window: (0@0 extent: 200@self optionalAnnotationHeight).
			topView addSubView: annotationPane below: packageListView.
			underPane _ annotationPane.
			y _ 110 - self optionalAnnotationHeight]
		ifFalse:
			[underPane _ packageListView.
			y _ 110].

	self wantsOptionalButtons ifTrue:
		[optionalButtonsView _ self buildOptionalButtonsView.
		optionalButtonsView borderWidth: 1.
		topView addSubView: optionalButtonsView below: underPane.
		underPane _ optionalButtonsView.
		y _ y - self optionalButtonHeight].

	browserCodeView := PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@y).
	topView addSubView: browserCodeView below: underPane.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView! !


!PluggableFileList methodsFor: 'file list menu' stamp: 'BG 10/29/2003 10:08'!
listForPattern: pat
	"Make the list be those file names which match the pattern."
	| entries sizePad newList allFiles |
	entries _ directory entries select: fileFilterBlock.
	sizePad _ (entries inject: 0 into: [:mx :entry | mx max: (entry at: 5)])
					asStringWithCommas size - 1.

	newList _ (SortedCollection new: 30) sortBlock: self sortBlock.

	allFiles _ pat = '*'.
	entries do:
		[:entry | "<dirflag><name><creationTime><modificationTime><fileSize>"
		(allFiles or: [entry isDirectory or: [pat match: entry first]]) ifTrue:
			[newList add: entry]].

	newList _ newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^ newList asArray" ,
		(ServerDirectory serverNames collect: [:n | '^' , n , self folderString])"].
	newFiles _ OrderedCollection new.
	^ newList asArray.! !


!RecentMessageSet methodsFor: 'message functions' stamp: 'BG 11/1/2003 13:27'!
messageListMenu: aMenu shifted: shifted
	"Answer the message-list menu"

	shifted ifTrue: [^ self shiftedMessageListMenu: aMenu].
	aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('browse hierarchy (h)'					classHierarchy)
			('browse method (O)'					openSingleMessageBrowser)
			('browse protocol (p)'					browseFullProtocol)
			-
			('fileOut (o)'							fileOutMessage)
			('printOut'								printOutMessage)
			('copy selector (c)'						copySelector)
			-
			('senders of... (n)'						browseSendersOfMessages)
			('implementors of... (m)'					browseMessages)
			('inheritance (i)'						methodHierarchy)
			('versions (v)'							browseVersions)
			-
			('inst var refs...'						browseInstVarRefs)
			('inst var defs...'						browseInstVarDefs)
			('class var refs...'						browseClassVarRefs)
			('class variables'						browseClassVariables)
			('class refs (N)'							browseClassRefs)
			-
			('remove method (x)'					removeMessage)
			('remove from RecentSubmissions'		removeFromRecentSubmissions)
			-
			('more...'								shiftedYellowButtonActivity)).
	^ aMenu! !


!SelectorBrowser methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 00:38'!
open
	"Create a Browser that lets you type part of a selector, shows a list of selectors,
	shows the classes of the one you chose, and spwns a full browser on it.
		SelectorBrowser new open
	"

	| selectorListView typeInView topView classListView exampleView |

	selectorIndex _ classListIndex _ 0.
	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	typeInView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	typeInView window: (0@0 extent: 50@14);
		askBeforeDiscardingEdits: false.
	topView addSubView: typeInView.

	selectorListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #selectorMenu:
		keystroke: #messageListKey:from:.
	selectorListView menuTitleSelector: #selectorMenuTitle.
	selectorListView window: (0 @ 0 extent: 50 @ 46).
	topView addSubView: selectorListView below: typeInView.

	classListView _ PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: nil	"never anything selected"
		keystroke: #arrowKey:from:.
	classListView menuTitleSelector: #classListSelectorTitle.
	classListView window: (0 @ 0 extent: 50 @ 60).
	topView addSubView: classListView toRightOf: typeInView.

	exampleView _ PluggableTextView on: self 
			text: #byExample accept: #byExample:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	exampleView window: (0@0 extent: 100@40);
		askBeforeDiscardingEdits: false.
	topView addSubView: exampleView below: selectorListView.


	topView label: 'Method Finder'.
	"topView minimumSize: 350@250; maximumSize: 350@250."
	topView subViews do: [:each | each controller].
	topView controller open.

! !


!Browser class methodsFor: 'instance creation' stamp: 'BG 10/30/2003 21:10'!
openBrowserView: aBrowserView label: aString 
	"Schedule aBrowserView, labelling the view aString."
	
	aBrowserView label: aString.
	aBrowserView minimumSize: 300 @ 200.
	aBrowserView subViews do: [:each | each controller].
	aBrowserView controller open! !


!ChangeList class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:46'!
open: aChangeList name: aString multiSelect: multiSelect
	"Create a standard system view for the messageSet, whose label is aString.
	The listView may be either single or multiple selection type"

	| topView listHeight annoHeight optButtonHeight codeHeight aListView underPane annotationPane buttonsView aBrowserCodeView |
	
	listHeight _ 70.
	annoHeight _ 10.
	optButtonHeight _ aChangeList optionalButtonHeight.
	codeHeight _ 110.

	topView _ (StandardSystemView new)
		model: aChangeList;
		label: aString;
		minimumSize: 200 @ 120;
		borderWidth: 1.

	aListView _ (multiSelect
			ifTrue: [PluggableListViewOfMany
						on: aChangeList
						list: #list
						primarySelection: #listIndex
						changePrimarySelection: #toggleListIndex:
						listSelection: #listSelectionAt:
						changeListSelection: #listSelectionAt:put:
						menu: (aChangeList showsVersions
								ifTrue: [#versionsMenu:]
								ifFalse: [#changeListMenu:])]
			ifFalse: [PluggableListView
						on: aChangeList
						list: #list
						selected: #listIndex
						changeSelected: #toggleListIndex:
						menu: (aChangeList showsVersions
								ifTrue: [#versionsMenu:]
								ifFalse: [#changeListMenu:])]).
	aListView window: (0 @ 0 extent: 200 @ listHeight).
	topView addSubView: aListView.

	underPane _ aListView.
	aChangeList wantsAnnotationPane
		ifTrue:
			[annotationPane _ PluggableTextView
				on: aChangeList
				text: #annotation
				accept: nil
				readSelection: nil
				menu: nil.
			annotationPane window: (0 @ 0 extent: 200 @ 10).
			topView addSubView: annotationPane below: underPane.
			underPane _ annotationPane.
			codeHeight _ codeHeight - annoHeight].

	aChangeList wantsOptionalButtons
		ifTrue:
			[buttonsView _ aChangeList optionalButtonsView.
			buttonsView borderWidth: 1.
			topView addSubView: buttonsView below: underPane.
			underPane _ buttonsView.
			codeHeight _ codeHeight - optButtonHeight].

	aBrowserCodeView _ PluggableTextView
			on: aChangeList
			text: #contents
			accept: #contents:
			readSelection: #contentsSelection
			menu: #codePaneMenu:shifted:.
	aBrowserCodeView
			controller: ReadOnlyTextController new;
			window: (0 @ 0 extent: 200 @ codeHeight).
	topView addSubView: aBrowserCodeView below: underPane.

	topView controller open.! !


!ChangeSorter class methodsFor: 'enumerating' stamp: 'BG 10/28/2003 20:38'!
promoteToTop: aChangeSet
	"make aChangeSet the first in the list from now on"
	AllChangeSets remove: aChangeSet ifAbsent: [^ self].
	AllChangeSets add: aChangeSet.! !

!ChangeSorter class methodsFor: 'services' stamp: 'BG 10/28/2003 20:38'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come the project changesets that come with the release.  These are mostly empty.
		Next come all numbered updates.
		Next come all remaining changesets
	In a ChangeSorter, they will appear in the reversed order."

	"ChangeSorter reorderChangeSets"

	| newHead newMid newTail |
	newHead _ OrderedCollection new.
	newMid _ OrderedCollection new.
	newTail _ OrderedCollection new.
	AllChangeSets do:
		[:aChangeSet |
			(self belongsInProjectsInRelease: aChangeSet)
				ifTrue:
					[newHead add: aChangeSet]
				ifFalse:
					[(self belongsInNumbered: aChangeSet)
						ifTrue:
							[newMid add: aChangeSet]
						ifFalse:
							[newTail add: aChangeSet]]].
	AllChangeSets _ newHead, newMid, newTail.! !


!Debugger class methodsFor: 'class initialization' stamp: 'BG 10/30/2003 20:37'!
openContext: aContext label: aString contents: contentsStringOrNil
	| isolationHead |
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."
	<primitive: 19> "Simulation guard"
	ErrorRecursion not & Preferences logDebuggerStackToFile ifTrue:
		[Smalltalk logError: aString inContext: aContext to: 'SqueakDebug.log'].
	ErrorRecursion ifTrue:
		[ErrorRecursion _ false.
		"(isolationHead _ CurrentProjectRefactoring currentIsolationHead)
			ifNil: [self primitiveError: aString]
			ifNotNil: [isolationHead revoke]"].
	ErrorRecursion _ true.
	self informExistingDebugger: aContext label: aString.
	(Debugger context: aContext isolationHead: nil)
		openNotifierContents: contentsStringOrNil
		label: aString.
	ErrorRecursion _ false.
	Processor activeProcess suspend.
! !


!FileList class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:56'!
open
	"Open a view of an instance of me on the default directory."
	"FileList open"
	| dir aFileList topView volListView templateView fileListView fileContentsView underPane pHeight |
	
	dir _ FileDirectory default.
	aFileList _ self new directory: dir.
	topView _ StandardSystemView new.
	topView
		model: aFileList;
		label: dir pathName;
		minimumSize: 200@200.
	topView borderWidth: 1.

	volListView _ PluggableListView on: aFileList
		list: #volumeList
		selected: #volumeListIndex
		changeSelected: #volumeListIndex:
		menu: #volumeMenu:.
	volListView autoDeselect: false.
	volListView window: (0@0 extent: 80@45).
	topView addSubView: volListView.

	templateView _ PluggableTextView on: aFileList
		text: #pattern
		accept: #pattern:.
	templateView askBeforeDiscardingEdits: false.
	templateView window: (0@0 extent: 80@15).
	topView addSubView: templateView below: volListView.

	aFileList wantsOptionalButtons
		ifTrue:
			[underPane _ aFileList optionalButtonView.
			underPane isNil
				ifTrue: [pHeight _ 60]
				ifFalse: [
					topView addSubView: underPane toRightOf: volListView.
					pHeight _ 60 - aFileList optionalButtonHeight]]
		ifFalse:
			[underPane _ nil.
			pHeight _ 60].

	fileListView _ PluggableListView on: aFileList
		list: #fileList
		selected: #fileListIndex
		changeSelected: #fileListIndex:
		menu: #fileListMenu:.
	fileListView window: (0@0 extent: 120@pHeight).
	underPane isNil
		ifTrue: [topView addSubView: fileListView toRightOf: volListView]
		ifFalse: [topView addSubView: fileListView below: underPane].
	fileListView controller terminateDuringSelect: true.  "Pane to left may change under scrollbar"

	fileContentsView _ PluggableTextView on: aFileList
		text: #contents accept: #put:
		readSelection: #contentsSelection menu: #fileContentsMenu:shifted:.
	fileContentsView window: (0@0 extent: 200@140).
	topView addSubView: fileContentsView below: templateView.

	topView controller open! !

!FileList class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:41'!
openEditorOn: aFileStream editString: editString
	"Open an editor on the given FileStream."

	| fileModel topView fileContentsView |

	fileModel _ FileList new setFileStream: aFileStream.	"closes the stream"
	topView _ StandardSystemView new.
	topView
		model: fileModel;
		label: aFileStream fullName;
		minimumSize: 180@120.
	topView borderWidth: 1.

	fileContentsView _ PluggableTextView on: fileModel 
		text: #contents accept: #put:
		readSelection: #contentsSelection menu: #fileContentsMenu:shifted:.
	fileContentsView window: (0@0 extent: 180@120).
	topView addSubView: fileContentsView.
	editString ifNotNil: [fileContentsView editString: editString.
			fileContentsView hasUnacceptedEdits: true].

	topView controller open.
! !


!FillInTheBlank class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:57'!
multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line.
	NOTE: The ENTER key does not work on Windows platforms."

	"FillInTheBlank
		multiLineRequest:
'Enter several lines; end input by accepting
or canceling via menu or press Alt+s/Alt+l'
		centerAt: Display center
		initialAnswer: 'Once upon a time...'
		answerHeight: 200"

	| model fillInView |
	

	model _ self new initialize.
	model contents: defaultAnswer.
	model responseUponCancel: nil.
	model acceptOnCR: false.
	fillInView _
		self fillInTheBlankViewClass
			multiLineOn: model
			message: queryString
			centerAt: aPoint
			answerHeight: answerHeight.
	^ model show: fillInView! !

!FillInTheBlank class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:57'!
request: queryString initialAnswer: defaultAnswer centerAt: aPoint
	"Create an instance of me whose question is queryString with the given
	initial answer. Invoke it centered at the given point, and answer the
	string the user accepts. Answer the empty string if the user cancels."

	"FillInTheBlank
		request: 'Type something, then type CR.'
		initialAnswer: 'yo ho ho!!'
		centerAt: Display center"

	| model fillInView |
	

	model _ self new initialize.
	model contents: defaultAnswer.
	fillInView _
		self fillInTheBlankViewClass
			on: model
			message: queryString
			centerAt: aPoint.
	^ model show: fillInView! !

!FillInTheBlank class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:57'!
requestPassword: queryString
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."

	"FillInTheBlank requestPassword: 'POP password'"

	| model fillInView |
	

	model _ self new initialize.
	model contents: ''.
	fillInView _
		self fillInTheBlankViewClass
			requestPassword: model
			message: queryString
			centerAt: Sensor cursorPoint
			answerHeight: 40.
	^ model show: fillInView! !


!Inspector class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 14:56'!
openOn: anObject withEvalPane: withEval withLabel: label
        
        ^ self openOn: anObject 
                withEvalPane: withEval 
                withLabel: label 
                valueViewClass: PluggableTextView
! !


!MessageSet class methodsFor: 'instance creation' stamp: 'BG 10/29/2003 08:40'!
open: aMessageSet name: aString 
	"Create a standard system view for the messageSet, aMessageSet, whose label is aString."
	| topView aListView aBrowserCodeView aTextView underPane y buttonsView winWidth |

	winWidth _ 200.
	topView _ (StandardSystemView new) model: aMessageSet.
	topView label: aString.
	topView minimumSize: winWidth @ 120.
	topView borderWidth: 1.

	aListView _ PluggableListView on: aMessageSet
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	aListView  menuTitleSelector: #messageListSelectorTitle.
	aListView window: (0 @ 0 extent: winWidth @ 100).
	topView addSubView: aListView.

	aMessageSet  wantsAnnotationPane
		ifTrue:
			[aTextView _ PluggableTextView on: aMessageSet 
			text: #annotation accept: nil
			readSelection: nil menu: nil.
			aTextView window: (0 @ 0 extent: winWidth @ 24).
			topView addSubView: aTextView below: aListView.
			underPane _ aTextView.
			y _ 300 - 24.
			aTextView askBeforeDiscardingEdits: false]
		ifFalse:
			[underPane _ aListView.
			y _ 300].

	aMessageSet wantsOptionalButtons ifTrue:
		[buttonsView _ aMessageSet buildOptionalButtonsView.
		topView addSubView: buttonsView below: underPane.
		underPane _ buttonsView.
		y _ y - aMessageSet optionalButtonHeight].

	aBrowserCodeView _ PluggableTextView on: aMessageSet 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	aBrowserCodeView window: (0 @ 0 extent: winWidth @ y).
	topView addSubView: aBrowserCodeView below: underPane.
	topView setUpdatablePanesFrom: #(messageList).
	topView controller open! !


!ChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'BG 10/28/2003 20:51'!
openMessageList: messageList name: labelString autoSelect: autoSelectString changeSet: aChangeSet
	| messageSet |

	messageSet _ self messageList: messageList.
	messageSet changeSet: aChangeSet.
	messageSet autoSelectString: autoSelectString.
	Smalltalk isMorphic
		ifTrue: []
		ifFalse: [ScheduledControllers scheduleActive:  (self open: messageSet name: labelString)]! !


!StringHolderView class methodsFor: 'instance creation' stamp: 'BG 10/28/2003 20:43'!
open: aStringHolder label: labelString 
	"NOTE this should be in the model class, and all senders so redirected,
	in order that the view class can be discarded in a morphic world."

	"Create a standard system view of the model, aStringHolder, as viewed by 
	an instance of me. The label of the view is aString."
	| aStringHolderView topView |


	aStringHolderView _ self container: aStringHolder.
	topView _ StandardSystemView new.
	topView model: aStringHolderView model.
	topView addSubView: aStringHolderView.
	topView label: labelString.
	topView minimumSize: 100 @ 50.
	topView controller open! !


!SyntaxError class methodsFor: 'instance creation' stamp: 'BG 10/29/2003 08:05'!
open: aSyntaxError
	"Answer a standard system view whose model is an instance of me."
	| topView |
	<primitive: 19> "Simulation guard"
	
	topView _ self buildMVCViewOn: aSyntaxError.
	topView controller openNoTerminateDisplayAt: Display extent // 2.
	Cursor normal show.
	Processor activeProcess suspend.
! !


!SystemDictionary methodsFor: 'browsing' stamp: 'BG 10/30/2003 21:12'!
browseMethodsWhoseNamesContain: aString
	"Launch a tool which shows all methods whose names contain the given string; case-insensitive.
	1/16/1996 sw, at the dawn of Squeak: this was the classic implementation that provided the underpinning for the 'method names containing it' (cmd-shift-W) feature that has always been in Squeak -- the feature that later inspired the MethodFinder (aka SelectorBrowser).
	sw 7/27/2001:	Switched to showing a MessageNames tool rather than a message-list browser, if in Morphic."

	| aList |
	aList _ Symbol selectorsContaining: aString.
			aList size > 0 ifTrue:
				[self browseAllImplementorsOfList: aList asSortedCollection title: 'Methods whose names contain ''', aString, '''']
	! !

!SystemDictionary methodsFor: 'memory space' stamp: 'BG 10/30/2003 20:41'!
lowSpaceWatcher
	"Wait until the low space semaphore is signalled, then take appropriate actions."

	| free |
	self garbageCollectMost <= self lowSpaceThreshold ifTrue: [
		self garbageCollect <= self lowSpaceThreshold ifTrue: [
			"free space must be above threshold before starting low space watcher"
			^ self beep]].

	LowSpaceSemaphore _ Semaphore new.
	self primLowSpaceSemaphore: LowSpaceSemaphore.
	self primSignalAtBytesLeft: self lowSpaceThreshold.  "enable low space interrupts"

	LowSpaceSemaphore wait.  "wait for a low space condition..."

	self primSignalAtBytesLeft: 0.  "disable low space interrupts"
	self primLowSpaceSemaphore: nil.
	LowSpaceProcess _ nil.
	"Note: user now unprotected until the low space watcher is re-installed"

	self memoryHogs isEmpty ifFalse: [
		free := self bytesLeft.
		self memoryHogs do: [ :hog | hog freeSomeSpace ].
		self bytesLeft > free ifTrue: [ ^ self installLowSpaceWatcher ]].

	Smalltalk isMorphic
			ifTrue: []
			ifFalse: [ScheduledControllers interruptName: 'Space is low']! !

!SystemDictionary methodsFor: 'miscellaneous' stamp: 'BG 10/30/2003 20:40'!
handleUserInterrupt
	Preferences cmdDotEnabled ifTrue:
		[Smalltalk isMorphic
			ifTrue: []
			ifFalse: [[ScheduledControllers interruptName: 'User Interrupt'] fork]]! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'BG 11/1/2003 14:06'!
allSentMessagesWithout: classesAndMessagesPair
	"Answer the set of selectors which are sent somewhere in the system, computed in the absence of the supplied classes and messages."
	| sent absentClasses absentSelectors literals hasMenuFromArray |
	sent _ IdentitySet new: CompiledMethod instanceCount.
	absentClasses _ classesAndMessagesPair first.
	absentSelectors _ classesAndMessagesPair second.
	Cursor execute showWhile: 
		[self classNames do: 
			[:cName | ((absentClasses includes: cName)
						ifTrue: [{}]
						ifFalse: [{(self at: cName). (self at: cName) class}]) do:
				[:cl | (absentSelectors isEmpty
						ifTrue: [cl selectors]
						ifFalse: [cl selectors copyWithoutAll: absentSelectors]) do: 
					[:sel | "Include all sels, but not if sent by self"
                         literals := (cl compiledMethodAt: sel) literals.
                         hasMenuFromArray :=
                          (literals includes: #addList:) | (literals includes: #fromArray:) .
					 literals  do: 
						[:m | 
						(m isMemberOf: Symbol) ifTrue:  "might be sent"
							[m == sel ifFalse: [sent add: m]].
						(m isMemberOf: Array) ifTrue:  "might be performed"
							[m do: [:x | (x isMemberOf: Symbol) ifTrue:
								             [x == sel ifFalse: [sent add: x]].
                                               (hasMenuFromArray and: [x isMemberOf: Array])
                                                 ifTrue: [x do: [:y | (y isMemberOf: Symbol) ifTrue:
								             [y == sel ifFalse: [sent add: y]].]]
                                 ]       ].
   
]]]].
		"The following may be sent without being in any literal frame"
		1 to: self specialSelectorSize do: 
			[:index | 
			sent add: (self specialSelectorAt: index)]].
	Smalltalk presumedSentMessages do: [:sel | sent add: sel].
	^ sent! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/1/2003 22:51'!
discard3D
	"Smalltalk discard3D"
	"Discard 3D Support."
	self discardWonderland.
	Smalltalk removeKey: #B3DEngineConstants ifAbsent: [].
	Smalltalk at: #InterpolatingImageMorph ifPresent:[:cls| cls removeFromSystem].
	SystemOrganization removeCategoriesMatching: 'Graphics-FXBlt'.
	SystemOrganization removeCategoriesMatching: 'Graphics-External'.
	SystemOrganization removeCategoriesMatching: 'Balloon3D-*'.
	SystemOrganization removeCategoriesMatching: 'Graphics-Tools-*'.
	Color removeSelector: #asB3DColor.
	Form removeSelector: #asTexture.
	(Smalltalk at: #Morph) removeSelector: #asTexture.
	FileList removeSelector: #open3DSFile.
	Point removeSelector: #@.
	Smalltalk at: #BalloonCanvas ifPresent:[:cc| cc removeSelector: #render:].
	Stream removeSelector: #asVRMLStream.
     (Smalltalk includesKey: #DataStream)
         ifTrue: [(Smalltalk at: #DataStream) initialize]! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'Jon 8/24/2001 13:31'!
discardGenie
	"Discard Genie support."

	SystemOrganization removeCategoriesMatching: 'Genie-Engine'.
	SystemOrganization removeCategoriesMatching: 'Genie-UI'.
	SystemOrganization removeCategoriesMatching: 'Genie-Integration'.! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/2/2003 23:18'!
discardIRC
	"Smalltalk discardIRC"
	SystemOrganization removeCategoriesMatching: 'Network-IRC Chat'.
	(Smalltalk at: #TheWorldMenu) compile:'openIRC self inform:''IRC is not present'' '.! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/29/2003 08:26'!
discardMorphic
	"Discard Morphic.
Updated for 2.8 TPR"

   "Smalltalk discardMorphic"

	| subs |
	"Check that we are in an MVC Project and that there are no Morphic Projects
		or WorldMorphViews."
	(Smalltalk at: #Flaps) clobberFlapTabList.
	Smalltalk discardFlash.
	Smalltalk discardTrueType.
	subs _ OrderedCollection new.
	(Smalltalk at: #Morph) allSubclassesWithLevelDo: [:c :i | subs addFirst: c]
		startingLevel: 0.
	subs do: [:c | c removeFromSystem].
	Smalltalk removeClassNamed: #CornerRounder.
	Smalltalk removeKey: #BalloonEngineConstants ifAbsent: [].
	SystemOrganization removeCategoriesMatching: 'Balloon-*'.
	SystemOrganization removeCategoriesMatching: 'Morphic-*'.
	SystemOrganization removeSystemCategory: 'Graphics-Transformations'.
	SystemOrganization removeSystemCategory: 'ST80-Morphic'.
	ScriptingSystem _ nil.
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/2/2003 22:47'!
discardOddsAndEnds
	"This method throws out lots of classes that are not frequently used."
	"Smalltalk discardOddsAndEnds"

"	SystemOrganization removeSystemCategory: 'System-Serial Port'. "
	SystemOrganization removeSystemCategory: 'ST80-Symbols'.
	"SystemOrganization removeSystemCategory: 'Tools-File Contents Browser'. "
	SystemOrganization removeSystemCategory: 'Tools-Explorer'.
	"SystemOrganization removeSystemCategory: 'System-Digital Signatures'."
	"SystemOrganization removeSystemCategory: 'System-Archives'."
	SystemOrganization removeSystemCategory: 'Framework-Download'.
	SystemOrganization removeSystemCategory: 'System-Download'.
     SystemOrganization removeCategoriesMatching: 'XML-Parser'.
	Smalltalk removeClassNamed: #PreferencesPanel.

	Form removeSelector: #edit.
	Smalltalk at: #FormView ifPresent:
		[:c | c compile: 'defaultControllerClass  ^ NoController'
			classified: 'controller access'].
	Smalltalk removeClassNamed: #FormEditorView.
	Smalltalk removeClassNamed: #FormEditor.
	SystemOrganization removeSystemCategory: 'ST80-Paths'.

	"bit editor (remove Form editor first):"
	Form removeSelector: #bitEdit.
	Form removeSelector: #bitEditAt:scale:.
	StrikeFont removeSelector: #edit:.
	Smalltalk removeClassNamed: #FormButtonCache.
	Smalltalk removeClassNamed: #FormMenuController.
	Smalltalk removeClassNamed: #FormMenuView.
	Smalltalk removeClassNamed: #BitEditor.

	"inspector for Dictionaries of Forms"
	Dictionary removeSelector: #inspectFormsWithLabel:.
	SystemDictionary removeSelector: #viewImageImports.
	ScreenController removeSelector: #viewImageImports.
	Smalltalk removeClassNamed: #FormHolderView.
	Smalltalk removeClassNamed: #FormInspectView.

	"experimental hand-drawn character recoginizer:"
	ParagraphEditor removeSelector: #recognizeCharacters.
	ParagraphEditor removeSelector: #recognizer:.
	ParagraphEditor removeSelector: #recognizeCharactersWhileMouseIn:.
	Smalltalk removeClassNamed: #CharRecog.

	"experimental updating object viewer:"
	Object removeSelector: #evaluate:wheneverChangeIn:.
	Smalltalk removeClassNamed: #ObjectViewer.
	Smalltalk removeClassNamed: #ObjectTracer.

	"miscellaneous classes:"
	Smalltalk removeClassNamed: #Array2D.
	Smalltalk removeClassNamed: #DriveACar.
	Smalltalk removeClassNamed: #EventRecorder.
	Smalltalk removeClassNamed: #FindTheLight.
	Smalltalk removeClassNamed: #PluggableTest.
	Smalltalk removeClassNamed: #SystemMonitor.
	Smalltalk removeClassNamed: #DocLibrary.

	Smalltalk removeClassNamed: #ObjectExplorerWrapper.
	Smalltalk removeClassNamed: #LinkedMessageSet.
	Smalltalk removeClassNamed: #ObjectExplorer.
	Smalltalk removeClassNamed: #PackageBrowser.
	Smalltalk removeClassNamed: #AbstractHierarchicalList.
"	Smalltalk removeClassNamed: #HtmlFileStream. "
	Smalltalk removeClassNamed: #FXGrafPort.
	Smalltalk removeClassNamed: #FXBlt.

	Smalltalk at: #SampledSound ifPresent: [:c |c initialize].
"	#(Helvetica Palatino Courier ComicBold ComicPlain) do:
		[:k | TextConstants removeKey: k ifAbsent: []]. "

Preferences setButtonFontTo:	(StrikeFont familyName: #NewYork size: 12).



! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'Jon 8/24/2001 14:00'!
discardSUnit
	"Discard SUnit support."

	SystemOrganization removeCategoriesMatching: 'SUnit-Framework'.
	SystemOrganization removeCategoriesMatching: 'SUnit-Interface'.
	SystemOrganization removeCategoriesMatching: 'SUnit-Tests'.
	SystemOrganization removeCategoriesMatching: 'SCAN-Tests'! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'ar 8/17/2001 17:25'!
discardVMConstruction
	"Discard the virtual machine construction classes and the Smalltalk-to-C translator. These are only needed by those wishing to build or study the Squeak virtual machine, or by those wishing to construct new primitives via Smalltalk-to-C translation.
Updated to suit 2.8 TPR"

	"remove the code for virtual machines"
	Smalltalk removeKey: #InterpreterLog ifAbsent: [].

	"remove the Smalltalk-to-C translator"
	Smalltalk at: #CCodeGenerator ifPresent: [:codeGen | codeGen removeCompilerMethods].

	"remove any plugins"
	Smalltalk at: #InterpreterPlugin ifPresent:[:cls|
		(ChangeSet superclassOrder: cls withAllSubclasses asArray) reverseDo: 
			[:c | c removeFromSystem]].
	#(B3DSimulRasterizer BalloonDebugEngine BalloonDebugCanvas) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| cls removeFromSystem]].
	SystemOrganization removeCategoriesMatching: 'VMConstruction-*'.
	SystemOrganization removeCategoriesMatching: 'Squeak-Plugins'.

	"Various references"
	#(
		(Object removeSelector: primitive:parameters:receiver:)
		(SystemDictionary macroBenchmarks)
		(SystemDictionary macroBenchmark2)
		(String primGetInteger32:)
		(String primPutInteger32:at:)
	) do:[:spec|
		Smalltalk at: spec first ifPresent:[:cls| cls removeSelector: spec last].
	].! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/29/2003 08:49'!
discardWonderland
	"Smalltalk discardWonderland"
	"Discard 3D Support."
	Smalltalk at: #Wonderland ifPresent:[:cls| cls removeActorPrototypesFromSystem].
	Smalltalk removeKey: #WonderlandConstants ifAbsent: [].
	Smalltalk removeKey: #AliceConstants ifAbsent: [].
	SystemOrganization removeCategoriesMatching: 'Balloon3D-Wonderland*'.
	SystemOrganization removeCategoriesMatching: 'Balloon3D-Alice*'.
	SystemOrganization removeCategoriesMatching: 'Balloon3D-Pooh*'.
	SystemOrganization removeCategoriesMatching: 'Balloon3D-UserObjects'.
	Smalltalk at: #VRMLWonderlandBuilder ifPresent:[:cls| cls removeFromSystem].
	FileList removeSelector: #openVRMLFile.
	(Smalltalk at: #HaloMorph) compile: 'addPoohHandle: ignored ^self'.! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/29/2003 00:10'!
lastRemoval			"Smalltalk lastRemoval"

	 | oldDicts newDicts |

	"Some explicit removals - add unwanted methods keeping other methods."
	#(abandonSources printSpaceAnalysis)
		do: [:each | self class removeSelector: each].

	"Get rid of all unsent methods."
	[self removeAllUnSentMessages > 0] whileTrue.

	"Shrink method dictionaries."
	Smalltalk garbageCollect.
	oldDicts _ MethodDictionary allInstances.
	newDicts _ Array new: oldDicts size.
	oldDicts withIndexDo: [:d :index | 
		newDicts at: index put: d rehashWithoutBecome.
	].
	oldDicts elementsExchangeIdentityWith: newDicts.
	oldDicts _ newDicts _ nil.

"	Smalltalk allClassesDo: [:c | c zapOrganization]. "
	SystemOrganization _ nil.

	Smalltalk changes initialize! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/2/2003 23:19'!
majorShrink    
	"Undertake a major shrinkage of the image.
	This method throws out lots of the system that is not needed for, eg, operation in a hand-held PC.  majorShrink produces a 999k image in Squeak 2.8
	Smalltalk majorShrink; abandonSources; lastRemoval"

	| oldDicts newDicts subs classesToLeave |
	Smalltalk isMorphic ifTrue: [^ self error: 'You can only run majorShrink in MVC'].
	Project current isTopProject ifFalse: [^ self error: 'You can only run majorShrink in the top project'].
	(Smalltalk confirm: 'All sub-projects will be deleted from this image.
You should already have made a backup copy,
or you must save with a different name after shrinking.
Shall we proceed to discard most of the content in this image?')
		ifFalse: [^ self inform: 'No changes have been made.'].

	"Remove all projects but the current one.  - saves 522k"
	ProjectView allInstancesDo: [:pv | pv controller closeAndUnscheduleNoTerminate].
	Project current setParent: Project current.
	(Smalltalk at: #MorphWorldView) allInstancesDo: [:pv | pv topView controller closeAndUnscheduleNoTerminate].
	Smalltalk at: #Wonderland ifPresent:[:cls| cls removeActorPrototypesFromSystem].
	subs := OrderedCollection new.
	(Smalltalk at: #Player)
		allSubclassesWithLevelDo: [:c :i | subs addFirst: c]
		startingLevel: 0.
	subs do: [:c | c removeFromSystem].


	(Smalltalk at: #MorphicModel) removeUninstantiatedModels.
	Utilities classPool at: #ScrapsBook put: nil.
	Utilities zapUpdateDownloader.
	ProjectHistory currentHistory initialize.
	Project rebuildAllProjects.
	self majorShrinkClassesToZap do: [:eachClassName |
		self removeClassNamed: eachClassName].

	Smalltalk discardVMConstruction.  "755k"
	Smalltalk discardSoundSynthesis.  "544k"
	Smalltalk discardOddsAndEnds.  "227k"
	Smalltalk discardNetworking.  "234k"
	Smalltalk discard3D.  "407k"
	Smalltalk discardFFI.  "33k"
	Smalltalk discardGenie.
	Smalltalk discardSUnit.
	Smalltalk discardMorphic.  "1372k"
	Symbol rehash.  "40k"
	"Above by itself saves about 4,238k"

	"Remove references to a few classes to be deleted, so that they won't leave obsolete versions around."
	
	ScreenController removeSelector: #viewImageImports.

	"Now delete various other classes.."
	SystemOrganization removeSystemCategory: 'Graphics-Files'.
	"SystemOrganization removeSystemCategory: 'System-Object Storage'."
	Smalltalk removeClassNamed: #Environment.
	Smalltalk removeClassNamed: #Component1.

	Smalltalk removeClassNamed: #FormSetFont.
	Smalltalk removeClassNamed: #FontSet.
	Smalltalk removeClassNamed: #InstructionPrinter.
	Smalltalk removeClassNamed: #EmphasizedMenu.
	Smalltalk removeClassNamed: #MessageTally.

	StringHolder class removeSelector: #originalWorkspaceContents.
	CompiledMethod removeSelector: #symbolic.

	RemoteString removeSelector: #makeNewTextAttVersion.
	Utilities class removeSelector: #absorbUpdatesFromServer.
	Smalltalk removeClassNamed: #PenPointRecorder.
	Smalltalk removeClassNamed: #Path.
	Smalltalk removeClassNamed: #Base64MimeConverter.
	"Smalltalk removeClassNamed: #EToySystem. Dont bother - its very small and used for timestamps etc"
	Smalltalk removeClassNamed: #WordNet.

	ListParagraph initialize.
	PopUpMenu initialize.
	StandardSystemView initialize.
     DataStream initialize.
	Smalltalk noChanges.
	ChangeSorter classPool at: #AllChangeSets 
		put: (OrderedCollection with: Smalltalk changes).

"	SystemDictionary removeSelector: #majorShrink. "

	classesToLeave := Smalltalk majorShrinkClassesToLeave.
	[Smalltalk removeAllUnSentMessages > 0] whileTrue: [
		Smalltalk unusedClasses do: [:c |
			(classesToLeave includes: c)
				ifFalse: [
					Transcript cr; show: 'Removing class: ', c.
					(Smalltalk at: c) removeFromSystem]].
		Transcript cr; show: 'Done removing classes...'].
Preferences removeObsoleteInstances.
    self discardDiscards. 

     Transcript cr; show: 'Done removing unsent methods...'; cr.
	SystemOrganization removeEmptyCategories.
	Smalltalk allClassesDo: [:c | c compactOrganization].
     Transcript show: 'Class Organization compaced.'; cr.

	Smalltalk garbageCollect.

	'Rehashing method dictionaries . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: MethodDictionary instanceCount
		during: [:bar |
			oldDicts _ MethodDictionary allInstances.
			newDicts _ Array new: oldDicts size.
			oldDicts withIndexDo: [:d :index | 
				bar value: index.
				newDicts at: index put: d rehashWithoutBecome.
			].
			oldDicts elementsExchangeIdentityWith: newDicts.
		].
	oldDicts _ newDicts _ nil.
	Project rebuildAllProjects.
	Smalltalk changes initialize.

	"seems to take more than one try to gc all the weak refs in SymbolTable"

	3 timesRepeat: [
		Smalltalk garbageCollect.
		Symbol compactSymbolTable.
	].
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/3/2003 22:35'!
majorShrinkClassesToLeave

	^#(True False ReadOnlyVariableBinding SystemOrganizer ClassOrganizer View Switch Button EndOfStream ValueHolder)! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/3/2003 13:42'!
majorShrinkClassesToZap

	^#(BDFFontReader 
		InstanceBrowser 
		Lexicon 
		TarArchiveMember 
		CRLookupResult
		"PluggableDictionary"
		" PluggableSet "
		MimeConverter
		QuotedPrintableMimeConverter
		RFC2047MimeConverter
		AbstractLauncher
		AutoStart
		AcornFileDirectory
		 " AssertionFailure "
		"CachedHTTPRequest "
		CachingCodeLoader
		CodeLoader
		EToyVectorVocabulary
		EToyVocabulary
          FileList2
		FTPConnectionException
		FullVocabulary
	"	HTTPClient
		HTTPLoader
		HTTPRequest "
		HtmlFileStream
		MacFileDirectory
		MacHFSPlusFileDirectory
		PluginHTTPRequest
		SUnitNameResolver
		ScreenedVocabulary
		ScriptNameType
		SoundType
		StringType
		SymbolListType
		TestCase
		TestFailure
		TestResource
		TestResult
		TestRunner
		TestSuite
		TestViaMethodCall
		TextSqkPageLink
		TextSqkProjectLink
	"	TextURL "
		UnknownType
		Verifier
		VerifierOfProperty
		Vocabulary
		BooleanType
		ButtonPhaseType
		ColorType
		DataType
		GraphicType
		MenuType
		NumberType
		PlayerType)! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/31/2003 20:09'!
postMajorShrink
   | oldDicts  newDicts |
	SystemOrganization removeEmptyCategories.
	Smalltalk allClassesDo: [:c | c compactOrganization]. 
	Smalltalk garbageCollect.

	'Rehashing method dictionaries . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: MethodDictionary instanceCount
		during: [:bar |
			oldDicts _ MethodDictionary allInstances.
			newDicts _ Array new: oldDicts size.
			oldDicts withIndexDo: [:d :index | 
				bar value: index.
				newDicts at: index put: d rehashWithoutBecome.
			].
			oldDicts elementsExchangeIdentityWith: newDicts.
		].
	oldDicts _ newDicts _ nil.
	Project rebuildAllProjects.
	Smalltalk changes initialize.

	"seems to take more than one try to gc all the weak refs in SymbolTable"

	3 timesRepeat: [
		Smalltalk garbageCollect.
		Symbol compactSymbolTable.
	].
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/28/2003 16:56'!
preMajorShrink
	"Smalltalk preMajorShrink"

	"You should already be in an MVC project before you run this...
	This gets rid of all the other projects, and makes the current MVC one the top project."
    References removeKey: #Car ifAbsent: [];
                  removeKey: #Wheel ifAbsent: [].
	Smalltalk zapAllOtherProjects.
	ProjectHistory currentHistory initialize.
	Smalltalk garbageCollect.
	Project rebuildAllProjects.
! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/8/2003 13:53'!
presumedSentMessages   | sent |
"Smalltalk presumedSentMessages"

	"The following should be preserved for doIts, etc"
	sent _ IdentitySet new.
	#( rehashWithoutBecome compactSymbolTable rebuildAllProjects
		browseAllSelect: printSpaceAnalysis lastRemoval
		scrollBarValue: scrollBarMenuButtonPressed: 
		withSelectionFrom:  to: removeClassNamed: instVarNamed:put:
          maximumSize: redButtonMenu:redButtonMessages:
		dragon: hilberts: mandala: web test3 factorial tinyBenchmarks benchFib
		newDepth: restoreAfter: forgetDoIts zapAllMethods obsoleteClasses
		removeAllUnSentMessages abandonSources removeUnreferencedKeys
		reclaimDependents zapOrganization condenseChanges browseObsoleteReferences
		subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
    " subclass creation: "
		variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		variableWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		weakSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		methodsFor:stamp: methodsFor:stamp:prior: instanceVariableNames:
		startTimerInterruptWatcher unusedClasses
		widthOfString: widthOfString:from:to: clipHeight clipHeight: 
		clipWidth clipWidth: clipX clipX: clipY clipY: 
          newDay:monthIndex:year: abandon pixelForValueAt:put:
          magnifyBy: replaceColor:withColor: dotOfSize:  "DisplayScreen> " fullBoundingBox
          originFromUser:
	" StrikeFont> "
		maxAscii minAscii maxWidth firstIndent:
          restIndent: rightIndent: newFontArray: fontArray:
          readStrikeFont2Family:  familySizeFace writeAsStrike2Named:
	"  Paragraph  "
		compositionRectangle: 
   " Form: these methods are meeded for gif creation "
		colorReduced colorsUsed
   " protocol from Pen class: "
		inkBrush simplePressurePen testMouseTracking testTabletTracking
   " two methods that are hidden from the method tracer "
		method: preamble:
   " convenience methods in True and False: "
		and:and: and:and:and: and:and:and:and:
		or:or: or:or:or: or:or:or:or:
  " mics "
     	pairsDo: font:  
  " ExternalEvent class: "
		registerClient:
     ) do:
		[:sel | sent add: sel].
	"The following may be sent by perform: in dispatchOnChar..."
	(ParagraphEditor classPool at: #CmdActions) asSet do:
		[:sel | sent add: sel].
	(ParagraphEditor classPool at: #ShiftCmdActions) asSet do:
		[:sel | sent add: sel].
 Smalltalk majorShrinkClassesToLeave do:
  [:symbol | sent addAll: (Smalltalk at: symbol) selectors;
                 addAll: (Smalltalk at: symbol) class selectors.].
	^ sent! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 10/28/2003 22:11'!
removeAllUnSentMessages   "Smalltalk removeAllUnSentMessages"
	"[Smalltalk unusedClasses do: [:c | (Smalltalk at: c) removeFromSystem].
		Smalltalk removeAllUnSentMessages > 0] whileTrue." 

	"Remove all implementations of unsent messages."
	| sels n |
	sels _ self allUnSentMessages.

	"The following should be preserved for doIts, etc"
	#(
		rehashWithoutBecome compactSymbolTable rebuildAllProjects	"needed even after #majorShrink is pulled"
			
		browseAllSelect: printSpaceAnalysis lastRemoval
		scrollBarValue: scrollBarMenuButtonPressed: 
		withSelectionFrom:  to: removeClassNamed:
		dragon: hilberts: mandala: web test3 factorial tinyBenchmarks benchFib
		newDepth: restoreAfter: forgetDoIts zapAllMethods obsoleteClasses
		removeAllUnSentMessages condenseChanges condenseSources abandonSources removeUnreferencedKeys postMajorShrink offerShiftedClassListMenu offerUnshiftedClassListMenu
         font1 font2 font3 font4
		reclaimDependents zapOrganization condenseChanges browseObsoleteReferences
		subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:
		methodsFor:stamp: methodsFor:stamp:prior: instanceVariableNames:
		startTimerInterruptWatcher unusedClasses) do:
		[:sel | sels remove: sel ifAbsent: []].
	"The following may be sent by perform: in dispatchOnChar..."
	(ParagraphEditor classPool at: #CmdActions) asSet do:
		[:sel | sels remove: sel ifAbsent: []].
	(ParagraphEditor classPool at: #ShiftCmdActions) asSet do:
		[:sel | sels remove: sel ifAbsent: []].
	sels size = 0 ifTrue: [^ 0].

	n _ 0. Smalltalk allBehaviorsDo: [:x | n _ n+1].
	'Removing ', sels size printString , ' messages . . .'
		displayProgressAt: Sensor cursorPoint
		from: 0 to: n
		during:
		[:bar |
		n _ 0.
		self allBehaviorsDo:
			[:class | bar value: (n _ n+1).
			sels do:
				[:sel | class removeSelectorSimply: sel]]].
	^ sels size! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'BG 11/1/2003 11:59'!
zapAllOtherProjects 
	"Smalltalk zapAllOtherProjects"
"Note: as of this writing, the only reliable way to get rid of all but the current project is te execute the following, one line at a time...
		Smalltalk zapAllOtherProjects.
		ProjectHistory currentHistory initialize.
		Smalltalk garbageCollect.
		Project rebuildAllProjects.
"

	
	Project allInstancesDo: [:p | p setParent: nil].
	Project current setParent: Project current.
	Project current isMorphic ifTrue: [ScheduledControllers _ nil].
	(Smalltalk includesKey: #TheWorldMenu)
        ifTrue: [(Smalltalk at: #TheWorldMenu) allInstancesDo: [:m | 1 to: m class instSize do: [:i | m instVarAt: i put: nil]]].
	ChangeSorter classPool at: #AllChangeSets put: nil.
	Project classPool at: #AllProjects put: nil.
	ProjectHistory currentHistory initialize.
	ChangeSorter initialize.
	Project rebuildAllProjects.  "Does a GC"
	Project allProjects size > 1 ifTrue: [Project allProjects inspect]! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'BG 10/29/2003 09:55'!
snapshot: save andQuit: quit embedded: embeddedFlag
	"Mark the changes file and close all files. If save is true, save the current state of this Smalltalk in the image file. If quit is true, then exit to the outer shell. The latter part of this method runs when resuming a previously saved image. The resume logic checks for a document file to process when starting up."

	| resuming msg sourceLink |
	Object flushDependents.
	Object flushEvents.
	save & (SourceFiles at: 2) notNil ifTrue:
		[msg _  (quit
			ifTrue: ['----QUIT----']
			ifFalse: ['----SNAPSHOT----'])
			, Date dateAndTimeNow printString, ' ', (FileDirectory default localNameFor: self imageName).
		sourceLink _ ' priorSource: ' , LastQuitLogPosition printString.
		self assureStartupStampLogged.
		LastQuitLogPosition _ (SourceFiles at: 2) setToEnd; position.
		self logChange: msg , sourceLink.
		Transcript cr; show: msg].

	self processShutDownList: quit.
	Cursor write show.
	save ifTrue: [resuming _ embeddedFlag 
					ifTrue: [self snapshotEmbeddedPrimitive] 
					ifFalse: [self snapshotPrimitive].  "<-- PC frozen here on image file"
				resuming == false "guard against failure" ifTrue:
					["Time to reclaim segment files is immediately after a save"
					Smalltalk at: #ImageSegment
						ifPresent: [:theClass | theClass reclaimObsoleteSegmentFiles]]]
		ifFalse: [resuming _ false].
	quit & (resuming == false) ifTrue: [self quitPrimitive].
	Cursor normal show.
	self setGCParameters.
	resuming == true ifTrue: [self clearExternalObjects].
	self processStartUpList: resuming == true.
	resuming == true ifTrue:[
		self setPlatformPreferences.
		self readDocumentFile].
	"Now it's time to raise an error"
	resuming == nil ifTrue: [self error:'Failed to write image file (disk full?)'].
	^ resuming! !


!SystemDictionary class methodsFor: 'initialization' stamp: 'BG 10/30/2003 21:34'!
initialize
	"SystemDictionary initialize"

	| oldList |
	oldList _ StartUpList.
	StartUpList _ OrderedCollection new.
	"These get processed from the top down..."
	#(
		DisplayScreen
		Cursor
		InputSensor
		ProcessorScheduler  "Starts low space watcher and bkground."
		Delay
		FileDirectory  "Enables file stack dump and opens sources."
		ShortIntegerArray
		ShortRunArray
		CrLfFileStream
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToStartUpList: cls].
	].
	oldList ifNotNil: [oldList do: [:className | Smalltalk at: className
						ifPresent: [:theClass | Smalltalk addToStartUpList: theClass]]].
	#(
		ImageSegment
		ControlManager
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToStartUpList: cls].
	].
		

	oldList _ ShutDownList.
	ShutDownList _ OrderedCollection new.
	"These get processed from the bottom up..."
	#(
		DisplayScreen
		Form
		ControlManager
		StrikeFont
		Color
		FileDirectory
		Delay
		HttpUrl
		Password
		PWS
		MailDB
		ImageSegment
	) do:[:clsName|
		Smalltalk at: clsName ifPresent:[:cls| Smalltalk addToShutDownList: cls].
	].

	oldList ifNotNil: [oldList reverseDo: [:className | Smalltalk at: className
						ifPresent: [:theClass | Smalltalk addToShutDownList: theClass]]].
! !


!TextURL methodsFor: 'as yet unclassified' stamp: 'BG 10/29/2003 18:53'!
actOnClickFor: anObject
	"Do what you can with this URL.  Later a web browser."

  " do nothing for now "! !


!Thumbnail methodsFor: 'release' stamp: 'BG 11/1/2003 20:44'!
nilInstanceVariables

   owner := nil.
   submorphs := nil.! !


!TimeProfileBrowser methodsFor: 'private' stamp: 'BG 10/29/2003 10:19'!
runBlock

    ^0 "
	| stream list result |
	tally := MessageTally new.
	result _ tally spyEvery: 16 on: block.
	stream := ReadWriteStream with: (String streamContents: [:s | tally report: s; close]).
	stream reset.
	list := OrderedCollection new.
	[stream atEnd] whileFalse: [list add: stream nextLine].
	self initializeMessageList: list.
	self changed: #messageList.
	self changed: #messageListIndex.
	^ result "
! !


!TranscriptStream methodsFor: 'initialization' stamp: 'BG 10/28/2003 20:42'!
closeAllViews  "Transcript closeAllViews"

	self dependents do:
		[:d |
		(d isKindOf: PluggableTextView) ifTrue: [d topView controller closeAndUnscheduleNoTerminate].
		]! !

!TranscriptStream methodsFor: 'initialization' stamp: 'BG 10/29/2003 08:05'!
open
	| openCount |
	openCount _ 0.
	self dependents do:
		[:d | ((d isKindOf: PluggableTextView) ) ifTrue: [openCount _ openCount + 1]].
	openCount = 0
		ifTrue: [self openLabel: 'Transcript']
		ifFalse: [self openLabel: 'Transcript #' , (openCount+1) printString]! !


!Utilities class methodsFor: 'fetching updates' stamp: 'BG 10/29/2003 09:00'!
serverUrls 
	"Return the current list of server URLs.  For code updates.  Format of UpdateUrlLists is 
#( ('squeak updates' ('url1' 'url2'))
    ('some other updates' ('url3' 'url4')))"

	| list |
	list _ UpdateUrlLists first last.

	"If there is a dead server, return a copy with that server last" 
	list clone withIndexDo: [:aName :ind |
	(aName beginsWith: '') ifTrue: [
		list _ list asOrderedCollection.	"and it's a copy"
		list removeAt: ind.
		list addLast: aName]].

	^ list asArray! !

!Utilities class methodsFor: 'user interface' stamp: 'BG 10/29/2003 09:40'!
informUserDuring: aBlock
	"Display a message above (or below if insufficient room) the cursor during execution of the given block."
	"Utilities informUserDuring:[:bar|
		#(one two three) do:[:info|
			bar value: info.
			(Delay forSeconds: 1) wait]]"
	Smalltalk isMorphic
		ifTrue:
			[
			^ self].
	aBlock value:[:string| Transcript cr; show: string]! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'BG 10/29/2003 08:13'!
lookUpDefinition
	| aWord aDefinition |
	(aWord _ FillInTheBlank request: 'Enter a word:') isEmpty ifTrue: [^ self].
"	(aDefinition _ WordNet definitionsFor: aWord) ifNil: [^ self]. "
     aDefinition := ''.
	(StringHolder new contents: aDefinition)
		openLabel: aWord

"Utilities lookUpDefinition"! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'BG 10/28/2003 20:42'!
storeTextWindowContentsToFileNamed: aName
	"Utilities storeTextWindowContentsToFileNamed: 'TextWindows'"
	| windows aDict aRefStream textToUse aTextView |

	"there is a reference to World, but this method seems to be unused"


	aDict _ Dictionary new.
	Smalltalk isMorphic
		ifTrue:
			[]
		ifFalse:
			[windows _ ScheduledControllers controllersSatisfying:
				[:c | (c model isMemberOf: StringHolder) or: [c model isKindOf: Workspace]].
			windows do:
				[:aController | 
					aTextView _ aController view subViews detect: [:m | m isKindOf: PluggableTextView] ifNone: [nil].
					textToUse _ aTextView
						ifNil:		[aController model contents]
						ifNotNil:	[aTextView controller text].  "The latest edits, whether accepted or not"
					aDict at: aController view label put: textToUse]].

	aDict size = 0 ifTrue: [^ self inform: 'no windows found to export.'].

	aRefStream _ ReferenceStream fileNamed: aName.
	aRefStream nextPut: aDict.
	aRefStream close.
	self inform: 'Done!!  ', aDict size printString, ' window(s) exported.'! !

!Utilities class methodsFor: 'common requests' stamp: 'BG 10/28/2003 20:42'!
closeAllDebuggers
	"Utilities closeAllDebuggers"
	Smalltalk isMorphic
	ifTrue:
		[]
	ifFalse:
		[(StandardSystemController allInstances select: [:w | w model isKindOf: Debugger])
			do: [:w | w closeAndUnscheduleNoTerminate]]! !

!Utilities class methodsFor: 'common requests' stamp: 'BG 10/29/2003 08:38'!
offerCommonRequests
	"Offer up the common-requests menu.  If the user chooses one, then evaluate it, and -- provided the value is a number or string -- show it in the Transcript."

	"Utilities offerCommonRequests"

	| reply result aMenu index normalItemCount strings |

	(CommonRequestStrings == nil or: [CommonRequestStrings isKindOf: Array])
		ifTrue:
			[self initializeCommonRequestStrings].
	strings _ CommonRequestStrings contents.
	normalItemCount _ strings asString lineCount.
	aMenu _ PopUpMenu labels: (strings asString, '
edit this menu') lines: (Array with: normalItemCount).

	index _ aMenu startUp.
	index == 0 ifTrue: [^ self].
	reply _ aMenu labelString lineNumber: index.
	reply size == 0 ifTrue: [^ self].
	index > normalItemCount ifTrue:
		[^ self editCommonRequestStrings].

	result _ self evaluate: reply in: nil to: nil.
	(result isKindOf: Number) | (result isKindOf: String)
		ifTrue:
			[Transcript cr; nextPutAll: result printString]! !

!Utilities class methodsFor: 'recent method submissions' stamp: 'BG 11/1/2003 18:33'!
recentSubmissionsWindow
	"Answer a SystemWindow holding recent submissions"

	| recentMessages messageSet |
	recentMessages _ RecentSubmissions copy reversed.
	messageSet _ RecentMessageSet messageList: recentMessages.
	messageSet autoSelectString: nil.
"	^ (messageSet inMorphicWindowLabeled: 'Recent submissions -- youngest first') applyModelExtent "

	"Utilities recentSubmissionsWindow openInHand"

! !

!Utilities class methodsFor: 'scraps' stamp: 'BG 10/29/2003 08:07'!
scrapsBook
	
	ScrapsBook ifNil:
		[].
	^ ScrapsBook

	"Utilities emptyScrapsBook"
! !


!VersionsBrowser methodsFor: 'menu' stamp: 'BG 10/28/2003 20:49'!
versionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"

	^ aMenu addList: #(

		('compare to current'		compareToCurrentVersion		'compare selected version to the current version')
		('revert to selected version'	fileInSelections					'resubmit the selected version, so that it becomes the current version')
		('remove from changes'		removeMethodFromChanges		'remove this method from the current change set, if present')
		('edit current method (O)'	openSingleMessageBrowser		'open a single-message browser on the current version of this method')		
		('find original change set'		findOriginalChangeSet		'locate the changeset which originally contained this version')
		-
		('toggle diffing (D)'			toggleDiffing					'toggle whether or not diffs should be shown here')
		('update list'				reformulateList					'reformulate the list of versions, in case it somehow got out of synch with reality')
		-
		('senders (n)'				browseSenders					'browse all senders of this selector')
		('implementors (m)'			browseImplementors				'browse all implementors of this selector')
		-
		('help...'					offerVersionsHelp				'provide an explanation of the use of this tool'))
! !


!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'BG 10/28/2003 20:48'!
versionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"

	
	^ aMenu addList: #(

		('compare to current'		compareToCurrentVersion		'compare selected version to the current version')
		('revert to selected version'	fileInSelections					'resubmit the selected version, so that it becomes the current version')
		('remove from changes'		removeMethodFromChanges		'remove this method from the current change set, if present')
		('edit current method (O)'	openSingleMessageBrowser		'open a single-message browser on the current version of this method')		
		-
		('toggle diffing (D)'			toggleDiffing					'toggle whether or not diffs should be shown here')
		('update list'				reformulateList					'reformulate the list of versions, in case it somehow got out of synch with reality')
		-
		('help...'					offerVersionsHelp				'provide an explanation of the use of this tool'))
! !


!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'BG 11/1/2003 18:37'!
resetVocabulary
  " Vocabulary resetVocabulary"
	"Answer a list of the currently-defined vocabularies in my AllStandardVocabularies list"

	AllStandardVocabularies _ IdentityDictionary new.
	! !


!Workspace methodsFor: 'as yet unclassified' stamp: 'BG 11/1/2003 22:13'!
saveContentsInFile
	"A bit of a hack to pass along this message to the controller or morph.  (Possibly this Workspace menu item could be deleted, since it's now in the text menu.)"
	| textView |


	textView _ self dependents detect: [:dep | dep isKindOf: PluggableTextView] ifNone: [nil].
	textView notNil ifTrue: [^ textView controller saveContentsInFile].
! !

Workspace class removeSelector: #prototypicalToolWindow!
Workspace removeSelector: #acceptDroppedMorphsWording!
Workspace removeSelector: #acceptDroppingMorph:event:inMorph:!
Workspace removeSelector: #acceptsDroppingMorphForReference!
Workspace removeSelector: #acceptsDroppingMorphForReference:!
Workspace removeSelector: #toggleDroppingMorphForReference!
Workspace removeSelector: #wantsDroppedMorph:event:inMorph:!
ClassCommentVersionsBrowser class removeSelector: #windowColorSpecification!
VersionsBrowser class removeSelector: #windowColorSpecification!
Utilities class removeSelector: #addSampleWindowsTo:!
Utilities class removeSelector: #fontMenuForStyle:target:selector:!
Utilities class removeSelector: #globalFlapTabOrDummy:!
Utilities class removeSelector: #newUpdatesOn:special:throughNumber:!
Utilities class removeSelector: #offerCommonRequestsInMorphic!
Utilities class removeSelector: #promptForFont:andSendTo:withSelector:!
Utilities class removeSelector: #readServer:special:updatesThrough:saveLocally:updateImage:!
Utilities class removeSelector: #readServerUpdatesSaveLocally:updateImage:!
Utilities class removeSelector: #readServerUpdatesThrough:saveLocally:updateImage:!
Utilities class removeSelector: #retrieveUrls:ontoQueue:withWaitSema:!
Utilities class removeSelector: #updateFromServer!
UndefinedObject removeSelector: #newTileMorphRepresentative!
TranscriptStream class removeSelector: #openMorphicTranscript!
TranscriptStream class removeSelector: #windowColorSpecification!
TranscriptStream removeSelector: #openAsMorph!
TranscriptStream removeSelector: #openAsMorphLabel:!
SystemVersion class removeSelector: #check:andRequestPluginUpdate:!
SystemDictionary initialize!
SyntaxError class removeSelector: #buildMorphicViewOn:!
StrokeSimplifier class removeSelector: #flattenExample!
SelectorBrowser class removeSelector: #prototypicalToolWindow!
SelectorBrowser class removeSelector: #windowColorSpecification!
PackagePaneBrowser class removeSelector: #prototypicalToolWindow!
PackagePaneBrowser class removeSelector: #windowColorSpecification!
MessageNames class removeSelector: #methodBrowserSearchingFor:!
MessageNames class removeSelector: #openMessageNames!
Lexicon class removeSelector: #windowColorSpecification!
MessageSet class removeSelector: #openAsMorph:name:!
MessageSet class removeSelector: #openAsMorph:name:inWorld:!
MessageSet class removeSelector: #windowColorSpecification!
InspectorBrowser class removeSelector: #openAsMorphOn:!
Inspector class removeSelector: #openAsMorphOn:!
Inspector class removeSelector: #openAsMorphOn:withEvalPane:withLabel:valueViewClass:!
Inspector class removeSelector: #openAsMorphOn:withLabel:!
FillInTheBlank class removeSelector: #fillInTheBlankMorphClass!
FileList2 class removeSelector: #modalFolderSelectorForProject:!
FileList class removeSelector: #addButtonsAndFileListPanesTo:at:plus:forFileList:!
FileList class removeSelector: #addVolumesAndPatternPanesTo:at:plus:forFileList:!
FileList class removeSelector: #openAsMorph!
FileList class removeSelector: #openMorphOn:editString:!
FileList class removeSelector: #prototypicalToolWindow!
FileList class removeSelector: #windowColorSpecification!
FileContentsBrowser class removeSelector: #windowColorSpecification!
Debugger class removeSelector: #windowColorSpecification!
ChangeSorter class removeSelector: #prototypicalToolWindow!
ChangeSorter class removeSelector: #windowColorSpecification!
ChangeList class removeSelector: #openAsMorph:name:multiSelect:!
ChangeList class removeSelector: #windowColorSpecification!
Browser class removeSelector: #prototypicalToolWindow!
Browser class removeSelector: #windowColorSpecification!
StringHolder class removeSelector: #windowColorSpecification!
SelectorBrowser removeSelector: #morphicWindow!
SelectorBrowser removeSelector: #openAsMorph!
PluggableFileList removeSelector: #openAsMorphLabel:inWorld:!
PackagePaneBrowser removeSelector: #openAsMorphEditing:!
MessageNames removeSelector: #inMorphicWindowLabeled:!
MessageNames removeSelector: #inMorphicWindowWithInitialSearchString:!
InstanceBrowser removeSelector: #openOnObject:inWorld:showingSelector:!
Lexicon removeSelector: #openOnClass:inWorld:showingSelector:!
MessageSet removeSelector: #inMorphicWindowLabeled:!
Inspector removeSelector: #exploreSelection!
FileList removeSelector: #askServerInfo!
FileList removeSelector: #putUpdate:!
FileContentsBrowser removeSelector: #openAsMorph!
Debugger removeSelector: #buildMorphicNotifierLabelled:message:!
Debugger removeSelector: #openFullMorphicLabel:!
ChangeSetBrowser removeSelector: #openAsMorphIn:rect:!
ChangeSorter removeSelector: #morphicWindow!
ChangeSorter removeSelector: #openAsMorph!
ChangeSorter removeSelector: #openAsMorphIn:rect:!
ChangeList removeSelector: #buildMorphicCodePaneWith:!
ChangeList removeSelector: #openAsMorphName:multiSelect:!
Browser removeSelector: #acceptDroppingMorph:event:inMorph:!
Browser removeSelector: #addMorphicSwitchesTo:at:!
Browser removeSelector: #buildMorphicClassList!
Browser removeSelector: #buildMorphicMessageCatList!
Browser removeSelector: #buildMorphicMessageList!
Browser removeSelector: #buildMorphicSwitches!
Browser removeSelector: #buildMorphicSystemCatList!
Browser removeSelector: #codeTextMorph!
Browser removeSelector: #openAsMorphClassEditing:!
Browser removeSelector: #openAsMorphEditing:!
Browser removeSelector: #openAsMorphMessageEditing:!
Browser removeSelector: #openAsMorphMsgCatEditing:!
Browser removeSelector: #openAsMorphSysCatEditing:!
CodeHolder removeSelector: #buildMorphicCodePaneWith:!
CodeHolder removeSelector: #installTextualCodingPane!
StringHolder removeSelector: #openAsMorphLabel:!
StringHolder removeSelector: #openAsMorphLabel:inWorld:!
String removeSelector: #asMorph!
String removeSelector: #asUrl!
String removeSelector: #decodeMimeHeader!
String removeSelector: #decodeQuotedPrintable!
String removeSelector: #newTileMorphRepresentative!
String removeSelector: #openAsMorph!
StrikeFont class removeSelector: #fromUser!
StrikeFont class removeSelector: #fromUser:!
StarSqueakMorph removeSelector: #display!
StandardScriptingSystem removeSelector: #spaceReclaimed!
StandardFileStream removeSelector: #asHtml!
ScreenController removeSelector: #absorbUpdatesFromServer!
ScreenController removeSelector: #beIsolated!
ScreenController removeSelector: #durableChangesMenu!
ScreenController removeSelector: #durableHelpMenu!
ScreenController removeSelector: #durableOpenMenu!
ScreenController removeSelector: #durableScreenMenu!
ScreenController removeSelector: #durableWindowMenu!
ScreenController removeSelector: #openMorphicProject!
ScreenController removeSelector: #openMorphicWorld!
ScreenController removeSelector: #propagateChanges!
ScreenController removeSelector: #standardGraphicsLibrary!
ScreenController removeSelector: #windowSpecificationPanel!
ResourceManager class removeSelector: #cacheResource:stream:!
ResourceManager removeSelector: #loaderProcess!
ResourceManager removeSelector: #startDownload!
ResourceCollector removeSelector: #writeResourceForm:fromLocator:!
Rectangle removeSelector: #encodeForRemoteCanvas!
Project class removeSelector: #fromUrl:!
Project class removeSelector: #newMorphic!
Project class removeSelector: #openBlankProjectNamed:!
Project removeSelector: #assureFlapIntegrity!
Project removeSelector: #cleanseDisabledGlobalFlapIDsList!
Project removeSelector: #createViewIfAppropriate!
Project removeSelector: #decideAboutCreatingBlank:!
Project removeSelector: #enableDisableGlobalFlap:!
Project removeSelector: #exportSegmentFileName:directory:!
Project removeSelector: #exportSegmentWithChangeSet:fileName:directory:!
Project removeSelector: #finalEnterActions!
Project removeSelector: #fromMyServerLoad:!
Project removeSelector: #globalFlapEnabledString:!
Project removeSelector: #globalFlapWithIDEnabledString:!
Project removeSelector: #initMorphic!
Project removeSelector: #navigatorFlapVisible!
Project removeSelector: #startResourceLoading!
Project removeSelector: #storeOnServer!
Project removeSelector: #storeOnServerAssumingNameValid!
Project removeSelector: #storeOnServerInnards!
Project removeSelector: #storeOnServerWithProgressInfo!
Project removeSelector: #validateProjectNameIfOK:!
Project removeSelector: #writeFileNamed:fromDirectory:toServer:!
ProcessBrowser class removeSelector: #prototypicalToolWindow!
ProcessBrowser removeSelector: #openAsMorph!
PreferencesPanel removeSelector: #findPreferencesMatching:!
Preferences class removeSelector: #chooseFlapsFont!
Preferences class removeSelector: #initializePreferencePanel:in:!
Preferences class removeSelector: #openFactoredPanel!
Preferences class removeSelector: #openFactoredPanelWithWidth:!
Preferences class removeSelector: #openNewPreferencesPanel!
Preferences class removeSelector: #openPreferencesControlPanel!
Preferences class removeSelector: #preferencesControlPanel!
Preferences class removeSelector: #setFlapsFontTo:!
Preferences class removeSelector: #setWindowColorFor:to:!
Preferences class removeSelector: #windowSpecificationPanel!
Preference removeSelector: #offerPreferenceNameMenu:with:in:!
Preference removeSelector: #representativeButtonWithColor:inPanel:!
Preference removeSelector: #tearOffButton!
Preference removeSelector: #toggleProjectLocalness!
PointerFinder removeSelector: #menu:shifted:!
PointerFinder removeSelector: #open!
Pen removeSelector: #arrowHead!
Pen removeSelector: #arrowHeadForArrowSpec:!
Pen removeSelector: #arrowHeadFrom:to:arrowSpec:!
Pen removeSelector: #arrowHeadFrom:to:forPlayer:!
PasteUpMorph removeSelector: #playfieldOptionsMenu!
ParagraphEditor removeSelector: #exploreIt!
ParagraphEditor removeSelector: #exploreIt:!
ParagraphEditor removeSelector: #makeProjectLink:!
ParagraphEditor removeSelector: #selectionAsTiles!
Number removeSelector: #newTileMorphRepresentative!
MultiCanvas removeSelector: #apply:!
MultiCanvas removeSelector: #contentsOfArea:into:!
MPEGDisplayMorph removeSelector: #drawScaledOn:!
CategoryViewer removeSelector: #makeSetter:event:from:!
Morph removeSelector: #addPaintingItemsTo:hand:!
Morph removeSelector: #asWearableCostume!
Morph removeSelector: #buildMetaMenu:!
Morph removeSelector: #chooseNewGraphicCoexisting:!
Morph removeSelector: #updateAllFromResources!
Morph removeSelector: #updateFromResource!
DualChangeSorter class removeSelector: #prototypicalToolWindow!
DualChangeSorter class removeSelector: #windowColorSpecification!
DualChangeSorter removeSelector: #morphicWindow!
DualChangeSorter removeSelector: #openAsMorph!
CPUWatcher removeSelector: #openMorphicWindowForSuspendedProcess:!
MimeConverter class removeSelector: #forEncoding:!
MailMessage removeSelector: #canonicalFields!
MailMessage removeSelector: #decoderClass!
InfiniteForm removeSelector: #encodeForRemoteCanvas!
HTTPLoader class removeSelector: #httpRequestClass!
GesturalVoice removeSelector: #newHead!
FormCanvas class removeSelector: #test1!
FormCanvas class removeSelector: #test2!
FormCanvas class removeSelector: #test3!
ColorForm removeSelector: #encodeForRemoteCanvas!
Form removeSelector: #stencil!
FileDirectory removeSelector: #eToyBaseFolderSpec!
FileDirectory removeSelector: #eToyUserList!
FileDirectory removeSelector: #eToyUserListUrl!
FileDirectory removeSelector: #eToyUserName:!
FileDirectory removeSelector: #hasEToyUserList!
DigitalSignatureAlgorithm removeSelector: #randomBitsFromSoundInput:!
DeepCopier removeSelector: #mapUniClasses!
DataStream initialize!
Color removeSelector: #newTileMorphRepresentative!
ChangeSet removeSelector: #buildMessageForMailOutWithUser:!
ChangeSet removeSelector: #mailOut!
CanvasDecoder removeSelector: #extentDepth:!
CanvasDecoder removeSelector: #initialize!
CachingCanvas removeSelector: #mainCanvas:!
Canvas removeSelector: #transform2By:clippingTo:during:smoothing:!
Boolean removeSelector: #newTileMorphRepresentative!
Object removeSelector: #couldOpenInMorphic!
Object removeSelector: #explore!
Object removeSelector: #newTileMorphRepresentative!
Smalltalk removeClassNamed: #EventManagerTestCase!
Smalltalk removeClassNamed: #EventTestCase!
Smalltalk removeClassNamed: #EyeMorph!
Smalltalk removeClassNamed: #FaceMorph!
Smalltalk removeClassNamed: #HeadMorph!
Smalltalk removeClassNamed: #LipsMorph!
Smalltalk removeClassNamed: #MidiInputMorph!
Smalltalk removeClassNamed: #ProjectLauncher!
"Postscript:
Leave the line above, and replace the rest of this comment by a useful one.
Executable statements should follow this comment, and should
be separated by periods, with no exclamation points (!!).
Be sure to put any further comments in double-quotes, like this one."

Flaps disableGlobalFlaps: false.
Preferences setParameter: #HaloSpecs to: IdentityDictionary new.
Preferences removeFlaps.

ParagraphEditor initializeTextEditorMenus.
Smalltalk removeKey: #ActiveEvent;
   removeKey: #ActiveHand;
   removeKey: #ActiveWorld;
   removeKey: #ChessConstants.

Vocabulary resetVocabulary.
ExternalDropHandler resetRegisteredHandlers.
DocLibrary resetServers.
ExternalSettings shutDown.
ExternalSettings registeredClients asArray do:
      [:item | ExternalSettings registeredClients remove: item].
 
ServerDirectory resetLocalProjectDirectories.
ServerDirectory resetServers.


	
  Flaps dropStandardFlaps.
!

